@@ -10,16 +10,16 @@
 #include "kernel/mod2.h"
 #include "kernel/structs.h"
 
-extern jmp_buf si_start_jmpbuf;
-extern short si_restart;
-extern int siRandomStart;
+EXTERN_VAR jmp_buf si_start_jmpbuf;
+EXTERN_VAR short si_restart;
+EXTERN_VAR int siRandomStart;
 void init_signals();
 
-extern BOOLEAN singular_in_batchmode;
+EXTERN_VAR BOOLEAN singular_in_batchmode;
 /* TRUE for child of a fork or started with --batch*/
 
 /* for deferring the call of m2_end() in SIGTERM handler if necessary */
-extern volatile BOOLEAN do_shutdown;
-extern volatile int defer_shutdown;
+EXTERN_VAR volatile BOOLEAN do_shutdown;
+EXTERN_VAR volatile int defer_shutdown;
 
 #endif
