@@ -28,7 +28,7 @@
   //DEBUG_BLOCK(true / false); to enable/disable debugging in this block
 # define DEBUG_BLOCK(x) bool debug_block = x;
   //standard if DEBUG_BLOCK( ); not used
-  static bool debug_block = false;
+  STATIC_VAR bool debug_block = false;
   //use like: DEBUG_PRINT(("Status %d",i))
 # define DEBUG_PRINT(x) do {if(debug_block) {Print x ;}} while(0)
 # define DEBUG_CMD(x) do {if(debug_block) {x;}} while(0)
@@ -962,7 +962,7 @@ bigintmat * minkowksi(bigintmat ** elementarray,int size_elementarray, number *
     }
 
     //char = 0
-    if ( !(nCoeff_is_Ring_Z(coef) || nCoeff_is_R(coef) || nCoeff_is_Q(coef) ||
+    if ( !(nCoeff_is_Z(coef) || nCoeff_is_R(coef) || nCoeff_is_Q(coef) ||
             nCoeff_is_long_R(coef) || nCoeff_is_long_C(coef)) ){
         WerrorS("Ground field not implemented!\n");
         return NULL;
