@@ -1,20 +1,20 @@
-#include <polys/monomials/p_polys.h>
-#include <coeffs/coeffs.h>
+#include "polys/monomials/p_polys.h"
+#include "coeffs/coeffs.h"
 
-#include <callgfanlib_conversion.h>
-#include <bbcone.h>
-#include <ppinitialReduction.h>
-#include <containsMonomial.h>
-#include <initial.h>
-#include <witness.h>
-#include <tropicalCurves.h>
-#include <tropicalStrategy.h>
-#include <startingCone.h>
-#include <groebnerFan.h>
-#include <groebnerComplex.h>
-#include <tropicalVariety.h>
+#include "callgfanlib_conversion.h"
+#include "bbcone.h"
+#include "ppinitialReduction.h"
+#include "containsMonomial.h"
+#include "initial.h"
+#include "witness.h"
+#include "tropicalCurves.h"
+#include "tropicalStrategy.h"
+#include "startingCone.h"
+#include "groebnerFan.h"
+#include "groebnerComplex.h"
+#include "tropicalVariety.h"
 
-int tropicalVerboseLevel = 0;
+VAR int tropicalVerboseLevel = 0;
 
 gfan::ZCone homogeneitySpace(ideal I, ring r)
 {
@@ -173,7 +173,7 @@ gfan::ZCone groebnerCone(const ideal I, const ring r, const gfan::ZVector &w)
       pIter(g);
       while (g!=NULL)
       {
-        pGetExpV(g,tailexpv);
+        p_GetExpV(g,tailexpv,currRing);
         tailexpw = intStar2ZVector(n, tailexpv);
         inequalities.appendRow(leadexpw-tailexpw);
         pIter(g);
@@ -193,7 +193,7 @@ gfan::ZCone groebnerCone(const ideal I, const ring r, const gfan::ZVector &w)
       pIter(g);
       while (g!=NULL)
       {
-        pGetExpV(g,tailexpv);
+        p_GetExpV(g,tailexpv,currRing);
         tailexpw = intStar2ZVector(n, tailexpv);
         equations.appendRow(leadexpw-tailexpw);
         pIter(g);
