@@ -38,7 +38,7 @@ int dim(ideal I, ring r)
     ideal vv = id_Head(I,currRing);
     if (i != -1) pDelete(&vv->m[i]);
     d = scDimInt(vv, currRing->qideal);
-    if (rField_is_Ring_Z(currRing) && (i==-1)) d++;
+    if (rField_is_Z(currRing) && (i==-1)) d++;
     idDelete(&vv);
     return d;
   }
@@ -151,7 +151,7 @@ tropicalStrategy::tropicalStrategy(const ideal I, const ring r,
   weightAdjustingAlgorithm2(nonvalued_adjustWeightUnderHomogeneity),
   extraReductionAlgorithm(noExtraReduction)
 {
-  assume(rField_is_Q(r) || rField_is_Zp(r) || rField_is_Ring_Z(r));
+  assume(rField_is_Q(r) || rField_is_Zp(r) || rField_is_Z(r));
   if (!completelyHomogeneous)
   {
     weightAdjustingAlgorithm1 = valued_adjustWeightForHomogeneity;
@@ -419,10 +419,12 @@ bool tropicalStrategy::reduce(ideal I, const ring r) const
   id_Test(I,r);
 
   nMapFunc identity = n_SetMap(startingRing->cf,r->cf);
-  number p = identity(uniformizingParameter,startingRing->cf,r->cf);
+  number p = NULL;
+  if (uniformizingParameter!=NULL)
+    p = identity(uniformizingParameter,startingRing->cf,r->cf);
   bool b = extraReductionAlgorithm(I,r,p);
   // putUniformizingBinomialInFront(I,r,p);
-  n_Delete(&p,r->cf);
+  if (p!=NULL) n_Delete(&p,r->cf);
 
   return b;
 }
