@@ -1,8 +1,8 @@
 #ifndef CALLGfANLIB_TROPICALVARIETYOFIDEALS
 #define CALLGfANLIB_TROPICALVARIETYOFIDEALS
 
-#include <polys/simpleideals.h>
-#include <tropicalStrategy.h>
+#include "polys/simpleideals.h"
+#include "tropicalStrategy.h"
 
 gfan::ZFan* tropicalVariety(const tropicalStrategy currentStrategy);
 
