@@ -1,5 +1,5 @@
 #include "kernel/mod2.h"
-#ifdef HAVE_PYTHON_MOD
+#ifdef HAVE_PYTHON
 #include <sstream>
 #include <boost/python.hpp>
 #include <boost/python/suite/indexing/vector_indexing_suite.hpp>
@@ -18,7 +18,7 @@
 #include "ring_wrap.h"
 #include "intvec_wrap.h"
 #include "poly_wrap.h"
-extern int inerror;
+EXTERN_VAR int inerror;
 using namespace boost::python;
 using boost::python::extract;
 static void free_leftv(leftv args)
