@@ -1,5 +1,5 @@
 #include "kernel/mod2.h"
-#ifdef HAVE_PYTHON_MOD
+#ifdef HAVE_PYTHON
 #include <boost/python.hpp>
 #include <boost/python/suite/indexing/vector_indexing_suite.hpp>
 #include "IIntvec.h"
@@ -34,4 +34,3 @@ boost::python::class_<Intvec>("IntVector")
      .def(boost::python::vector_indexing_suite<Intvec>());
 }
 #endif
-
