@@ -6,15 +6,11 @@
 */
 
 #include "kernel/mod2.h"
-
 #include "omalloc/omalloc.h"
 #include "misc/mylimits.h"
-
 #include "resources/feResource.h"
 #include "reporter/reporter.h"
-
 #include "resources/omFindExec.h"
-
 #include "reporter/si_signals.h"
 
 #include "ipid.h"
@@ -72,8 +68,8 @@ static BOOLEAN heGenInit(int,int);    static void heGenHelp(heEntry hentry,int);
 static BOOLEAN heDummyInit(int,int);   static void heDummyHelp(heEntry hentry,int);
 static BOOLEAN heEmacsInit(int,int);   static void heEmacsHelp(heEntry hentry,int);
 
-static heBrowser heCurrentHelpBrowser = NULL;
-static int heCurrentHelpBrowserIndex= -1;
+STATIC_VAR heBrowser heCurrentHelpBrowser = NULL;
+STATIC_VAR int heCurrentHelpBrowserIndex= -1;
 
 
 /*****************************************************************
@@ -83,7 +79,7 @@ static int heCurrentHelpBrowserIndex= -1;
  *****************************************************************/
 // order is important -- first possible help is chosen
 // moved to LIB/help.cnf
-static heBrowser_s *heHelpBrowsers=NULL;
+STATIC_VAR heBrowser_s *heHelpBrowsers=NULL;
 
 /*****************************************************************
  *
@@ -702,7 +698,7 @@ static BOOLEAN heOnlineHelp(char* s)
        ||
        ((fp=feFopen(str,"rb", libnamebuf))!=NULL)))
   {
-    extern FILE *yylpin;
+    EXTERN_VAR FILE *yylpin;
     lib_style_types lib_style; // = OLD_LIBSTYLE;
 
     yylpin = fp;
@@ -767,7 +763,7 @@ static long heKeyChksum(char* key)
  *
  *****************************************************************/
 
-static BOOLEAN feHelpCalled = FALSE;
+STATIC_VAR BOOLEAN feHelpCalled = FALSE;
 
 static void heBrowserHelp(heEntry hentry)
 {
@@ -822,7 +818,7 @@ static BOOLEAN heGenInit(int warn, int br)
     {
       case '#': break;
       case ' ': break;
-      case 'i': /* singular.hlp */
+      case 'i': /* singular.info */
       case 'x': /* singular.idx */
       case 'h': /* html dir */
                if (feResource(*p, warn) == NULL)
@@ -961,13 +957,13 @@ static void heGenHelp(heEntry hentry, int br)
                    i=strlen(sys);
                    break;
                  }
-        case 'i': /* singular.hlp */
+        case 'i': /* singular.info */
                  {
                    char *i_res=feResource('i');
                    if (i_res!=NULL) strcat(sys,i_res);
                    else
                    {
-                     WarnS("singular.hlp not found");
+                     WarnS("singular.info not found");
                      return;
                    }
                    i=strlen(sys);
