@@ -15,8 +15,7 @@
 // need global defines:
 #include "kernel/mod2.h"
 // need to include all tokens: *_CMD:
-#include "grammar.h"
-#include "tok.h"
+#include "Singular/tok.h"
 
 #ifdef HAVE_RINGS
   #define RING_MASK        4
@@ -26,16 +25,19 @@
   #define ZERODIVISOR_MASK 0
 #endif
 
-inline int RingDependend(int t) { return (BEGIN_RING<t)&&(t<END_RING); }
+static inline int RingDependend(int t) { return (BEGIN_RING<t)&&(t<END_RING); }
 
 // to produce convert_table.texi for doc:
-int produce_convert_table=0;
+VAR int produce_convert_table=0;
 
 // bits 0,1 for PLURAL
-#define NO_PLURAL        0
+#define NO_NC            0
 #define ALLOW_PLURAL     1
 #define COMM_PLURAL      2
-#define PLURAL_MASK      3
+// bit 6: non-commutative letterplace
+#define ALLOW_LP         64
+#define NC_MASK          (3+64)
+#define ALLOW_NC         ALLOW_LP|ALLOW_PLURAL
 
 // bit 2 for RING-CF
 #define ALLOW_RING       4
@@ -260,7 +262,7 @@ const char * iiTwoOps(int t)
 {
   if (t<127)
   {
-    static char ch[2];
+    STATIC_VAR char ch[2];
     switch (t)
     {
       case '&':
@@ -324,7 +326,7 @@ int iiTestConvert (int inputType, int outputType)
   // Tok2Cmdname(inputType), Tok2Cmdname(outputType));
   return 0;
 }
-char *iparith_inc;
+VAR char *iparith_inc;
 void ttGen1()
 {
   iparith_inc=strdup("iparith.xxxxxx");
@@ -347,6 +349,8 @@ void ttGen1()
   fprintf(outfile,"// - otherwise search for the first possibility\n");
   fprintf(outfile,"//   with converted types of the arguments\n");
   fprintf(outfile,"// - otherwise report an error\n//\n");
+  fprintf(outfile,"// --------------------------------------------------\n");
+  fprintf(outfile,"// depends on Singular/table.h and kernel/mod2.h\n\n");
 
   int op;
   i=0;
@@ -361,9 +365,11 @@ void ttGen1()
           Tok2Cmdname(dArith1[i].res));
     if (RingDependend(dArith1[i].res) && (!RingDependend(dArith1[i].arg)))
       fprintf(outfile," requires currRing");
-    if ((dArith1[i].valid_for & PLURAL_MASK)==2)
+    if ((dArith1[i].valid_for & NC_MASK)==2)
       fprintf(outfile,", commutative subalgebra");
-    else if ((dArith1[i].valid_for & PLURAL_MASK)==0)
+    else if ((dArith1[i].valid_for & NC_MASK)==ALLOW_LP)
+      fprintf(outfile,", letterplace rings");
+    else if ((dArith1[i].valid_for & NC_MASK)==0)
       fprintf(outfile,", only commutative rings");
     if ((dArith1[i].valid_for & RING_MASK)==0)
       fprintf(outfile,", field coeffs");
@@ -393,9 +399,9 @@ void ttGen1()
     {
       fprintf(outfile," requires currRing");
     }
-    if ((dArith2[i].valid_for & PLURAL_MASK)==2)
+    if ((dArith2[i].valid_for & NC_MASK)==COMM_PLURAL)
       fprintf(outfile,", commutative subalgebra");
-    else if ((dArith2[i].valid_for & PLURAL_MASK)==0)
+    else if ((dArith2[i].valid_for & NC_MASK)==0)
       fprintf(outfile,", only commutative rings");
     if ((dArith2[i].valid_for & RING_MASK)==0)
       fprintf(outfile,", field coeffs");
@@ -427,9 +433,9 @@ void ttGen1()
     {
       fprintf(outfile," requires currRing");
     }
-    if ((dArith3[i].valid_for & PLURAL_MASK)==2)
+    if ((dArith3[i].valid_for & NC_MASK)==COMM_PLURAL)
       fprintf(outfile,", commutative subalgebra");
-    else if ((dArith3[i].valid_for & PLURAL_MASK)==0)
+    else if ((dArith3[i].valid_for & NC_MASK)==0)
       fprintf(outfile,", only commutative rings");
     if ((dArith3[i].valid_for & RING_MASK)==0)
       fprintf(outfile,", field coeffs");
@@ -579,16 +585,13 @@ void ttGen2b()
 
   fprintf(
     outfile,
-    "#ifdef MODULE_GENERATOR\n"
-    "#define omAlloc0(A) malloc(A)\n"
-    "#endif\n"
     "void iiInitCmdName()\n{\n"
     "  sArithBase.nCmdUsed      = 0;\n"
     "  sArithBase.nCmdAllocated = %d;\n"
-    "  sArithBase.sCmds = (cmdnames*)omAlloc0(sArithBase.nCmdAllocated*sizeof(cmdnames));\n"
+    "  sArithBase.sCmds = (cmdnames*)omAlloc0(%d/*sArithBase.nCmdAllocated*/ *sizeof(cmdnames));\n"
     "\n"
     "  // name-string                   alias  tokval toktype index\n",
-    cmd_size);
+    cmd_size,cmd_size);
   int m=0;
   int id_nr=0;
 
@@ -608,6 +611,7 @@ void ttGen2b()
         case CMD_3:            fprintf(outfile,"CMD_3"); break;
         case CMD_12:           fprintf(outfile,"CMD_12"); break;
         case CMD_123 :         fprintf(outfile,"CMD_123"); break;
+        case CMD_13 :          fprintf(outfile,"CMD_13"); break;
         case CMD_23:           fprintf(outfile,"CMD_23"); break;
         case CMD_M:            fprintf(outfile,"CMD_M"); break;
         case SYSVAR:           fprintf(outfile,"SYSVAR"); break;
@@ -859,9 +863,9 @@ void ttGen4()
       {
         old_s=s;
         #ifdef HAVE_PLURAL
-        switch (dArith1[i].valid_for & PLURAL_MASK)
+        switch (dArith1[i].valid_for & NC_MASK)
         {
-          case NO_PLURAL:
+          case NO_NC:
             fprintf(outfile,"@item @ref{%s} @tab @code{---}\n",s);
             break;
           case ALLOW_PLURAL:
@@ -889,9 +893,9 @@ void ttGen4()
       {
         old_s=s;
         #ifdef HAVE_PLURAL
-        switch (dArith2[i].valid_for & PLURAL_MASK)
+        switch (dArith2[i].valid_for & NC_MASK)
         {
-          case NO_PLURAL:
+          case NO_NC:
             fprintf(outfile,"@item @ref{%s} @tab @code{---}\n",s);
             break;
           case ALLOW_PLURAL:
@@ -919,9 +923,9 @@ void ttGen4()
       {
         old_s=s;
         #ifdef HAVE_PLURAL
-        switch (dArith3[i].valid_for & PLURAL_MASK)
+        switch (dArith3[i].valid_for & NC_MASK)
         {
-          case NO_PLURAL:
+          case NO_NC:
             fprintf(outfile,"@item @ref{%s} @tab @code{---}\n",s);
             break;
           case ALLOW_PLURAL:
@@ -947,9 +951,9 @@ void ttGen4()
     {
         old_s=s;
         #ifdef HAVE_PLURAL
-        switch (dArithM[i].valid_for & PLURAL_MASK)
+        switch (dArithM[i].valid_for & NC_MASK)
         {
-          case NO_PLURAL:
+          case NO_NC:
             fprintf(outfile,"@item @ref{%s} @tab @code{---}\n",s);
             break;
           case ALLOW_PLURAL:
@@ -972,7 +976,7 @@ void ttGen4()
 }
 /*-------------------------------------------------------------------*/
 
-int main(int argc, char**)
+int main(int argc, char** argv)
 {
   if (argc>1)
   {
