@@ -12,8 +12,6 @@
 
 #include "kernel/mod2.h"
 
-#include "omalloc/omalloc.h"
-
 #include "misc/options.h"
 #include "misc/intvec.h"
 
@@ -44,43 +42,31 @@
 #include "polys/mod_raw.h"
 #endif /* HAVE_DYNAMIC_LOADING */
 
-omBin sip_command_bin = omGetSpecBin(sizeof(sip_command));
-omBin sip_package_bin = omGetSpecBin(sizeof(sip_package));
+VAR omBin sip_command_bin = omGetSpecBin(sizeof(sip_command));
+VAR omBin sip_package_bin = omGetSpecBin(sizeof(sip_package));
 //omBin ip_package_bin = omGetSpecBin(sizeof(ip_package));
-omBin idrec_bin = omGetSpecBin(sizeof(idrec));
+VAR omBin idrec_bin = omGetSpecBin(sizeof(idrec));
 
-coeffs coeffs_BIGINT;
+VAR coeffs coeffs_BIGINT;
 
-proclevel *procstack=NULL;
+VAR proclevel *procstack=NULL;
 //idhdl idroot = NULL;
 
-idhdl currPackHdl = NULL;
-idhdl basePackHdl = NULL;
-package currPack = NULL;
-package basePack = NULL;
-idhdl currRingHdl = NULL;
+VAR idhdl currPackHdl = NULL;
+VAR idhdl basePackHdl = NULL;
+VAR package currPack = NULL;
+VAR package basePack = NULL;
+VAR idhdl currRingHdl = NULL;
 
 void paCleanUp(package pack);
 
 /*0 implementation*/
 
-int iiS2I(const char *s)
+static inline long iiS2I(const char *s)
 {
-  int i;
-  i=s[0];
-  if (s[1]!='\0')
-  {
-    i=(i<<8)+s[1];
-    if (s[2]!='\0')
-    {
-      i=(i<<8)+s[2];
-      if (s[3]!='\0')
-      {
-        i=(i<<8)+s[3];
-      }
-    }
-  }
-  return i;
+  long l;
+  strncpy((char*)&l,s,SIZEOF_LONG);
+  return l;
 }
 
 idhdl idrec::get(const char * s, int level)
@@ -91,8 +77,9 @@ idhdl idrec::get(const char * s, int level)
   idhdl found=NULL;
   int l;
   const char *id_;
-  int i=iiS2I(s);
-  int less4=(i < (1<<24));
+  unsigned long i=iiS2I(s);
+  char *dummy=(char*)&i;
+  BOOLEAN less4=(dummy[SIZEOF_LONG-1]=='\0');
   while (h!=NULL)
   {
     omCheckAddr((ADDRESS)IDID(h));
@@ -102,9 +89,9 @@ idhdl idrec::get(const char * s, int level)
       if (i==h->id_i)
       {
         id_=IDID(h);
-        if (less4 || (0 == strcmp(s+4,id_+4)))
+        if (less4 || (0 == strcmp(s+SIZEOF_LONG,id_+SIZEOF_LONG)))
         {
-          if (l==level) return h;
+          if(l==level) return h;
           found=h;
         }
       }
@@ -114,6 +101,32 @@ idhdl idrec::get(const char * s, int level)
   return found;
 }
 
+idhdl idrec::get_level(const char * s, int level)
+{
+  assume(s!=NULL);
+  assume((level>=0) && (level<=1000)); //not really, but if it isnt in that bounds..
+  idhdl h = this;
+  int l;
+  const char *id_;
+  unsigned long i=iiS2I(s);
+  int less4=(i < (1L<<((SIZEOF_LONG-1)*8)));
+  while (h!=NULL)
+  {
+    omCheckAddr((ADDRESS)IDID(h));
+    l=IDLEV(h);
+    if ((l==level)&&(i==h->id_i))
+    {
+      id_=IDID(h);
+      if (less4 || (0 == strcmp(s+SIZEOF_LONG,id_+SIZEOF_LONG)))
+      {
+        return h;
+      }
+    }
+    h = IDNEXT(h);
+  }
+  return NULL;
+}
+
 //idrec::~idrec()
 //{
 //  if (id!=NULL)
@@ -138,6 +151,14 @@ void *idrecDataInit(int t)
 #endif
     case BIGINTMAT_CMD:
       return (void *)new bigintmat();
+    case BUCKET_CMD:
+      if (currRing!=NULL)
+      return (void*)sBucketCreate(currRing);
+      else
+      {
+        WerrorS("need basering for polyBucket");
+        return NULL;
+      }
     case INTVEC_CMD:
     case INTMAT_CMD:
       return (void *)new intvec();
@@ -151,6 +172,7 @@ void *idrecDataInit(int t)
     case IDEAL_CMD:
     case MODUL_CMD:
     case MATRIX_CMD:
+    case SMATRIX_CMD:
       return (void*) idInit(1,1);
     case MAP_CMD:
     {
@@ -169,8 +191,6 @@ void *idrecDataInit(int t)
     //the types with the standard init: set the struct to zero
     case LINK_CMD:
       return (void*) omAlloc0Bin(sip_link_bin);
-    case RING_CMD:
-      return NULL;
     case PACKAGE_CMD:
     {
       package pa=(package)omAlloc0Bin(sip_package_bin);
@@ -187,14 +207,15 @@ void *idrecDataInit(int t)
     }
     case RESOLUTION_CMD:
       return  (void *)omAlloc0(sizeof(ssyStrategy));
-    //other types: without init (int,script,poly,def,package)
-    case CRING_CMD:
+    //other types: without alloc. (int,script,poly,def,package,..)
     case INT_CMD:
     case DEF_CMD:
     case POLY_CMD:
     case VECTOR_CMD:
+    case RING_CMD:
+    case CRING_CMD:
     case QRING_CMD:
-       return (void*)0L;
+      return NULL;
     default:
       {
         if (t>MAX_TOK)
@@ -223,6 +244,7 @@ idhdl idrec::set(const char * s, int level, int t, BOOLEAN init)
   IDNEXT(h) = this;
   BOOLEAN at_start=(this==IDROOT);
   h->id_i=iiS2I(s);
+  if (t==BUCKET_CMD) WarnS("defining polyBucket");
   if (init)
   {
     if ((t==IDEAL_CMD)||(t==MODUL_CMD))
@@ -247,7 +269,7 @@ idhdl idrec::set(const char * s, int level, int t, BOOLEAN init)
 char * idrec::String(BOOLEAN typed)
 {
   sleftv tmp;
-  memset(&tmp,0,sizeof(sleftv));
+  tmp.Init();
   tmp.rtyp=IDTYP(this);
   tmp.data=IDDATA(this);
   tmp.name=IDID(this);
@@ -269,68 +291,77 @@ idhdl enterid(const char * s, int lev, int t, idhdl* root, BOOLEAN init, BOOLEAN
     }
   }
   // is it already defined in root ?
-  if ((h=(*root)->get(s,lev))!=NULL)
+  if ((h=(*root)->get_level(s,lev))!=NULL)
   {
-    if (IDLEV(h)==lev)
+    if ((IDTYP(h) == t)||(t==DEF_CMD))
+    {
+      if (IDTYP(h)==PACKAGE_CMD)
+      {
+        if (strcmp(s,"Top")==0)
+        {
+          goto errlabel;
+        }
+        else return h;
+      }
+      else
+      {
+        if (BVERBOSE(V_REDEFINE))
+        {
+          const char *f=VoiceName();
+          if (strcmp(f,"STDIN")==0)
+            Warn("redefining %s (%s)",s,my_yylinebuf);
+          else
+            Warn("redefining %s (%s) %s:%d",s,my_yylinebuf,f, yylineno);
+        }
+        if (s==IDID(h)) IDID(h)=NULL;
+        killhdl2(h,root,currRing);
+      }
+    }
+    else
+      goto errlabel;
+  }
+  // is it already defined in currRing->idroot ?
+  else if (search && (currRing!=NULL)&&((*root) != currRing->idroot))
+  {
+    if ((h=currRing->idroot->get_level(s,lev))!=NULL)
     {
       if ((IDTYP(h) == t)||(t==DEF_CMD))
       {
-        if (IDTYP(h)==PACKAGE_CMD)
+        if (BVERBOSE(V_REDEFINE))
         {
-          if (strcmp(s,"Top")==0)
-          {
-            goto errlabel;
-          }
-          else return h;
-        }
-        else
-        {
-          if (BVERBOSE(V_REDEFINE))
+          const char *f=VoiceName();
+          if (strcmp(f,"STDIN")==0)
             Warn("redefining %s (%s)",s,my_yylinebuf);
-          if (s==IDID(h)) IDID(h)=NULL;
-          killhdl2(h,root,currRing);
+          else
+            Warn("redefining %s (%s) %s:%d",s,my_yylinebuf,f, yylineno);
         }
+        if (s==IDID(h)) IDID(h)=NULL;
+        killhdl2(h,&currRing->idroot,currRing);
       }
       else
         goto errlabel;
     }
   }
-  // is it already defined in currRing->idroot ?
-  else if (search && (currRing!=NULL)&&((*root) != currRing->idroot))
-  {
-    if ((h=currRing->idroot->get(s,lev))!=NULL)
-    {
-      if (IDLEV(h)==lev)
-      {
-        if ((IDTYP(h) == t)||(t==DEF_CMD))
-        {
-          if (BVERBOSE(V_REDEFINE))
-            Warn("redefining %s (%s)",s,my_yylinebuf);
-          if (s==IDID(h)) IDID(h)=NULL;
-          killhdl2(h,&currRing->idroot,currRing);
-        }
-        else
-          goto errlabel;
-      }
-    }
-  }
   // is it already defined in idroot ?
   else if (search && (*root != IDROOT))
   {
-    if ((h=IDROOT->get(s,lev))!=NULL)
+    if ((h=IDROOT->get_level(s,lev))!=NULL)
     {
-      if (IDLEV(h)==lev)
+      if ((IDTYP(h) == t)||(t==DEF_CMD))
       {
-        if ((IDTYP(h) == t)||(t==DEF_CMD))
+        if (BVERBOSE(V_REDEFINE))
         {
-          if (BVERBOSE(V_REDEFINE))
+          const char *f=VoiceName();
+          if (strcmp(f,"STDIN")==0)
             Warn("redefining %s (%s)",s,my_yylinebuf);
-          if (s==IDID(h)) IDID(h)=NULL;
-          killhdl2(h,&IDROOT,NULL);
+          else
+            Warn("redefining %s (%s) %s:%d",s,my_yylinebuf,f, yylineno);
         }
-        else
-          goto errlabel;
+        if (s==IDID(h)) IDID(h)=NULL;
+        killhdl2(h,&IDROOT,NULL);
       }
+      else
+        goto errlabel;
     }
   }
   *root = (*root)->set(s, lev, t, init);
@@ -510,6 +541,56 @@ idhdl ggetid(const char *n, BOOLEAN /*local*/, idhdl *packhdl)
 }
 #endif
 
+#if 0
+// debug version
+idhdl ggetid(const char *n)
+{
+  if (currRing!=NULL)
+  {
+    idhdl h2 = currRing->idroot->get(n,myynest);
+    idhdl h = IDROOT->get(n,myynest);
+    if ((h!=NULL)&&(h2!=NULL)&&(IDLEV(h)==IDLEV(h2)))
+    {
+      Warn("SHADOW %s(%s) vs %s(%s) in %s\n",IDID(h),Tok2Cmdname(IDTYP(h)),IDID(h2),Tok2Cmdname(IDTYP(h2)),my_yylinebuf);
+    }
+    if ((h2!=NULL)&&(IDLEV(h2)==myynest)) return h2;
+    if (h!=NULL) return h;
+    if (h2!=NULL) return h2;
+  }
+  else
+  {
+    idhdl h = IDROOT->get(n,myynest);
+    if (h!=NULL) return h;
+  }
+  if (basePack!=currPack)
+    return basePack->idroot->get(n,myynest);
+  return NULL;
+}
+#endif
+#if 1
+// try currRing before non-ring stuff
+idhdl ggetid(const char *n)
+{
+  if (currRing!=NULL)
+  {
+    idhdl h2 = currRing->idroot->get(n,myynest);
+    if ((h2!=NULL)&&(IDLEV(h2)==myynest)) return h2;
+    idhdl h = IDROOT->get(n,myynest);
+    if (h!=NULL) return h;
+    if (h2!=NULL) return h2;
+  }
+  else
+  {
+    idhdl h = IDROOT->get(n,myynest);
+    if (h!=NULL) return h;
+  }
+  if (basePack!=currPack)
+    return basePack->idroot->get(n,myynest);
+  return NULL;
+}
+#endif
+#if 0
+// try non-ring stuff before ring stuff
 idhdl ggetid(const char *n)
 {
   idhdl h = IDROOT->get(n,myynest);
@@ -524,6 +605,7 @@ idhdl ggetid(const char *n)
     return basePack->idroot->get(n,myynest);
   return NULL;
 }
+#endif
 
 void ipListFlag(idhdl h)
 {
@@ -696,7 +778,6 @@ void paCleanUp(package pack)
   (pack->ref)--;
   if (pack->ref < 0)
   {
-#ifndef HAVE_STATIC
     if( pack->language == LANG_C)
     {
       Print("//dlclose(%s)\n",pack->libname);
@@ -704,7 +785,6 @@ void paCleanUp(package pack)
       dynl_close (pack->handle);
 #endif /* HAVE_DYNAMIC_LOADING */
     }
-#endif /* HAVE_STATIC */
     omFree((ADDRESS)pack->libname);
     memset((void *) pack, 0, sizeof(sip_package));
     pack->language=LANG_NONE;
@@ -797,7 +877,8 @@ BOOLEAN iiAlias(leftv p)
       case MAP_CMD:
          {
            map im = IDMAP(pp);
-           omFree((ADDRESS)im->preimage);
+           omFree((ADDRESS)im->preimage); 
+	   im->preimage=NULL;// and continue
          }
          // continue as ideal:
       case IDEAL_CMD:
