@@ -10,7 +10,6 @@
 #include "Singular/tok.h"
 #include "misc/options.h"
 #include "Singular/ipid.h"
-#include "omalloc/omalloc.h"
 #include "polys/monomials/ring.h"
 #include "Singular/subexpr.h"
 #include "Singular/ipid.h"
@@ -18,6 +17,10 @@
 #include "Singular/fevoices.h"
 #include "Singular/lists.h"
 
+#ifndef SINGULAR_PATH_LENGTH
+#define SINGULAR_PATH_LENGTH 512
+#endif
+
 #include <ctype.h>
 
 #if SIZEOF_LONG == 8
@@ -58,11 +61,11 @@ extern int iiArithAddCmd(const char *szName, short nAlias, short nTokval,
 #ifdef HAVE_LIBPARSER
 void yylprestart (FILE *input_file );
 int current_pos(int i=0);
-extern int yylp_errno;
-extern int yylplineno;
-extern char *yylp_errlist[];
+EXTERN_VAR int yylp_errno;
+EXTERN_VAR int yylplineno;
+extern const char *yylp_errlist[];
 void print_init();
-libstackv library_stack;
+VAR libstackv library_stack;
 #endif
 
 //int IsCmd(char *n, int tok);
@@ -71,7 +74,7 @@ char mytolower(char c);
 /*2
 * return TRUE if the libray libname is already loaded
 */
-BOOLEAN iiGetLibStatus(char *lib)
+BOOLEAN iiGetLibStatus(const char *lib)
 {
   idhdl hl;
 
@@ -82,7 +85,9 @@ BOOLEAN iiGetLibStatus(char *lib)
   {
     return FALSE;
   }
-  return (strcmp(lib,IDPACKAGE(hl)->libname)==0);
+  if ((IDPACKAGE(hl)->language!=LANG_C)&&(IDPACKAGE(hl)->libname!=NULL))
+    return (strcmp(lib,IDPACKAGE(hl)->libname)==0);
+  return FALSE;
 }
 
 /*2
@@ -191,7 +196,7 @@ char * iiProcArgs(char *e,BOOLEAN withParenth)
 */
 char* iiGetLibProcBuffer(procinfo *pi, int part )
 {
-  char buf[256], *s = NULL, *p;
+  char buf[SINGULAR_PATH_LENGTH], *s = NULL, *p;
   long procbuflen;
 
   FILE * fp = feFopen( pi->libname, "rb", NULL, TRUE );
@@ -290,18 +295,31 @@ char* iiGetLibProcBuffer(procinfo *pi, int part )
   return NULL;
 }
 
-BOOLEAN iiAllStart(procinfov pi, char *p,feBufferTypes t, int l)
+BOOLEAN iiAllStart(procinfov pi, const char *p, feBufferTypes t, int l)
 {
+  int save_trace=traceit;
+  int restore_traceit=0;
+  if (traceit_stop
+  && (traceit & TRACE_SHOW_LINE))
+  {
+    traceit &=(~TRACE_SHOW_LINE);
+    traceit_stop=0;
+    restore_traceit=1;
+  }
   // see below:
   BITSET save1=si_opt_1;
   BITSET save2=si_opt_2;
   newBuffer( omStrDup(p /*pi->data.s.body*/), t /*BT_proc*/,
                pi, l );
   BOOLEAN err=yyparse();
+
   if (sLastPrinted.rtyp!=0)
   {
     sLastPrinted.CleanUp();
   }
+
+  if (restore_traceit) traceit=save_trace;
+
   // the access to optionStruct and verboseStruct do not work
   // on x86_64-Linux for pic-code
   if ((TEST_V_ALLWARN) &&
@@ -383,13 +401,12 @@ BOOLEAN iiPStart(idhdl pn, leftv v)
   {
     iiCurrArgs=(leftv)omAllocBin(sleftv_bin);
     memcpy(iiCurrArgs,v,sizeof(sleftv)); // keeps track of v->next etc.
-    memset(v,0,sizeof(sleftv));
+    v->Init();
   }
   else
   {
     iiCurrArgs=NULL;
   }
-  iiCurrProc=pn;
   /* start interpreter ======================================*/
   myynest++;
   if (myynest > SI_MAX_NEST)
@@ -399,7 +416,9 @@ BOOLEAN iiPStart(idhdl pn, leftv v)
   }
   else
   {
+    iiCurrProc=pn;
     err=iiAllStart(pi,pi->data.s.body,BT_proc,pi->data.s.body_lineno-(v!=NULL));
+    iiCurrProc=NULL;
 
     if (iiLocalRing[myynest-1] != currRing)
     {
@@ -451,9 +470,9 @@ BOOLEAN iiPStart(idhdl pn, leftv v)
   return err;
 }
 
-ring    *iiLocalRing;
-sleftv  iiRETURNEXPR;
-int     iiRETURNEXPR_len=0;
+VAR ring    *iiLocalRing;
+INST_VAR sleftv  iiRETURNEXPR;
+VAR int     iiRETURNEXPR_len=0;
 
 #ifdef RDEBUG
 static void iiShowLevRings()
@@ -482,7 +501,7 @@ static void iiCheckNest()
     iiRETURNEXPR_len+=16;
   }
 }
-BOOLEAN iiMake_proc(idhdl pn, package pack, leftv sl)
+BOOLEAN iiMake_proc(idhdl pn, package pack, leftv args)
 {
   int err;
   procinfov pi = IDPROC(pn);
@@ -529,11 +548,11 @@ BOOLEAN iiMake_proc(idhdl pn, package pack, leftv sl)
                    currPackHdl=packFindHdl(currPack);
                    //Print("set pack=%s\n",IDID(currPackHdl));
                  }
-                 err=iiPStart(pn,sl);
+                 err=iiPStart(pn,args);
                  break;
     case LANG_C:
                  leftv res = (leftv)omAlloc0Bin(sleftv_bin);
-                 err = (pi->data.o.function)(res, sl);
+                 err = (pi->data.o.function)(res, args);
                  memcpy(&iiRETURNEXPR,res,sizeof(iiRETURNEXPR));
                  omFreeBin((ADDRESS)res,  sleftv_bin);
                  break;
@@ -572,17 +591,16 @@ static void iiCallLibProcBegin()
   idhdl tmp_ring=NULL;
   if (currRing!=NULL)
   {
-    if (IDRING(currRingHdl)!=currRing)
+    if ((currRingHdl!=NULL) && (IDRING(currRingHdl)!=currRing))
     {
       // clean up things depending on currRingHdl:
       sLastPrinted.CleanUp(IDRING(currRingHdl));
       sLastPrinted.Init();
-      // need to define a ring-hdl for currRingHdl
-      tmp_ring=enterid(" tmpRing",myynest,RING_CMD,&IDROOT,FALSE);
-      IDRING(tmp_ring)=currRing;
-      currRing->ref++;
-      rSetHdl(tmp_ring);
     }
+    // need to define a ring-hdl for currRingHdl
+    tmp_ring=enterid(" tmpRing",myynest,RING_CMD,&IDROOT,FALSE);
+    IDRING(tmp_ring)=rIncRefCnt(currRing);
+    rSetHdl(tmp_ring);
   }
 }
 static void iiCallLibProcEnd(idhdl save_ringhdl, ring save_ring)
@@ -590,7 +608,7 @@ static void iiCallLibProcEnd(idhdl save_ringhdl, ring save_ring)
   if ((currRing!=NULL)
   &&(currRing!=save_ring))
   {
-    currRing->ref--;
+    rDecRefCnt(currRing);
     idhdl hh=IDROOT;
     idhdl prev=NULL;
     while((hh!=currRingHdl) && (hh!=NULL)) { prev=hh; hh=hh->next; }
@@ -601,10 +619,6 @@ static void iiCallLibProcEnd(idhdl save_ringhdl, ring save_ring)
       omFree((ADDRESS)IDID(hh));
       omFreeBin((ADDRESS)hh, idrec_bin);
     }
-    else
-    {
-      WarnS("internal: lost ring in iiCallLib");
-    }
   }
   currRingHdl=save_ringhdl;
   currRing=save_ring;
@@ -642,9 +656,49 @@ void* iiCallLibProc1(const char*n, void *arg, int arg_type, BOOLEAN &err)
   }
   return NULL;
 }
-/// args: NULL terminated arry of arguments
+
+// return NULL on failure
+ideal ii_CallProcId2Id(const char *lib,const char *proc, ideal arg, const ring R)
+{
+  char *plib = iiConvName(lib);
+  idhdl h=ggetid(plib);
+  omFree(plib);
+  if (h==NULL)
+  {
+    BOOLEAN bo=iiLibCmd(lib,TRUE,TRUE,FALSE);
+    if (bo) return NULL;
+  }
+  ring oldR=currRing;
+  rChangeCurrRing(R);
+  BOOLEAN err;
+  ideal I=(ideal)iiCallLibProc1(proc,idCopy(arg),IDEAL_CMD,err);
+  rChangeCurrRing(oldR);
+  if (err) return NULL;
+  return I;
+}
+
+int ii_CallProcId2Int(const char *lib,const char *proc, ideal arg, const ring R)
+{
+  char *plib = iiConvName(lib);
+  idhdl h=ggetid(plib);
+  omFree(plib);
+  if (h==NULL)
+  {
+    BOOLEAN bo=iiLibCmd(lib,TRUE,TRUE,FALSE);
+    if (bo) return 0;
+  }
+  BOOLEAN err;
+  ring oldR=currRing;
+  rChangeCurrRing(R);
+  int I=(int)(long)iiCallLibProc1(proc,idCopy(arg),IDEAL_CMD,err);
+  rChangeCurrRing(oldR);
+  if (err) return 0;
+  return I;
+}
+
+/// args: NULL terminated array of arguments
 /// arg_types: 0 terminated array of corresponding types
-void* iiCallLibProcM(const char*n, void **args, int* arg_types, BOOLEAN &err)
+leftv ii_CallLibProcM(const char*n, void **args, int* arg_types, const ring R, BOOLEAN &err)
 {
   idhdl h=ggetid(n);
   if ((h==NULL)
@@ -656,6 +710,7 @@ void* iiCallLibProcM(const char*n, void **args, int* arg_types, BOOLEAN &err)
   // ring handling
   idhdl save_ringhdl=currRingHdl;
   ring save_ring=currRing;
+  rChangeCurrRing(R);
   iiCallLibProcBegin();
   // argument:
   if (arg_types[0]!=0)
@@ -668,7 +723,7 @@ void* iiCallLibProcM(const char*n, void **args, int* arg_types, BOOLEAN &err)
     tmp.rtyp=arg_types[0];
     while(arg_types[i]!=0)
     {
-      tt->next=(leftv)omAlloc0(sizeof(sleftv));
+      tt->next=(leftv)omAlloc0Bin(sleftv_bin);
       tt=tt->next;
       tt->rtyp=arg_types[i];
       tt->data=args[i];
@@ -685,10 +740,10 @@ void* iiCallLibProcM(const char*n, void **args, int* arg_types, BOOLEAN &err)
   // return
   if (err==FALSE)
   {
-    void*r=iiRETURNEXPR.data;
-    iiRETURNEXPR.data=NULL;
-    iiRETURNEXPR.CleanUp();
-    return r;
+    leftv h=(leftv)omAllocBin(sleftv_bin);
+    memcpy(h,&iiRETURNEXPR,sizeof(sleftv));
+    iiRETURNEXPR.Init();
+    return h;
   }
   return NULL;
 }
@@ -746,10 +801,14 @@ extern "C"
 #  undef  SI_GET_BUILTIN_MOD_INIT0
 };
 
+extern "C" int flint_mod_init(SModulFunctions* psModulFunctions);
 
 SModulFunc_t
 iiGetBuiltinModInit(const char* libname)
 {
+#ifdef HAVE_FLINT
+  if (strcmp(libname,"flint.so")==0) return SI_MOD_INIT0(flint);
+#endif
 #  define SI_GET_BUILTIN_MOD_INIT(name) if (strcmp(libname, #name ".so") == 0){ return SI_MOD_INIT0(name); }
           SI_FOREACH_BUILTIN(SI_GET_BUILTIN_MOD_INIT)
 #  undef  SI_GET_BUILTIN_MOD_INIT
@@ -778,20 +837,19 @@ BOOLEAN iiTryLoadLib(leftv v, const char *id)
     *libname = mytolower(*libname);
     if((LT = type_of_LIB(libname, libnamebuf)) > LT_NOTFOUND)
     {
-      char *s=omStrDup(libname);
       #ifdef HAVE_DYNAMIC_LOADING
       char libnamebuf[1024];
       #endif
 
       if (LT==LT_SINGULAR)
-        LoadResult = iiLibCmd(s, FALSE, FALSE,TRUE);
+        LoadResult = iiLibCmd(libname, FALSE, FALSE,TRUE);
       #ifdef HAVE_DYNAMIC_LOADING
       else if ((LT==LT_ELF) || (LT==LT_HPUX))
-        LoadResult = load_modules(s,libnamebuf,FALSE);
+        LoadResult = load_modules(libname,libnamebuf,FALSE);
       #endif
       else if (LT==LT_BUILTIN)
       {
-        LoadResult=load_builtin(s,FALSE, iiGetBuiltinModInit(s));
+        LoadResult=load_builtin(libname,FALSE, iiGetBuiltinModInit(libname));
       }
       if(!LoadResult )
       {
@@ -823,14 +881,11 @@ BOOLEAN iiLocateLib(const char* lib, char* where)
     return FALSE;;
 }
 
-BOOLEAN iiLibCmd( char *newlib, BOOLEAN autoexport, BOOLEAN tellerror, BOOLEAN force )
+BOOLEAN iiLibCmd( const char *newlib, BOOLEAN autoexport, BOOLEAN tellerror, BOOLEAN force )
 {
+  if (strcmp(newlib,"Singular")==0) return FALSE;
   char libnamebuf[1024];
-  // procinfov pi;
-  // idhdl h;
   idhdl pl;
-  // idhdl hl;
-  // long pos = 0L;
   char *plib = iiConvName(newlib);
   FILE * fp = feFopen( newlib, "r", libnamebuf, tellerror );
   // int lines = 1;
@@ -852,19 +907,22 @@ BOOLEAN iiLibCmd( char *newlib, BOOLEAN autoexport, BOOLEAN tellerror, BOOLEAN f
   {
     if(IDTYP(pl)!=PACKAGE_CMD)
     {
+      omFree(plib);
       WarnS("not of type package.");
       fclose(fp);
       return TRUE;
     }
-    if (!force) return FALSE;
+    if (!force)
+    {
+      omFree(plib);
+      return FALSE;
+    }
   }
   LoadResult = iiLoadLIB(fp, libnamebuf, newlib, pl, autoexport, tellerror);
-  omFree((ADDRESS)newlib);
 
   if(!LoadResult) IDPACKAGE(pl)->loaded = TRUE;
   omFree((ADDRESS)plib);
-
- return LoadResult;
+  return LoadResult;
 }
 /*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*/
 static void iiCleanProcs(idhdl &root)
@@ -915,7 +973,7 @@ static void iiRunInit(package p)
 BOOLEAN iiLoadLIB(FILE *fp, const char *libnamebuf, const char*newlib,
              idhdl pl, BOOLEAN autoexport, BOOLEAN tellerror)
 {
-  extern FILE *yylpin;
+  EXTERN_VAR FILE *yylpin;
   libstackv ls_start = library_stack;
   lib_style_types lib_style;
 
@@ -923,7 +981,7 @@ BOOLEAN iiLoadLIB(FILE *fp, const char *libnamebuf, const char*newlib,
   #if YYLPDEBUG > 1
   print_init();
   #endif
-  extern int lpverbose;
+  EXTERN_VAR int lpverbose;
   if (BVERBOSE(V_DEBUG_LIB)) lpverbose=1;
   else lpverbose=0;
   // yylplex sets also text_buffer
@@ -1022,9 +1080,11 @@ int iiAddCproc(const char *libname, const char *procname, BOOLEAN pstatic,
   && (IDTYP(h)==PROC_CMD))
   {
     pi = IDPROC(h);
+    #if 0
     if ((pi->language == LANG_SINGULAR)
     &&(BVERBOSE(V_REDEFINE)))
       Warn("extend `%s`",procname);
+    #endif
   }
   else
   {
@@ -1088,12 +1148,38 @@ int iiAddCprocTop(const char *libname, const char *procname, BOOLEAN pstatic,
 
 /*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*/
 #ifdef HAVE_DYNAMIC_LOADING
-BOOLEAN load_modules(const char *newlib, char *fullname, BOOLEAN autoexport)
+#include <map>
+#include <string>
+#include <pthread.h>
+
+THREAD_VAR std::map<std::string, void *> *dyn_modules;
+
+bool registered_dyn_module(char *fullname) {
+  if (dyn_modules == NULL)
+    return false;
+  std::string fname = fullname;
+  return dyn_modules->count(fname) != 0;
+}
+
+void register_dyn_module(char *fullname, void * handle) {
+  std::string fname = fullname;
+  if (dyn_modules == NULL)
+    dyn_modules = new std::map<std::string, void *>();
+  dyn_modules->insert(std::pair<std::string, void *>(fname, handle));
+}
+
+void close_all_dyn_modules() {
+  for (std::map<std::string, void *>::iterator it = dyn_modules->begin();
+       it != dyn_modules->end();
+       it++)
+  {
+    dynl_close(it->second);
+  }
+  delete dyn_modules;
+  dyn_modules = NULL;
+}
+BOOLEAN load_modules_aux(const char *newlib, char *fullname, BOOLEAN autoexport)
 {
-#ifdef HAVE_STATIC
-  WerrorS("mod_init: static version can not load modules");
-  return TRUE;
-#else
 /*
   typedef int (*fktn_t)(int(*iiAddCproc)(const char *libname, const char *procname,
                                BOOLEAN pstatic,
@@ -1104,13 +1190,12 @@ BOOLEAN load_modules(const char *newlib, char *fullname, BOOLEAN autoexport)
   char *plib = iiConvName(newlib);
   BOOLEAN RET=TRUE;
   int token;
-  char FullName[256];
-
-  memset(FullName,0,256);
+  int l=si_max((int)strlen(fullname),(int)strlen(newlib))+3;
+  char *FullName=(char*)omAlloc0(l);
 
   if( *fullname != '/' &&  *fullname != '.' )
     sprintf(FullName, "./%s", newlib);
-  else strncpy(FullName, fullname,255);
+  else strncpy(FullName, fullname,l);
 
 
   if(IsCmd(plib, token))
@@ -1139,12 +1224,14 @@ BOOLEAN load_modules(const char *newlib, char *fullname, BOOLEAN autoexport)
   else
   {
     pl = enterid( plib,0, PACKAGE_CMD, &IDROOT, TRUE );
+    omFree(plib); /* enterid copied plib*/
     IDPACKAGE(pl)->libname=omStrDup(newlib);
   }
   IDPACKAGE(pl)->language = LANG_C;
   if (dynl_check_opened(FullName))
   {
     if (BVERBOSE(V_LOAD_LIB)) Warn( "%s already loaded as C library", fullname);
+    omFreeSize(FullName,l);
     return FALSE;
   }
   if((IDPACKAGE(pl)->handle=dynl_open(FullName))==(void *)NULL)
@@ -1177,6 +1264,7 @@ BOOLEAN load_modules(const char *newlib, char *fullname, BOOLEAN autoexport)
       }
       currPack->loaded=1;
       currPack=s; /* reset currPack to previous */
+      register_dyn_module(fullname, IDPACKAGE(pl)->handle);
       RET=FALSE;
     }
     else
@@ -1189,8 +1277,17 @@ BOOLEAN load_modules(const char *newlib, char *fullname, BOOLEAN autoexport)
   }
 
   load_modules_end:
+  omFreeSize(FullName,l);
   return RET;
-#endif /*STATIC */
+}
+
+BOOLEAN load_modules(const char *newlib, char *fullname, BOOLEAN autoexport)
+{
+  GLOBAL_VAR static pthread_mutex_t mutex = PTHREAD_MUTEX_INITIALIZER;
+  pthread_mutex_lock(&mutex);
+  BOOLEAN r = load_modules_aux(newlib, fullname, autoexport);
+  pthread_mutex_unlock(&mutex);
+  return r;
 }
 #endif /* HAVE_DYNAMIC_LOADING */
 /*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*/
@@ -1225,6 +1322,7 @@ BOOLEAN load_builtin(const char *newlib, BOOLEAN autoexport, SModulFunc_t init)
     pl = enterid( plib,0, PACKAGE_CMD, &IDROOT, TRUE );
     IDPACKAGE(pl)->libname=omStrDup(newlib);
   }
+  omFree(plib);
   IDPACKAGE(pl)->language = LANG_C;
 
   IDPACKAGE(pl)->handle=(void *)NULL;
@@ -1271,10 +1369,10 @@ void module_help_proc(const char *newlib,const char *p, const char *help)
   {
     package s=currPack;
     currPack=IDPACKAGE(pl);
-    char buff[256];
-    buff[255]='\0';
-    strncpy(buff,p,255);
-    strncat(buff,"_help",255-strlen(p));
+    char buff[SINGULAR_PATH_LENGTH];
+    buff[SINGULAR_PATH_LENGTH-1]='\0';
+    strncpy(buff,p,SINGULAR_PATH_LENGTH-1);
+    strncat(buff,"_help",SINGULAR_PATH_LENGTH-1-strlen(p));
     idhdl h=enterid(buff,0,STRING_CMD,&IDROOT,FALSE);
     IDSTRING(h)=omStrDup(help);
     currPack=s;
