@@ -8,7 +8,6 @@
 
 #include "kernel/mod2.h"
 #include "misc/options.h"
-#include "omalloc/omalloc.h"
 
 #include "Singular/tok.h"
 #include "Singular/subexpr.h"
@@ -24,12 +23,13 @@ static BOOLEAN DumpAscii(FILE *fd, idhdl h,char ***list_of_libs);
 static BOOLEAN DumpAsciiIdhdl(FILE *fd, idhdl h,char ***list_of_libs);
 static const char* GetIdString(idhdl h);
 static int DumpRhs(FILE *fd, idhdl h);
-static BOOLEAN DumpQring(FILE *fd, idhdl h, const char *type_str);
+static BOOLEAN DumpQring(FILE *fd, idhdl h);
+static BOOLEAN DumpNCring(FILE *fd, idhdl h);
 static BOOLEAN DumpAsciiMaps(FILE *fd, idhdl h, idhdl rhdl);
 static BOOLEAN CollectLibs(char *name, char ***list_of_libs);
 //static BOOLEAN DumpLibs(FILE *fd, char ***list_of_libs);
 
-extern si_link_extension si_link_root;
+EXTERN_VAR si_link_extension si_link_root;
 
 /* =============== ASCII ============================================= */
 BOOLEAN slOpenAscii(si_link l, short flag, leftv /*h*/)
@@ -170,6 +170,21 @@ BOOLEAN slWriteAscii(si_link l, leftv v)
         }
         break;
       }
+    #if 1
+    case LIST_CMD:
+      {
+        lists l=(lists)v->Data();
+        for(int i=0;i<l->nr;i++)
+        {
+          char *s=l->m[i].String();
+          fwrite(s,strlen(s),1,outfile);
+          omFree(s);
+          if (i!=l->nr-1) fputc(',',outfile);
+          fputc('\n',outfile);
+        }
+        break;
+      }
+    #endif
     default:
       s = v->String();
       // free v ??
@@ -294,24 +309,30 @@ static BOOLEAN DumpAsciiIdhdl(FILE *fd, idhdl h, char ***list_of_libs)
   {
     if (strcmp(IDID(h),"Top")==0) return FALSE; // do not dump "Top"
     if (IDPACKAGE(h)->language==LANG_SINGULAR) return FALSE;
+    if (IDPACKAGE(h)->language==LANG_MIX) return FALSE;
   }
   if (type_id == CRING_CMD)
   {
     // do not dump the default CRINGs:
     if (strcmp(IDID(h),"QQ")==0) return FALSE;
     if (strcmp(IDID(h),"ZZ")==0) return FALSE;
+    #ifdef SINGULAR_4_2
     if (strcmp(IDID(h),"AE")==0) return FALSE;
     if (strcmp(IDID(h),"QAE")==0) return FALSE;
-    if (strcmp(IDID(h),"flint_poly_Q")==0) return FALSE;
+    #endif
   }
 
   // we do not throw an error if a wrong type was attempted to be dumped
   if (type_str == NULL)
     return FALSE;
 
+  // handle nc-rings separately
+  if ((type_id == RING_CMD)&&(rIsNCRing(IDRING(h))))
+    return DumpNCring(fd,h);
+
   // handle qrings separately
   if ((type_id == RING_CMD)&&(IDRING(h)->qideal!=NULL))
-    return DumpQring(fd, h, type_str);
+    return DumpQring(fd, h);
 
   // C-proc not to be dumped
   if ((type_id == PROC_CMD) && (IDPROC(h)->language == LANG_C))
@@ -337,6 +358,11 @@ static BOOLEAN DumpAsciiIdhdl(FILE *fd, idhdl h, char ***list_of_libs)
     if (fprintf(fd, "[%d][%d]", IDINTVEC(h)->rows(), IDINTVEC(h)->cols())
         == EOF) return TRUE;
   }
+  else if (type_id == SMATRIX_CMD)
+  {
+    ideal id = IDIDEAL(h);
+    if (fprintf(fd, "[%d][%d]", (int)id->rank, IDELEMS(id))== EOF) return TRUE;
+  }
 
   if (type_id == PACKAGE_CMD)
   {
@@ -362,13 +388,16 @@ static const char* GetIdString(idhdl h)
   switch(type)
   {
     case LIST_CMD:
-    {
-      lists l = IDLIST(h);
-      int i, nl = l->nr + 1;
-
-      for (i=0; i<nl; i++)
-        if (GetIdString((idhdl) &(l->m[i])) == NULL) return NULL;
-    }
+    //{
+    //
+    //
+    //  lists l = IDLIST(h);
+    //  int i, nl = l->nr + 1;
+//
+    //  for (i=0; i<nl; i++)
+    //    if (GetIdString((idhdl) &(l->m[i])) == NULL) return NULL;
+    //  break;
+    //}
     case CRING_CMD:
     #ifdef SINGULAR_4_2
     case CNUMBER_CMD:
@@ -389,6 +418,7 @@ static const char* GetIdString(idhdl h)
     case VECTOR_CMD:
     case MODUL_CMD:
     case MATRIX_CMD:
+    case SMATRIX_CMD:
       return Tok2Cmdname(type);
 
     case MAP_CMD:
@@ -401,16 +431,46 @@ static const char* GetIdString(idhdl h)
   }
 }
 
-static BOOLEAN DumpQring(FILE *fd, idhdl h, const char *type_str)
+static BOOLEAN DumpNCring(FILE *fd, idhdl h)
 {
   char *ring_str = h->String();
-  if (fprintf(fd, "%s temp_ring = %s;\n", Tok2Cmdname(RING_CMD), ring_str)
-              == EOF) return TRUE;
-  if (fprintf(fd, "%s temp_ideal = %s;\n", Tok2Cmdname(IDEAL_CMD),
-              iiStringMatrix((matrix) IDRING(h)->qideal, 1, currRing, n_GetChar(currRing->cf)))
+  ring r=IDRING(h);
+
+  if (rIsPluralRing(r))
+  {
+    if (fprintf(fd, "ring temp_ring = %s;\n", ring_str)
+      == EOF) return TRUE;
+    if (fprintf(fd, "ideal temp_C = %s;\n",
+              iiStringMatrix((matrix) r->GetNC()->C, 2, r, n_GetChar(r->cf)))
+      == EOF) return TRUE;
+    if (fprintf(fd, "ideal temp_D = %s;\n",
+              iiStringMatrix((matrix) r->GetNC()->D, 2, r, n_GetChar(r->cf)))
+      == EOF) return TRUE;
+    if (fprintf(fd, "def %s = nc_algebra(temp_C,temp_D);\n",IDID(h)) == EOF)
+      return TRUE;
+    if (fputs("kill temp_ring;\n",fd) == EOF) return TRUE;
+  }
+  if (rIsLPRing(r))
+  {
+    //if (fprintf(fd, "ring %s = %s;\n", IDID(h), ring_str) == EOF) return TRUE;
+    //if (fprintf(fd, "attrib(%s,\"isLetterplaceRing\",%d);\n",IDID(h),r->isLPring) ==EOF) return TRUE;
+    Warn("cannot write LP ring %s",IDID(h));
+    return TRUE;
+  }
+  omFree(ring_str);
+  return FALSE;
+}
+
+static BOOLEAN DumpQring(FILE *fd, idhdl h)
+{
+  char *ring_str = h->String();
+  ring r=IDRING(h);
+  if (fprintf(fd, "ring temp_ring = %s;\n", ring_str) == EOF) return TRUE;
+  if (fprintf(fd, "ideal temp_ideal = %s;\n",
+              iiStringMatrix((matrix) r->qideal, 1, currRing, n_GetChar(r->cf)))
       == EOF) return TRUE;
   if (fputs("attrib(temp_ideal, \"isSB\", 1);\n",fd) == EOF) return TRUE;
-  if (fprintf(fd, "%s %s = temp_ideal;\n", type_str, IDID(h)) == EOF)
+  if (fprintf(fd, "qring %s = temp_ideal;\n",IDID(h)) == EOF)
     return TRUE;
   if (fputs("kill temp_ring;\n",fd) == EOF) return TRUE;
   else
@@ -513,7 +573,8 @@ static int DumpRhs(FILE *fd, idhdl h)
     BOOLEAN need_klammer=FALSE;
     if (type_id == INTVEC_CMD) { fputs("intvec(",fd);need_klammer=TRUE; }
     else if (type_id == IDEAL_CMD) { fputs("ideal(",fd);need_klammer=TRUE; }
-    else if (type_id == MODUL_CMD) { fputs("module(",fd);need_klammer=TRUE; }
+    else if ((type_id == MODUL_CMD)||(type_id == SMATRIX_CMD))
+                                   { fputs("module(",fd);need_klammer=TRUE; }
     else if (type_id == BIGINT_CMD) { fputs("bigint(",fd);need_klammer=TRUE; }
 
     if (fputs(rhs,fd) == EOF) return EOF;
