@@ -8,7 +8,6 @@
 
 #include "kernel/mod2.h"
 
-#include "omalloc/omalloc.h"
 #include "misc/options.h"
 #include "misc/intvec.h"
 #include "reporter/si_signals.h"
@@ -37,13 +36,13 @@
 // #endif
 // #endif
 
-omBin s_si_link_extension_bin = omGetSpecBin(sizeof(s_si_link_extension));
-omBin sip_link_bin = omGetSpecBin(sizeof(sip_link));
-omBin ip_link_bin = omGetSpecBin(sizeof(ip_link));
+VAR omBin s_si_link_extension_bin = omGetSpecBin(sizeof(s_si_link_extension));
+VAR omBin sip_link_bin = omGetSpecBin(sizeof(sip_link));
+VAR omBin ip_link_bin = omGetSpecBin(sizeof(ip_link));
 
 /* ====================================================================== */
 static si_link_extension slTypeInit(si_link_extension s, const char* type);
-si_link_extension si_link_root=NULL;
+VAR si_link_extension si_link_root=NULL;
 
 BOOLEAN slInit(si_link l, char *istr)
 {
