@@ -15,22 +15,16 @@
 
 #include "kernel/mod2.h"
 #include "misc/sirandom.h"
-
 #include "omalloc/omalloc.h"
 #include "misc/mylimits.h"
-
 #include "reporter/si_signals.h"
-
 #include "factory/factory.h"
-
 #include "coeffs/si_gmp.h"
 #include "coeffs/coeffs.h"
-#include "coeffs/OPAE.h"
-#include "coeffs/OPAEQ.h"
-#include "coeffs/OPAEp.h"
 #include "coeffs/flintcf_Q.h"
+#include "coeffs/flintcf_Qrat.h"
 #include "coeffs/flintcf_Zn.h"
-
+#include "coeffs/rmodulon.h"
 #include "polys/ext_fields/algext.h"
 #include "polys/ext_fields/transext.h"
 #include "polys/nc/gb_hack.h"
@@ -74,15 +68,7 @@
 
 #ifdef HAVE_NTL
 #include <NTL/version.h>
-#include <NTL/tools.h>
-#ifdef NTL_CLIENT
-NTL_CLIENT
 #endif
-#endif
-
-
-static FORCE_INLINE void number2mpz(number n, mpz_t m){ number2mpz(n, coeffs_BIGINT, m); }
-static FORCE_INLINE number mpz2number(mpz_t m){ return mpz2number(m, coeffs_BIGINT); }
 
 
 void setListEntry(lists L, int index, mpz_t n)
@@ -98,7 +84,7 @@ void setListEntry(lists L, int index, mpz_t n)
       return;
     }
   }
-  number nn = mpz2number(n);
+  number nn = mpz2number(n, coeffs_BIGINT);
   L->m[index].rtyp = BIGINT_CMD; L->m[index].data = (void*)nn;
 }
 
@@ -118,7 +104,7 @@ void setListEntry_ui(lists L, int index, unsigned long ui)
 }
 
 /* Factoring with Pollard's rho method. stolen from GMP/demos */
-static unsigned add[] = {4, 2, 4, 2, 4, 6, 2, 6};
+STATIC_VAR unsigned add[] = {4, 2, 4, 2, 4, 6, 2, 6};
 
 static int factor_using_division (mpz_t t, unsigned int limit,lists primes, int *multiplicities,int &index, unsigned long bound)
 {
@@ -372,7 +358,7 @@ lists primeFactorisation(const number n, const int pBound)
 {
   int i;
   int index=0;
-  mpz_t nn; number2mpz(n, nn);
+  mpz_t nn; number2mpz(n, coeffs_BIGINT, nn);
   lists primes = (lists)omAllocBin(slists_bin); primes->Init(1000);
   int* multiplicities = (int*)omAlloc0(1000*sizeof(int));
   int positive=1;
@@ -419,10 +405,6 @@ lists primeFactorisation(const number n, const int pBound)
   return L;
 }
 
-#ifdef HAVE_STATIC
-#undef HAVE_DYN_RL
-#endif
-
 //#ifdef HAVE_LIBPARSER
 //#  include "libparse.h"
 //#endif /* HAVE_LIBPARSER */
@@ -457,7 +439,7 @@ void singular_example(char *str)
     *ss='\0';
     ss--;
   }
-  idhdl h=IDROOT->get(s,myynest);
+  idhdl h=IDROOT->get_level(s,0);
   if ((h!=NULL) && (IDTYP(h)==PROC_CMD))
   {
     char *lib=iiGetLibName(IDPROC(h));
@@ -532,13 +514,14 @@ const struct soptionStruct optionStruct[]=
   {"sugarCrit",    Sy_bit(OPT_SUGARCRIT),      ~Sy_bit(OPT_SUGARCRIT)   },
   {"teach",        Sy_bit(OPT_DEBUG),          ~Sy_bit(OPT_DEBUG)  },
   {"notSyzMinim",  Sy_bit(OPT_NO_SYZ_MINIM),   ~Sy_bit(OPT_NO_SYZ_MINIM)  },
-  /* 9 return SB in syz, quotient, intersect */
+  /* 9 return SB in syz, quotient, intersect, modulo */
   {"returnSB",     Sy_bit(OPT_RETURN_SB),      ~Sy_bit(OPT_RETURN_SB)  },
   {"fastHC",       Sy_bit(OPT_FASTHC),         ~Sy_bit(OPT_FASTHC)  },
   /* 11-19 sort in L/T */
   {"staircaseBound",Sy_bit(OPT_STAIRCASEBOUND),~Sy_bit(OPT_STAIRCASEBOUND)  },
   {"multBound",    Sy_bit(OPT_MULTBOUND),      ~Sy_bit(OPT_MULTBOUND)  },
   {"degBound",     Sy_bit(OPT_DEGBOUND),       ~Sy_bit(OPT_DEGBOUND)  },
+  {"redTailSyz",   Sy_bit(OPT_REDTAIL_SYZ),    ~Sy_bit(OPT_REDTAIL_SYZ) },
   /* 25 no redTail(p)/redTail(s) */
   {"redTail",      Sy_bit(OPT_REDTAIL),        ~Sy_bit(OPT_REDTAIL)  },
   {"redThrough",   Sy_bit(OPT_REDTHROUGH),     ~Sy_bit(OPT_REDTHROUGH)  },
@@ -686,7 +669,7 @@ BOOLEAN setOption(leftv res, leftv v)
         #ifdef YYDEBUG
         #if YYDEBUG
         /*debugging the bison grammar --> grammar.cc*/
-        extern int    yydebug;
+        EXTERN_VAR int    yydebug;
         if (BVERBOSE(V_YACC)) yydebug=1;
         else                  yydebug=0;
         #endif
@@ -700,7 +683,7 @@ BOOLEAN setOption(leftv res, leftv v)
         #ifdef YYDEBUG
         #if YYDEBUG
         /*debugging the bison grammar --> grammar.cc*/
-        extern int    yydebug;
+        EXTERN_VAR int    yydebug;
         if (BVERBOSE(V_YACC)) yydebug=1;
         else                  yydebug=0;
         #endif
@@ -717,7 +700,6 @@ BOOLEAN setOption(leftv res, leftv v)
   } while (v!=NULL);
 
    // set global variable to show memory usage
-  extern int om_sing_opt_show_mem;
   if (BVERBOSE(V_SHOW_MEM)) om_sing_opt_show_mem = 1;
   else om_sing_opt_show_mem = 0;
 
@@ -788,13 +770,13 @@ const char *singular_date=__DATE__ " " __TIME__;
 char * versionString(/*const bool bShowDetails = false*/ )
 {
   StringSetS("");
-  StringAppend("Singular for %s version %s (%d, %d bit) %s #%s",
+  StringAppend("Singular for %s version %s (%d, %d bit) %s",
                S_UNAME, VERSION, // SINGULAR_VERSION,
                SINGULAR_VERSION, sizeof(void*)*8,
 #ifdef MAKE_DISTRIBUTION
-               VERSION_DATE, GIT_VERSION);
+               VERSION_DATE);
 #else
-               singular_date, GIT_VERSION);
+               singular_date);
 #endif
   StringAppendS("\nwith\n\t");
 
@@ -806,15 +788,15 @@ char * versionString(/*const bool bShowDetails = false*/ )
               StringAppend("GMP(%s),", gmp_version);
 #endif
 #ifdef HAVE_NTL
-#include <NTL/version.h>
               StringAppend("NTL(%s),",NTL_VERSION);
 #endif
 
 #ifdef HAVE_FLINT
-              StringAppend("FLINT(%s),",version);
+              StringAppend("FLINT(%s),",FLINT_VERSION);
 #endif
-              StringAppendS("factory(" FACTORYVERSION "),\n\t");
-#ifdef XMEMORY_H
+//              StringAppendS("factory(" FACTORYVERSION "),");
+              StringAppendS("\n\t");
+#ifndef HAVE_OMALLOC
               StringAppendS("xalloc,");
 #else
               StringAppendS("omalloc,");
@@ -846,6 +828,13 @@ char * versionString(/*const bool bShowDetails = false*/ )
 #ifdef HAVE_PLURAL
               StringAppendS("Plural,");
 #endif
+#ifdef HAVE_VSPACE
+  #if defined(__GNUC__) && (__GNUC__<9) &&!defined(__clang__)
+              StringAppendS("vspace(1),");
+  #else
+              StringAppendS("vspace(2),");
+  #endif
+#endif
 #ifdef HAVE_DBM
               StringAppendS("DBM,\n\t");
 #else
@@ -854,7 +843,9 @@ char * versionString(/*const bool bShowDetails = false*/ )
 #ifdef HAVE_DYNAMIC_LOADING
               StringAppendS("dynamic modules,");
 #endif
-              if (p_procs_dynamic) StringAppendS("dynamic p_Procs,");
+#ifdef HAVE_DYNANIC_PPROCS
+              StringAppendS("dynamic p_Procs,");
+#endif
 #if YYDEBUG
               StringAppendS("YYDEBUG=1,");
 #endif
@@ -889,6 +880,11 @@ char * versionString(/*const bool bShowDetails = false*/ )
 #ifdef __NO_INLINE__
               StringAppendS("CC:NO_INLINE,");
 #endif
+#ifdef HAVE_NTL
+  #ifdef NTL_AVOID_BRANCHING
+  #undef HAVE_GENERIC_ADD
+  #endif
+#endif
 #ifdef HAVE_GENERIC_ADD
               StringAppendS("GenericAdd,");
 #else
@@ -930,7 +926,9 @@ char * versionString(/*const bool bShowDetails = false*/ )
 #ifdef __GNUC__
               "(ver: " __VERSION__ ")"
 #endif
-              "\n",AC_CONFIGURE_ARGS, CC,CFLAGS, CXX,CXXFLAGS,  DEFS,CPPFLAGS,  LDFLAGS,LIBS);
+              "\n",AC_CONFIGURE_ARGS, CC,CFLAGS " " PTHREAD_CFLAGS,
+              CXX,CXXFLAGS " " PTHREAD_CFLAGS,  DEFS,CPPFLAGS,  LDFLAGS,
+              LIBS " " PTHREAD_LIBS);
               feStringAppendResources(0);
               feStringAppendBrowsers(0);
               StringAppendS("\n");
@@ -1100,7 +1098,7 @@ void m2_end(int i)
 {
   if (!m2_end_called)
   {
-    extern FILE* File_Profiling;
+    EXTERN_VAR FILE* File_Profiling;
     if (File_Profiling!=NULL) { fclose(File_Profiling); File_Profiling=NULL; }
     m2_end_called = TRUE;
 #ifdef HAVE_SIMPLEIPC
@@ -1204,35 +1202,14 @@ extern "C"
   }
 }
 
-#ifdef SINGULAR_4_2
-static n_coeffType n_pAE=n_unknown;
-static BOOLEAN ii_pAE_init(leftv res,leftv a)
-{
-  if (a->Typ()!=INT_CMD)
-  {
-    WerrorS("`int` expected");
-    return TRUE;
-  }
-  else
-  {
-    res->rtyp=CRING_CMD;
-    res->data=(void*)nInitChar(n_pAE,(void*)a->Data());
-    return FALSE;
-  }
-}
-#endif
 #ifdef HAVE_FLINT
-static n_coeffType n_FlintZn=n_unknown;
+STATIC_VAR n_coeffType n_FlintZn=n_unknown;
+STATIC_VAR n_coeffType n_FlintQ=n_unknown;
+//STATIC_VAR n_coeffType n_FlintQrat=n_unknown;
 static BOOLEAN ii_FlintZn_init(leftv res,leftv a)
 {
-  if ((a->Typ()!=INT_CMD)
-  ||(a->next==NULL)
-  ||(a->next->Typ()!=STRING_CMD))
-  {
-    WerrorS("`int`i,`string` expected");
-    return TRUE;
-  }
-  else
+  const short t[]={2,INT_CMD,STRING_CMD};
+  if (iiCheckTypes(a,t,1))
   {
     flintZn_struct p;
     p.ch=(int)(long)a->Data();
@@ -1241,6 +1218,75 @@ static BOOLEAN ii_FlintZn_init(leftv res,leftv a)
     res->data=(void*)nInitChar(n_FlintZn,(void*)&p);
     return FALSE;
   }
+  return TRUE;
+}
+static BOOLEAN ii_FlintQ_init(leftv res,leftv a)
+{
+  const short t[]={1,STRING_CMD};
+  if (iiCheckTypes(a,t,1))
+  {
+    char* p;
+    p=(char*)a->Data();
+    res->rtyp=CRING_CMD;
+    res->data=(void*)nInitChar(n_FlintQ,(void*)p);
+    return FALSE;
+  }
+  return TRUE;
+}
+#if __FLINT_RELEASE >= 20503
+static BOOLEAN ii_FlintQrat_init(leftv res,leftv a)
+{
+  if (a==NULL)
+  {
+    WerrorS("at least one name required");
+    return TRUE;
+  }
+  QaInfo par;
+  #ifdef QA_DEBUG
+  par.C=r->cf;
+  a=a->next;
+  #endif
+  par.N=a->listLength();
+  par.names=(char**)omAlloc(par.N*sizeof(char*));
+  int i=0;
+  while(a!=NULL)
+  {
+    par.names[i]=omStrDup(a->Name());
+    i++;
+    a=a->next;
+  }
+  res->rtyp=CRING_CMD;
+  res->data=(void*)nInitChar(n_FlintQrat,&par);
+  for(i=par.N-1;i>=0;i--)
+  {
+    omFree(par.names[i]);
+  }
+  omFreeSize(par.names,par.N*sizeof(char*));
+  return FALSE;
+}
+#endif
+extern "C" int flint_mod_init(SModulFunctions* psModulFunctions)
+{
+    package save=currPack;
+    currPack=basePack;
+    n_FlintQ=nRegister(n_unknown,flintQ_InitChar);
+    if (n_FlintQ!=n_unknown)
+    {
+      iiAddCproc("kernel","flintQp",FALSE,ii_FlintQ_init);
+      nRegisterCfByName(flintQInitCfByName,n_FlintQ);
+    }
+#if __FLINT_RELEASE >= 20503
+    iiAddCproc("kernel","flintQ",FALSE,ii_FlintQrat_init);
+    nRegisterCfByName(flintQInitCfByName,n_FlintQ);
+#endif
+    n_FlintZn=nRegister(n_unknown,flintZn_InitChar);
+    if (n_FlintZn!=n_unknown)
+    {
+      iiAddCproc("kernel","flintZn",FALSE,ii_FlintZn_init);
+      nRegisterCfByName(flintZnInitCfByName,n_FlintZn);
+    }
+    currPack=save;
+    return MAX_TOK;
 }
 #endif
 
@@ -1307,25 +1353,9 @@ static BOOLEAN iiCrossProd(leftv res, leftv args)
 /*2
 * initialize components of Singular
 */
+static void callWerrorS(const char *s) { WerrorS(s); }
 void siInit(char *name)
 {
-// factory default settings: -----------------------------------------------
-  On(SW_USE_EZGCD);
-  On(SW_USE_CHINREM_GCD);
-  //On(SW_USE_FF_MOD_GCD);
-  On(SW_USE_EZGCD_P);
-  On(SW_USE_QGCD);
-  Off(SW_USE_NTL_SORT); // may be changed by an command line option
-  factoryError=WerrorS;
-
-// NTL error handling (>= 9.3.0)
-#ifdef HAVE_NTL
-#if (((NTL_MAJOR_VERSION==9)&&(NTL_MINOR_VERSION>=3))||(NTL_MAJOR_VERSION>=10))
-  ErrorMsgCallback=WerrorS;
-  ErrorCallback=HALT;
-#endif
-#endif
-
 // memory initialization: -----------------------------------------------
     om_Opts.OutOfMemoryFunc = omSingOutOfMemoryFunc;
 #ifndef OM_NDEBUG
@@ -1338,7 +1368,11 @@ void siInit(char *name)
     om_Opts.Keep = 0; /* OM_NDEBUG */
 #endif
     omInitInfo();
-
+// factory
+#ifndef HAVE_NTL
+  extern void initPT();
+  initPT();
+#endif
 // options ---------------------------------------------------------------
   si_opt_1=0;
 // interpreter tables etc.: -----------------------------------------------
@@ -1414,45 +1448,13 @@ void siInit(char *name)
     IDDATA(h)=(char*)nInitChar(n_Q,NULL);
     h=enterid("ZZ",0/*level*/, CRING_CMD,&(basePack->idroot),FALSE /*init*/,FALSE /*search*/);
     IDDATA(h)=(char*)nInitChar(n_Z,NULL);
+    nRegisterCfByName(nrnInitCfByName,n_Zn); // and n_Znm
     iiAddCproc("kernel","crossprod",FALSE,iiCrossProd);
     iiAddCproc("kernel","Float",FALSE,iiFloat);
     //h=enterid("RR",0/*level*/, CRING_CMD,&(basePack->idroot),FALSE /*init*/,FALSE /*search*/);
     //IDDATA(h)=(char*)nInitChar(n_R,NULL);
     //h=enterid("CC",0/*level*/, CRING_CMD,&(basePack->idroot),FALSE /*init*/,FALSE /*search*/);
     //IDDATA(h)=(char*)nInitChar(n_long_C,NULL);
-    n_coeffType t;
-#ifdef SINGULAR_4_2
-    t=nRegister(n_unknown,n_AEInitChar);
-    if (t!=n_unknown)
-    {
-      h=enterid("AE",0/*level*/, CRING_CMD,&(basePack->idroot),FALSE /*init*/,FALSE /*search*/);
-      IDDATA(h)=(char*)nInitChar(t,NULL);
-    }
-    t=nRegister(n_unknown,n_QAEInitChar);
-    if (t!=n_unknown)
-    {
-      h=enterid("QAE",0/*level*/, CRING_CMD,&(basePack->idroot),FALSE /*init*/,FALSE /*search*/);
-      IDDATA(h)=(char*)nInitChar(t,NULL);
-    }
-    n_pAE=nRegister(n_unknown,n_pAEInitChar);
-    if (n_pAE!=n_unknown)
-    {
-      iiAddCproc("kernel","pAE",FALSE,ii_pAE_init);
-    }
-#endif
-    #ifdef HAVE_FLINT
-    t=nRegister(n_unknown,flintQ_InitChar);
-    if (t!=n_unknown)
-    {
-      h=enterid("flint_poly_Q",0/*level*/, CRING_CMD,&(basePack->idroot),FALSE /*init*/,FALSE /*search*/);
-      IDDATA(h)=(char*)nInitChar(t,NULL);
-    }
-    n_FlintZn=nRegister(n_unknown,flintZn_InitChar);
-    if (n_FlintZn!=n_unknown)
-    {
-      iiAddCproc("kernel","flintZ",FALSE,ii_FlintZn_init);
-    }
-    #endif
   }
 // setting routines for PLURAL QRINGS:
 // allowing to use libpolys without libSingular(kStd)
@@ -1470,8 +1472,12 @@ void siInit(char *name)
     BITSET save1,save2;
     SI_SAVE_OPT(save1,save2);
     si_opt_2 &= ~Sy_bit(V_LOAD_LIB);
-    iiLibCmd(omStrDup("standard.lib"), TRUE,TRUE,TRUE);
+    iiLibCmd("standard.lib", TRUE,TRUE,TRUE);
     SI_RESTORE_OPT(save1,save2);
   }
+  // interpreter error handling
+  #ifndef __CYGWIN__
+  factoryError=callWerrorS; // to honour later changes of variable WerrorS
+  #endif
   errorreported = 0;
 }
