@@ -3,7 +3,7 @@
 #include "coeffs/numbers.h" // nRegister, coeffs.h
 #include "coeffs/rmodulon.h" // ZnmInfo
 #include "coeffs/bigintmat.h" // bigintmat
-#include "coeffs/longrat.h" // BIGINTs: nlGMP
+#include "coeffs/longrat.h" // BIGINTs: nlMPZ
 #include "polys/ext_fields/algext.h" // AlgExtInfo
 #include "misc/prime.h" // IsPrime
 #include "Singular/blackbox.h" // blackbox type
@@ -35,12 +35,22 @@ void crPrint(coeffs c)
 BOOLEAN jjCRING_Zp(leftv res, leftv a, leftv b)
 {
   coeffs c1=(coeffs)a->Data();
-  int    i2=(int)(long)b->Data();
+  unsigned long i2=(unsigned long)b->Data();
   if (c1->type==n_Z)
   {
-    if (i2==IsPrime(i2))
+    if (i2==(unsigned long)IsPrime(i2))
     {
-      res->data=(void *)nInitChar(n_Zp,(void*)(long)i2);
+      #ifndef TEST_ZN_AS_ZP
+      res->data=(void *)nInitChar(n_Zp,(void*)i2);
+      #else
+      ZnmInfo info;
+      mpz_t modBase;
+      mpz_init_set_ui(modBase,i2);
+      info.base= modBase;
+      info.exp= 1;
+      res->data=(void *)nInitChar(n_Zn,&info);
+      mpz_clear(modBase);
+      #endif
     }
     else
     {
@@ -56,7 +66,7 @@ BOOLEAN jjCRING_Zp(leftv res, leftv a, leftv b)
         if (i2<(8*sizeof(unsigned long)))
         {
           mpz_clear(modBase);
-          res->data=(void *) nInitChar(n_Z2m,(void*)(long)i2);
+          res->data=(void *) nInitChar(n_Z2m,(void*)i2);
           return FALSE;
         }
         else
@@ -83,8 +93,7 @@ BOOLEAN jjCRING_Zm(leftv res, leftv a, leftv b)
   {
     ZnmInfo info;
     mpz_t modBase;
-    mpz_init(modBase);
-    nlGMP(i2,modBase,coeffs_BIGINT); // FIXME? TODO? // extern void   nlGMP(number &i, mpz_t n, const coeffs r); // to be replaced with n_MPZ(modBase,i2,coeffs_BIGINT); // ?
+    nlMPZ(modBase,i2,coeffs_BIGINT);
     info.base= (mpz_ptr)modBase;
     info.exp= 1;
     if (mpz_popcount(modBase)==1) // is a power of 2
@@ -116,6 +125,7 @@ BOOLEAN jjEQUAL_CR(leftv res, leftv a, leftv b)
   coeffs a2=(coeffs)a->Data();
   coeffs b2=(coeffs)b->Data();
   res->data=(void*)(long)(a2==b2);
+  if (iiOp==NOTEQUAL) res->data=(char *)(!(long)res->data);
   return FALSE;
 }
 
