@@ -5,56 +5,47 @@
 * ABSTRACT: handling of leftv
 */
 
-#include <kernel/mod2.h>
+#include "kernel/mod2.h"
+#include "omalloc/omalloc.h"
+#include "misc/intvec.h"
+#include "misc/options.h"
+#include "coeffs/numbers.h"
+#include "coeffs/bigintmat.h"
+#include "coeffs/ffields.h" // nfShowMipo // minpoly printing...
+#include "polys/monomials/maps.h"
+#include "polys/matpol.h"
+#include "polys/monomials/ring.h"
 
-#include <omalloc/omalloc.h>
+// #include "coeffs/longrat.h"
 
-#include <misc/intvec.h>
-#include <misc/options.h>
+#include "kernel/polys.h"
+#include "kernel/ideals.h"
+#include "kernel/GBEngine/kstd1.h"
+#include "kernel/GBEngine/syz.h"
+#include "kernel/oswrapper/timer.h"
+#include "Singular/tok.h"
+#include "Singular/ipid.h"
+#include "Singular/ipshell.h"
+#include "Singular/lists.h"
+#include "Singular/attrib.h"
+#include "Singular/links/silink.h"
+#include "Singular/attrib.h"
+#include "Singular/ipprint.h"
+#include "Singular/subexpr.h"
+#include "Singular/blackbox.h"
+#include "Singular/number2.h"
 
-#include <coeffs/numbers.h>
-#include <coeffs/bigintmat.h>
-
-#include <coeffs/ffields.h> // nfShowMipo // minpoly printing...
-
-#include <polys/monomials/maps.h>
-#include <polys/matpol.h>
-#include <polys/monomials/ring.h>
-
-// #include <coeffs/longrat.h>
-
-#include <kernel/polys.h>
-#include <kernel/ideals.h>
-#include <kernel/GBEngine/kstd1.h>
-#include <kernel/GBEngine/syz.h>
-#include <kernel/oswrapper/timer.h>
-
-#include <Singular/tok.h>
-#include <Singular/ipid.h>
-#include <Singular/ipshell.h>
-#include <Singular/lists.h>
-#include <Singular/attrib.h>
-#include <Singular/links/silink.h>
-#include <Singular/attrib.h>
-#include <Singular/subexpr.h>
-#include <Singular/blackbox.h>
-#include <Singular/number2.h>
-
-#include <stdlib.h>
-#include <stdio.h>
-#include <string.h>
 #include <ctype.h>
-#include <unistd.h>
 
-omBin sSubexpr_bin = omGetSpecBin(sizeof(_ssubexpr));
-omBin sleftv_bin = omGetSpecBin(sizeof(sleftv));
-omBin procinfo_bin = omGetSpecBin(sizeof(procinfo));
-omBin libstack_bin = omGetSpecBin(sizeof(libstack));
-static omBin size_two_bin = omGetSpecBin(2);
+VAR omBin sSubexpr_bin = omGetSpecBin(sizeof(_ssubexpr));
+VAR omBin sleftv_bin = omGetSpecBin(sizeof(sleftv));
+VAR omBin procinfo_bin = omGetSpecBin(sizeof(procinfo));
+VAR omBin libstack_bin = omGetSpecBin(sizeof(libstack));
+STATIC_VAR omBin size_two_bin = omGetSpecBin(2);
 
-sleftv     sLastPrinted;
+INST_VAR sleftv     sLastPrinted;
 #ifdef SIQ
-BOOLEAN siq=FALSE;
+VAR BOOLEAN siq=FALSE;
 #endif
 
 int sleftv::listLength()
@@ -117,7 +108,25 @@ void sleftv::Print(leftv store, int spaces)
         case CMATRIX_CMD: // like BIGINTMAT
 #endif
         case BIGINTMAT_CMD:
-          ((bigintmat *)d)->pprint(80);
+          ((bigintmat *)d)->pprint(colmax);
+          break;
+        case BUCKET_CMD:
+          {
+            sBucket_pt b=(sBucket_pt)d;
+            if ((e==NULL)
+            && (TEST_V_QRING)
+            &&(currRing->qideal!=NULL))
+            {
+              poly p=pCopy(sBucketPeek(b));
+              p=jjNormalizeQRingP(p);
+              PrintNSpaces(spaces);
+              pWrite0(p);
+              pDelete(&p);
+              break;
+            }
+            else
+              sBucketPrint(b);
+          }
           break;
         case UNKNOWN:
         case DEF_CMD:
@@ -128,7 +137,7 @@ void sleftv::Print(leftv store, int spaces)
           PrintNSpaces(spaces);
           paPrint(n,(package)d);
           break;
-	case LIB_CMD:
+        case LIB_CMD:
         case NONE:
           return;
         case INTVEC_CMD:
@@ -138,13 +147,20 @@ void sleftv::Print(leftv store, int spaces)
         case RING_CMD:
         {
           PrintNSpaces(spaces);
-          const ring r = (const ring)d;
+          const ring r = (ring)d;
           rWrite(r, currRing == r);
           break;
         }
         case MATRIX_CMD:
           iiWriteMatrix((matrix)d,n,2, currRing, spaces);
           break;
+        case SMATRIX_CMD:
+        {
+          matrix m = id_Module2Matrix(id_Copy((ideal)d,currRing),currRing);
+          ipPrint_MA0(m, n);
+          id_Delete((ideal *) &m,currRing);
+          break;
+        }
         case MODUL_CMD:
         case IDEAL_CMD:
           if ((TEST_V_QRING)  &&(currRing->qideal!=NULL)
@@ -166,7 +182,7 @@ void sleftv::Print(leftv store, int spaces)
           {
             setFlag(this,FLAG_QRING);
             poly p=(poly)d;
-            jjNormalizeQRingP(p);
+            p=jjNormalizeQRingP(p);
             if (p!=(poly)d)
             {
               d=(void*)p;
@@ -301,7 +317,7 @@ void sleftv::Print(leftv store, int spaces)
     else if (t!=LIST_CMD) PrintS(" ");
     next->Print(NULL,spaces);
   }
-  else if (t!=LIST_CMD)
+  else if ((t!=LIST_CMD)&&(t!=SMATRIX_CMD))
   {
     PrintLn();
   }
@@ -336,7 +352,7 @@ void sleftv::CleanUp(ring r)
     if ((name!=NULL) && (name!=sNoName_fe) && (rtyp!=ALIAS_CMD))
     {
       //::Print("free %x (%s)\n",name,name);
-      omFree((ADDRESS)name);
+      omFree((ADDRESS)name); // may be larger >1000 char (large int)
     }
     //name=NULL;
     //flag=0;
@@ -430,11 +446,14 @@ static inline void * s_internalCopy(const int t,  void *d)
 #endif
     case BIGINTMAT_CMD:
       return (void*)bimCopy((bigintmat *)d);
+    case BUCKET_CMD:
+      return (void*)sBucketCopy((sBucket_pt)d);
     case INTVEC_CMD:
     case INTMAT_CMD:
       return (void *)ivCopy((intvec *)d);
     case MATRIX_CMD:
       return (void *)mp_Copy((matrix)d, currRing);
+    case SMATRIX_CMD:
     case IDEAL_CMD:
     case MODUL_CMD:
       return  (void *)idCopy((ideal)d);
@@ -464,8 +483,8 @@ static inline void * s_internalCopy(const int t,  void *d)
         ring r=(ring)d;
         if (r!=NULL)
         {
-          r->ref++;
-          //Print("s_internalCopy:+  ring %d, ref %d\n",r,r->ref);
+          rIncRefCnt(r);
+          //Print("s_internalCopy:+  ring %lx, ref %d\n",r,r->ref);
         }
         return d;
       }
@@ -500,13 +519,13 @@ void s_internalDelete(const int t,  void *d, const ring r)
     case CRING_CMD:
       {
         coeffs cf=(coeffs)d;
-        if ((cf->ref<=1)&&
-        ((cf->type <=n_long_R)
+        if ((cf->ref<1)&&
+        ((cf->type <=n_GF)
           ||((cf->type >=n_long_C)&&(cf->type <=n_CF))))
         {
-          Werror("cannot kill %s",nCoeffName(cf));
+          Warn("cannot kill `%s`",nCoeffName(cf));
         }
-        else
+        else // allow nKillChar for n_long_R, extensions, and user defined:
           nKillChar((coeffs)d);
         break;
       }
@@ -531,6 +550,12 @@ void s_internalDelete(const int t,  void *d, const ring r)
       delete v;
       break;
     }
+    case BUCKET_CMD:
+    {
+      sBucket_pt b=(sBucket_pt)d;
+      sBucketDeleteAndDestroy(&b);
+      break;
+    }
     case INTVEC_CMD:
     case INTMAT_CMD:
     {
@@ -545,6 +570,7 @@ void s_internalDelete(const int t,  void *d, const ring r)
       m->preimage=NULL;
       /* no break: continue as IDEAL*/
     }
+    case SMATRIX_CMD:
     case MATRIX_CMD:
     case IDEAL_CMD:
     case MODUL_CMD:
@@ -656,35 +682,6 @@ void s_internalDelete(const int t,  void *d, const ring r)
   }
 }
 
-void * slInternalCopy(leftv source, const int t, void *d, Subexpr e)
-{
-  if (t==STRING_CMD)
-  {
-      if ((e==NULL)
-      || (source->rtyp==LIST_CMD)
-      || ((source->rtyp==IDHDL)
-          &&((IDTYP((idhdl)source->data)==LIST_CMD)
-            || (IDTYP((idhdl)source->data)>MAX_TOK)))
-      || (source->rtyp>MAX_TOK))
-        return (void *)omStrDup((char *)d);
-      else if (e->next==NULL)
-      {
-        char *s=(char*)omAllocBin(size_two_bin);
-        s[0]=*(char *)d;
-        s[1]='\0';
-        return s;
-      }
-      #ifdef TEST
-      else
-      {
-        Werror("not impl. string-op in `%s`",my_yylinebuf);
-        return NULL;
-      }
-      #endif
-  }
-  return s_internalCopy(t,d);
-}
-
 void sleftv::Copy(leftv source)
 {
   Init();
@@ -692,7 +689,13 @@ void sleftv::Copy(leftv source)
   void *d=source->Data();
   if(!errorreported)
   {
-    data=s_internalCopy(rtyp,d);
+    if (rtyp==BUCKET_CMD)
+    {
+      rtyp=POLY_CMD;
+      data=(void*)pCopy(sBucketPeek((sBucket_pt)d));
+    }
+    else
+      data=s_internalCopy(rtyp,d);
     if ((source->attribute!=NULL)||(source->e!=NULL))
       attribute=source->CopyA();
     flag=source->flag;
@@ -706,6 +709,13 @@ void sleftv::Copy(leftv source)
 
 void * sleftv::CopyD(int t)
 {
+  if (Sy_inset(FLAG_OTHER_RING,flag))
+  {
+     flag&=~Sy_bit(FLAG_OTHER_RING);
+     WerrorS("object from another ring");
+     return NULL;
+  }
+
   if ((rtyp!=IDHDL)&&(rtyp!=ALIAS_CMD)&&(e==NULL))
   {
     if (iiCheckRing(t)) return NULL;
@@ -724,7 +734,7 @@ void * sleftv::CopyD(int t)
     return x;
   }
   void *d=Data(); // will also do a iiCheckRing
-  if ((!errorreported) && (d!=NULL)) return slInternalCopy(this,t,d,e);
+  if ((!errorreported) && (d!=NULL)) return s_internalCopy(t,d);
   return NULL;
 }
 
@@ -847,7 +857,8 @@ char *  sleftv::String(void *d, BOOLEAN typed, int dim)
           StringAppendS((char*) (typed ? ")" : ""));
           return StringEndS();
           }
-
+        case BUCKET_CMD:
+          return sBucketString((sBucket_pt)d);
         case MATRIX_CMD:
           s= iiStringMatrix((matrix)d,dim, currRing);
           if (typed)
@@ -863,18 +874,23 @@ char *  sleftv::String(void *d, BOOLEAN typed, int dim)
             return omStrDup(s);
           }
 
-        case MODUL_CMD:
         case IDEAL_CMD:
         case MAP_CMD:
+        case MODUL_CMD:
+        case SMATRIX_CMD:
           s= iiStringMatrix((matrix)d,dim, currRing);
           if (typed)
           {
             char* ns = (char*) omAlloc(strlen(s) + 10);
-            sprintf(ns, "%s(%s)", (t/*Typ()*/==MODUL_CMD ? "module" : "ideal"), s);
+            if ((t/*Typ()*/==IDEAL_CMD)||(t==MAP_CMD))
+              sprintf(ns, "ideal(%s)", s);
+            else /*MODUL_CMD, SMATRIX_CMD */
+              sprintf(ns, "module(%s)", s);
+            omFree(s);
             omCheckAddr(ns);
             return ns;
           }
-          return omStrDup(s);
+          return s;
 
         case INTVEC_CMD:
         case INTMAT_CMD:
@@ -1056,6 +1072,7 @@ int  sleftv::Typ()
     case IDEAL_CMD:
     case MATRIX_CMD:
     case MAP_CMD:
+    case SMATRIX_CMD:
       r=POLY_CMD;
       break;
     case MODUL_CMD:
@@ -1131,13 +1148,19 @@ int  sleftv::LTyp()
 }
 
 #ifdef SINGULAR_4_2
-static snumber2 iiNumber2Data[4];
-static int iiCmatrix_index=0;
+STATIC_VAR snumber2 iiNumber2Data[4];
+STATIC_VAR int iiCmatrix_index=0;
 #endif
 void * sleftv::Data()
 {
   if ((rtyp!=IDHDL) && iiCheckRing(rtyp))
      return NULL;
+  if (Sy_inset(FLAG_OTHER_RING,flag))
+  {
+     flag&=~Sy_bit(FLAG_OTHER_RING);
+     WerrorS("object from another ring");
+     return NULL;
+  }
   if (e==NULL)
   {
     switch (rtyp)
@@ -1169,7 +1192,7 @@ void * sleftv::Data()
           return (void *)A->qideal->m[0];
         }
         else
-          return (void *)currRing->cf->nNULL;
+          return (void *)nInit(0);
 
       case VNOETHER:   return (void *) (currRing->ppNoether);
       case IDHDL:
@@ -1281,6 +1304,45 @@ void * sleftv::Data()
         r=(char *)I->m[index-1];
       break;
     }
+    case SMATRIX_CMD:
+    {
+      ideal I=(ideal)d;
+      int c;
+      sleftv tmp;
+      tmp.Init();
+      tmp.rtyp=POLY_CMD;
+      if ((index>0)&& (index<=I->rank)
+      && (e->next!=NULL)
+      && ((c=e->next->start)>0) &&(c<=IDELEMS(I)))
+      {
+        r=(char*)SMATELEM(I,index-1,c-1,currRing);
+      }
+      else
+      {
+        r=NULL;
+      }
+      tmp.data=r;
+      if ((rtyp==IDHDL)||(rtyp==SMATRIX_CMD))
+      {
+        tmp.next=next; next=NULL;
+        d=NULL;
+        CleanUp();
+        memcpy(this,&tmp,sizeof(tmp));
+      }
+      // and, remember, r is also the result...
+      else
+      {
+        // ???
+        // here we still have a memory leak...
+        // example: list L="123","456";
+        // L[1][2];
+        // therefore, it should never happen:
+        assume(0);
+        // but if it happens: here is the temporary fix:
+        // omMarkAsStaticAddr(r);
+      }
+      break;
+    }
     case STRING_CMD:
     {
       // this was a memory leak
@@ -1533,48 +1595,47 @@ void syMake(leftv v,const char * id, package pa)
   if (siq<=0)
 #endif
   {
-    if (!isdigit(id[0]))
+    if (strcmp(id,"basering")==0)
     {
-      if (strcmp(id,"basering")==0)
+      if (currRingHdl!=NULL)
       {
-        if (currRingHdl!=NULL)
-        {
-          if (id!=IDID(currRingHdl)) omFreeBinAddr((ADDRESS)id);
-          h=currRingHdl;
-          goto id_found;
-        }
-        else
-        {
-          v->name = id;
-          return; /* undefined */
-        }
-      }
-      else if (strcmp(id,"Current")==0)
-      {
-        if (currPackHdl!=NULL)
-        {
-          omFreeBinAddr((ADDRESS)id);
-          h=currPackHdl;
-          goto id_found;
-        }
-        else
-        {
-          v->name = id;
-          return; /* undefined */
-        }
-      }
-      if(v->req_packhdl!=currPack)
-      {
-        h=v->req_packhdl->idroot->get(id,myynest);
-      }
-      else
-      h=ggetid(id);
-      /* 3) existing identifier, local */
-      if ((h!=NULL) && (IDLEV(h)==myynest))
-      {
-        if (id!=IDID(h)) omFreeBinAddr((ADDRESS)id); /*assume strlen(id) <1000 */
+        if (id!=IDID(currRingHdl)) omFreeBinAddr((ADDRESS)id);
+        h=currRingHdl;
         goto id_found;
       }
+      else
+      {
+        v->name = id;
+        return; /* undefined */
+      }
+    }
+    else if (strcmp(id,"Current")==0)
+    {
+      if (currPackHdl!=NULL)
+      {
+        omFreeBinAddr((ADDRESS)id);
+        h=currPackHdl;
+        goto id_found;
+      }
+      else
+      {
+        v->name = id;
+        return; /* undefined */
+      }
+    }
+    if(v->req_packhdl!=currPack)
+    {
+      h=v->req_packhdl->idroot->get(id,myynest);
+    }
+    else
+    {
+      h=ggetid(id);
+    }
+    /* 3) existing identifier, local */
+    if ((h!=NULL) && (IDLEV(h)==myynest))
+    {
+      if (id!=IDID(h)) omFreeBinAddr((ADDRESS)id); /*assume strlen(id) <1000 */
+      goto id_found;
     }
     if (yyInRingConstruction)
     {
@@ -1647,9 +1708,18 @@ void syMake(leftv v,const char * id, package pa)
         }
         else
         {
+          v->name = id;
+        #ifdef HAVE_SHIFTBBA
+          if ((currRing->isLPring!=0)
+          && (p_Totaldegree(p,currRing)>1))
+          {
+            p_LmDelete(&p,currRing);
+            /* v->rtyp = UNKNOWN; - already set */
+            return; /* error, report "unknown id" */
+          }
+        #endif
           v->data = p;
           v->rtyp = POLY_CMD;
-          v->name = id;
         }
         return;
       }
@@ -1668,12 +1738,7 @@ void syMake(leftv v,const char * id, package pa)
         {
           v->data = (void *)nInit(0);
           v->rtyp = NUMBER_CMD;
-          #ifdef HAVE_PLURAL
-          // in this case we may have monomials equal to 0 in p_Read
-          v->name = id;
-          #else
           omFreeBinAddr((ADDRESS)id);
-          #endif
         }
         else
         if (pIsConstant(p))
@@ -1757,6 +1822,89 @@ id_found: // we have an id (in h) found, to set the data in from h
   currRingHdl=save_ring;
 }
 
+void syMakeMonom(leftv v,const char * id)
+{
+  if (!isdigit(id[0]))
+  {
+    Print("non-digit:%s\n",id);
+  }
+  /* resolv an identifier: (to DEF_CMD, if siq>0)
+  * 6) monom (resp. number), local ring
+  * 7) monom (resp. number), non-local ring
+  * 10) everything else is of type 0
+  */
+#ifdef TEST
+  if ((*id<' ')||(*id>(char)126))
+  {
+    Print("wrong id :%s:\n",id);
+  }
+#endif
+  idhdl save_ring=currRingHdl;
+  v->Init();
+  v->req_packhdl = currPack;
+#ifdef SIQ
+  if (siq<=0)
+#endif
+  {
+    /* 6. local ring: number/poly */
+    BOOLEAN ok=FALSE;
+    poly p = pmInit(id,ok);
+    if (ok)
+    {
+      if (p==NULL)
+      {
+        v->data = (void *)nInit(0);
+        v->rtyp = NUMBER_CMD;
+        #ifdef HAVE_PLURAL
+        // in this case we may have monomials equal to 0 in p_Read
+        if (rIsPluralRing(currRing)) v->name = omStrDup(id);
+        #endif
+      }
+      else if (pIsConstant(p))
+      {
+        v->data = pGetCoeff(p);
+        pGetCoeff(p)=NULL;
+        pLmFree(p);
+        v->rtyp = NUMBER_CMD;
+      }
+      else
+      {
+        v->name = omStrDup(id);
+        #ifdef HAVE_SHIFTBBA
+        if ((currRing->isLPring!=0)
+        && (p_Totaldegree(p,currRing)>1))
+        {
+          p_LmDelete(&p,currRing);
+          /* v->rtyp = UNKNOWN; - already set */
+          return; /* error, report "unknown id" */
+        }
+        #endif
+        v->data = p;
+        v->rtyp = POLY_CMD;
+      }
+      return;
+    }
+  }
+#ifdef SIQ
+  else
+  {
+    v->rtyp=DEF_CMD;
+  }
+#endif
+  /* 9: _ */
+  if (strcmp(id,"_")==0)
+  {
+    v->Copy(&sLastPrinted);
+  }
+  else
+  {
+    /* 10: everything else */
+    /* v->rtyp = UNKNOWN;*/
+    v->name = omStrDup(id);
+  }
+  currRingHdl=save_ring;
+}
+
 int sleftv::Eval()
 {
   BOOLEAN nok=FALSE;
@@ -1791,7 +1939,7 @@ int sleftv::Eval()
           if (!nok)
           {
             memcpy(this,&iiRETURNEXPR,sizeof(sleftv));
-            memset(&iiRETURNEXPR,0,sizeof(sleftv));
+            iiRETURNEXPR.Init();
           }
         }
       }
@@ -1836,7 +1984,7 @@ int sleftv::Eval()
                     omCheckAddr((ADDRESS)d->arg1.name));
           if (!nok)
           {
-            memset(&d->arg1,0,sizeof(sleftv));
+            d->arg1.Init();
             this->CleanUp();
             rtyp=NONE;
           }
