@@ -9,17 +9,17 @@
 
 #include <string.h>
 
-#include <polys/monomials/ring.h>
+#include "polys/monomials/ring.h"
 
-//#include <Singular/ipid.h>
+//#include "Singular/ipid.h"
 
-#include <kernel/mod2.h>
-#include <Singular/grammar.h>
-#include <Singular/tok.h>
-#include <Singular/attrib.h>
-#include <Singular/fevoices.h> /* for sNoName_fe*/
+#include "kernel/mod2.h"
+#include "Singular/grammar.h"
+#include "Singular/tok.h"
+#include "Singular/attrib.h"
+#include "Singular/fevoices.h" /* for sNoName_fe*/
 
-typedef enum { LANG_NONE, LANG_TOP, LANG_SINGULAR, LANG_C, LANG_MAX} language_defs;
+typedef enum { LANG_NONE, LANG_TOP, LANG_SINGULAR, LANG_C, LANG_MIX, LANG_MAX} language_defs;
 class proc_singular
 {
 public:
@@ -73,7 +73,7 @@ struct _ssubexpr
 
 typedef struct _ssubexpr *Subexpr;
 
-extern BOOLEAN siq;
+EXTERN_VAR BOOLEAN siq;
 
 class sleftv;
 typedef sleftv * leftv;
@@ -140,9 +140,10 @@ class sleftv
 };
 
 inline BOOLEAN RingDependend(int t) { return (BEGIN_RING<t)&&(t<END_RING); }
-extern sleftv sLastPrinted;
+EXTERN_INST_VAR sleftv sLastPrinted;
 
 void syMake(leftv v,const char * name, package pa = NULL);
+void syMakeMonom(leftv v,const char * name);
 BOOLEAN assumeStdFlag(leftv h);
 
 inline procinfov piCopy(procinfov pi)
@@ -170,9 +171,9 @@ class libstack
 };
 #endif /* HAVE_LIBPARSER */
 
-extern omBin sSubexpr_bin;
-extern omBin procinfo_bin;
-extern omBin libstack_bin;
+EXTERN_VAR omBin sSubexpr_bin;
+EXTERN_VAR omBin procinfo_bin;
+EXTERN_VAR omBin libstack_bin;
 
 void s_internalDelete(const int t,  void *d, const ring r);
 
