@@ -28,9 +28,10 @@ public:
     static int gettype () { return currenttype; }
     static void settype ( int type )
     {
-      ASSERT( type==FiniteFieldDomain || type==GaloisFieldDomain || type==IntegerDomain || type==RationalDomain, "illegal basic domain!" );
+      ASSERT( type==FiniteFieldDomain || type==GaloisFieldDomain || type==IntegerDomain || type==RationalDomain || type==PrimePowerDomain, "illegal basic domain!" );
       currenttype = type;
     };
+    static InternalCF * basic ( int value );
     static InternalCF * basic ( long value );
     static InternalCF * basic ( int type, long value );
     static InternalCF * basic ( const char * str );
@@ -45,5 +46,4 @@ public:
 };
 
 void getmpi ( InternalCF * value, mpz_t mpi);
-
 #endif /* ! INCL_CF_FACTORY_H */
