@@ -40,7 +40,7 @@
  * replacevar_between().
  *
 **/
-static Variable sv_x1, sv_x2;
+STATIC_INST_VAR Variable sv_x1, sv_x2;
 
 /** static void swapvar_between ( const CanonicalForm & f, CanonicalForm & result, const CanonicalForm & term, int expx2 )
  *
@@ -624,8 +624,7 @@ size ( const CanonicalForm & f, const Variable & v )
  * Returns one if f is in an coefficient domain.
  *
 **/
-int
-size ( const CanonicalForm & f )
+int size ( const CanonicalForm & f )
 {
     if ( f.inCoeffDomain() )
         return 1;
@@ -639,6 +638,21 @@ size ( const CanonicalForm & f )
     }
 }
 
+int size_maxexp ( const CanonicalForm & f, int& maxexp )
+{
+    if ( f.inCoeffDomain() )
+        return 1;
+    else
+    {
+        if (f.degree()>maxexp) maxexp=f.degree();
+        int result = 0;
+        CFIterator i;
+        for ( i = f; i.hasTerms(); i++ )
+            result += size_maxexp( i.coeff(), maxexp );
+        return result;
+    }
+}
+
 /** polynomials in M.mvar() are considered coefficients
  *  M univariate monic polynomial
  *  the coefficients of f are reduced modulo M
