@@ -4,7 +4,7 @@
  *  Created on: Sep 29, 2010
  *      Author: anders
  */
-
+#include <stddef.h>
 #include "gfanlib_zcone.h"
 
 #include <vector>
@@ -14,14 +14,14 @@
 #include <config.h>
 extern "C"
 {
-#ifdef HAVE_CDD_SETOPER_H
-#include <cdd/setoper.h>
-#include <cdd/cdd.h>
-#else
 #ifdef HAVE_CDDLIB_SETOPER_H
 #include <cddlib/setoper.h>
 #include <cddlib/cdd.h>
 #else
+#ifdef HAVE_CDD_SETOPER_H
+#include <cdd/setoper.h>
+#include <cdd/cdd.h>
+#else
 #include <setoper.h>
 #include <cdd.h>
 #endif //HAVE_CDDLIB_SETOPER_H
@@ -36,8 +36,8 @@ namespace gfan{
         }
         void initializeCddlibIfRequired() // calling this frequently will cause memory leaks because deinitialisation is not possible with old versions of cddlib.
         {
-	  if (dd_statStartTime==0)
-	  {
+          if (dd_statStartTime==0)
+          {
                 dd_set_global_constants();
           }
         }
@@ -664,7 +664,7 @@ _L99:
 
 };
 
-LpSolver lpSolver;
+INST_VAR LpSolver lpSolver;
 
 bool ZCone::isInStateMinimum(int s)const
 {
