@@ -10,8 +10,6 @@
 
 #ifdef HAVE_PLURAL
 
-
-#include "omalloc/omalloc.h"
 #include "misc/options.h"
 #include "misc/intvec.h"
 
@@ -101,7 +99,7 @@ int redGrFirst (LObject* h,kStrategy strat)
 #endif
       if ((*h).p == NULL)
       {
-        if (h->lcm!=NULL) p_LmFree((*h).lcm, currRing);
+        kDeleteLcm(h);
         return 0;
       }
       if (TEST_OPT_INTSTRATEGY)
@@ -285,7 +283,7 @@ int redGrRatGB (LObject* h,kStrategy strat)
 #endif
         if ((*h).p == NULL)
         {
-          if (h->lcm!=NULL) p_LmFree((*h).lcm, currRing);
+          kDeleteLcm(h);
           return 0;
         }
         ratGB_divide_out((*h).p);
@@ -398,8 +396,7 @@ static int nc_redHomog (LObject* h,kStrategy strat)
       if ((*h).p == NULL)
       {
         if (TEST_OPT_DEBUG) PrintS(" to 0\n");
-        if (h->lcm!=NULL) pLmFree((*h).lcm);
-        (*h).lcm=NULL;
+        kDeleteLcm(h);
         return 0;
       }
 /*
@@ -465,8 +462,7 @@ static int nc_redHomog0 (LObject* h,kStrategy strat)
       if ((*h).p == NULL)
       {
         if (TEST_OPT_DEBUG) PrintS(" to 0\n");
-        if (h->lcm!=NULL) pLmFree((*h).lcm);
-        (*h).lcm=NULL;
+        kDeleteLcm(h);
         return 0;
       }
       else
@@ -547,8 +543,7 @@ static int nc_redLazy (LObject* h,kStrategy strat)
       if ((*h).p == NULL)
       {
         if (TEST_OPT_DEBUG) PrintS(" to 0\n");
-        if (h->lcm!=NULL) pLmFree((*h).lcm);
-        (*h).lcm=NULL;
+        kDeleteLcm(h);
         return 0;
       }
 //      else if (strat->syzComp)
@@ -724,8 +719,7 @@ static int nc_redHoney (LObject*  h,kStrategy strat)
       }
       if ((*h).p == NULL)
       {
-        if (h->lcm!=NULL) pLmFree((*h).lcm);
-        (*h).lcm=NULL;
+        kDeleteLcm(h);
         return 0;
       }
       if (TEST_OPT_INTSTRATEGY)
@@ -869,11 +863,7 @@ static int nc_redBest (LObject*  h,kStrategy strat)
               {
                 pLmFree(p);
                 pDelete(&((*h).p));
-                if (h->lcm!=NULL)
-                {
-                  pLmFree((*h).lcm);
-                  (*h).lcm=NULL;
-                }
+                kDeleteLcm(h);
                 return 0;
               }
               else if (pLmCmp(ph,p) == -1)
@@ -894,12 +884,7 @@ static int nc_redBest (LObject*  h,kStrategy strat)
       }
       else
       {
-        if (h->lcm!=NULL)
-        {
-          pLmFree((*h).lcm);
-          (*h).lcm=NULL;
-        }
-        (*h).p = NULL;
+        kDeleteLcm(h);
         return 0;
       }
       if (strat->honey && currRing->pLexOrder)
@@ -1065,7 +1050,7 @@ ideal k_gnc_gr_bba(const ideal F, const ideal Q, const intvec *, const intvec *,
 #endif
 #endif
 
-  assume(currRing->OrdSgn != -1); // no mora!!! it terminates only for global ordering!!! (?)
+  assume(currRing->OrdSgn != -1); // no mora!!! it terminates only for global ordering!!! (?)// alternate algebra?
 
   // intvec *w=NULL;
   // intvec *hilb=NULL;
@@ -1244,11 +1229,8 @@ ideal k_gnc_gr_bba(const ideal F, const ideal Q, const intvec *, const intvec *,
         }
 //      if (hilb!=NULL) khCheck(Q,w,hilb,hilbeledeg,hilbcount,strat);
       }
-      if (strat->P.lcm!=NULL) pLmFree(strat->P.lcm);
+      kDeleteLcm(&strat->P);
     }
-#ifdef KDEBUG
-    strat->P.lcm=NULL;
-#endif
     //kTest(strat);
   }
   if (TEST_OPT_DEBUG) messageSets(strat);
@@ -1289,12 +1271,6 @@ ideal k_gnc_gr_bba(const ideal F, const ideal Q, const intvec *, const intvec *,
   if (Q!=NULL) updateResult(strat->Shdl,Q,strat);
 
 
-#ifdef PDEBUG
-/* for counting number of pairs [enterL] in Plural */
-/*   extern int zaehler; */
-/*   Print("Total pairs considered:%d\n",zaehler); zaehler=0; */
-#endif /*PDEBUG*/
-
 #if MYTEST
   PrintS("</gnc_gr_bba>\n");
 #endif
@@ -1306,12 +1282,14 @@ ideal k_gnc_gr_bba(const ideal F, const ideal Q, const intvec *, const intvec *,
 
 ideal k_gnc_gr_mora(const ideal F, const ideal Q, const intvec *, const intvec *, kStrategy strat, const ring _currRing)
 {
-#ifndef SING_NDEBUG
-  // Not yet!
-  WarnS("Sorry, non-commutative mora is not yet implemented!");
-#endif
-
-  return gnc_gr_bba(F, Q, NULL, NULL, strat, _currRing);
+  if ((ncRingType(_currRing)==nc_skew)
+  || id_HomIdeal(F,Q,_currRing))
+    return gnc_gr_bba(F, Q, NULL, NULL, strat, _currRing);
+  else
+  {
+    WerrorS("not implemented: std for inhomogeneous ideasl in local orderings");
+    return NULL;
+  }
 }
 
 #endif
