@@ -19,12 +19,15 @@
 
 #include "omalloc/omalloc.h"
 #include "misc/options.h"
-
 #include "polys/monomials/p_polys.h"
 #include "polys/kbuckets.h"
 
 #include "kernel/polys.h"
 
+#ifdef HAVE_SHIFTBBA
+#include "polys/shiftop.h"
+#endif
+
 
 #define HAVE_TAIL_BIN
 // This doesn't really work, fixme, if necessary
@@ -119,12 +122,32 @@ KINLINE void sTObject::Set(poly p_in, ring r)
   if (r != currRing)
   {
     assume(r == tailRing);
-    p_Test(p_in, r);
+#ifdef HAVE_SHIFTBBA
+    if (r->isLPring)
+    {
+      shift = si_max(p_mFirstVblock(p_in, r) - 1, 0);
+      if (!shift) p_Test(p_in, r);
+    }
+    else
+#endif
+    {
+      p_Test(p_in, r);
+    }
     t_p = p_in;
   }
   else
   {
-    p_Test(p_in, currRing);
+#ifdef HAVE_SHIFTBBA
+    if (currRing->isLPring)
+    {
+      shift = si_max(p_mFirstVblock(p_in, currRing) - 1, 0);
+      if (!shift) p_Test(p_in, currRing);
+    }
+    else
+#endif
+    {
+      p_Test(p_in, currRing);
+    }
     p = p_in;
   }
   pLength=::pLength(p_in);
@@ -141,7 +164,17 @@ KINLINE void sTObject::Set(poly p_in, ring c_r, ring t_r)
   if (c_r != t_r)
   {
     assume(c_r == currRing && t_r == tailRing);
-    p_Test(p_in, currRing);
+#ifdef HAVE_SHIFTBBA
+    if (c_r->isLPring)
+    {
+      shift = si_max(p_mFirstVblock(p_in, c_r) - 1, 0);
+      if (!shift) p_Test(p_in, currRing);
+    }
+    else
+#endif
+    {
+      p_Test(p_in, currRing);
+    }
     p = p_in;
     pLength=::pLength(p_in);
   }
@@ -333,6 +366,13 @@ KINLINE void sLObject::Normalize()
   {
     pNormalize(p);
   }
+  if (bucket!=NULL) kBucketNormalize(bucket);
+}
+
+KINLINE void sLObject::CanonicalizeP()
+{
+  if (bucket != NULL)
+    kBucketCanonicalize(bucket);
 }
 
 KINLINE void sLObject::HeadNormalize()
@@ -429,7 +469,8 @@ KINLINE long sTObject::SetDegStuffReturnLDeg()
 KINLINE void  sTObject::pCleardenom()
 {
   assume(p != NULL);
-  if (TEST_OPT_CONTENTSB)
+  BOOLEAN is_ring=rField_is_Ring(currRing);
+  if ((TEST_OPT_CONTENTSB) && (!is_ring))
   {
     number n;
     if (t_p != NULL)
@@ -450,6 +491,47 @@ KINLINE void  sTObject::pCleardenom()
     }
     nDelete(&n);
   }
+  #ifdef HAVE_RINGS
+  else if (is_ring)
+  {
+    number c;
+    if (t_p != NULL)
+      c=pGetCoeff(t_p);
+    else
+      c=pGetCoeff(p);
+    const coeffs C=tailRing->cf;
+    number u=n_GetUnit(c,C);
+
+    if (t_p != NULL)
+    {
+      if (!n_IsOne(u,C))
+      {
+        number uInv = n_Invers(u, C);
+        t_p=p_Mult_nn(t_p,uInv,tailRing);
+        n_Delete(&uInv,C);
+      }
+      if(!n_GreaterZero(pGetCoeff(t_p),tailRing->cf))
+      {
+        t_p = p_Neg(t_p,tailRing);
+      }
+      pSetCoeff0(p, pGetCoeff(t_p));
+    }
+    else
+    {
+      if (!n_IsOne(u,C))
+      {
+        number uInv = n_Invers(u, C);
+        p=p_Mult_nn(p,uInv,tailRing);
+        n_Delete(&uInv,C);
+      }
+      if(!n_GreaterZero(pGetCoeff(p),tailRing->cf))
+      {
+        p = p_Neg(p,tailRing);
+      }
+    }
+    n_Delete(&u,C);
+  }
+  #endif
   else
   {
     if (t_p != NULL)
@@ -464,6 +546,28 @@ KINLINE void  sTObject::pCleardenom()
   }
 }
 
+KINLINE void  sTObject::pContent()
+{
+  assume(p != NULL);
+  if (t_p != NULL)
+  {
+    p_SimpleContent(t_p, 1, tailRing);
+    if (!n_GreaterZero(pGetCoeff(t_p),tailRing->cf))
+    {
+      t_p=p_Neg (t_p,tailRing);
+    }
+    pSetCoeff0(p, pGetCoeff(t_p));
+  }
+  else
+  {
+    p_SimpleContent(p, 1, currRing);
+    if (!n_GreaterZero(pGetCoeff(p),currRing->cf))
+    {
+      p=p_Neg (p,currRing);
+    }
+  }
+}
+
 KINLINE void sTObject::pNorm() // pNorm seems to be a _bad_ method name...
 {
   assume(p != NULL);
@@ -489,7 +593,8 @@ KINLINE void sLObject::Clear()
   sTObject::Clear();
   sev = 0;
 }
-// Initialization
+
+
 KINLINE void sLObject::Delete()
 {
   sTObject::Delete();
@@ -639,21 +744,6 @@ KINLINE poly sLObject::LmExtractAndIter()
   return ret;
 }
 
-KINLINE poly sLObject::CanonicalizeP()
-{
-  //kTest_L(this);
-  int i = -1;
-
-  if (bucket != NULL)
-    i = kBucketCanonicalize(bucket);
-
-  if (p == NULL)
-    p = k_LmInit_tailRing_2_currRing(t_p, tailRing);
-
-  if (i >= 0) pNext(p) = bucket->buckets[i];
-  return p;
-}
-
 KINLINE poly sLObject::GetTP()
 {
   //kTest_L(this);
@@ -676,10 +766,10 @@ KINLINE poly sLObject::GetP(omBin lmBin)
   if (p == NULL)
   {
     p = k_LmInit_tailRing_2_currRing(t_p, tailRing,
-                                     (lmBin!=NULL?lmBin:currRing->PolyBin));
+                                     ((lmBin!=NULL)?lmBin:currRing->PolyBin));
     FDeg = pFDeg();
   }
-  else if (lmBin != NULL && lmBin != currRing->PolyBin)
+  else if ((lmBin != NULL) && (lmBin != currRing->PolyBin))
   {
     p = p_LmShallowCopyDelete(p, currRing);
     FDeg = pFDeg();
@@ -1027,6 +1117,44 @@ KINLINE void k_GetStrongLeadTerms(const poly p1, const poly p2, const ring leadR
  * Misc things
  *
  ***************************************************************/
+KINLINE int ksReducePolyTailLC_Z(LObject* PR, TObject* PW, LObject* Red)
+{
+  BOOLEAN ret;
+  number mult, rest;
+  TObject red = *PW;
+  red.Copy();
+  rest = n_QuotRem(pGetCoeff(Red->p), pGetCoeff(red.p),
+          &mult, currRing->cf);
+  red.Mult_nn(rest);
+
+  assume(PR->GetLmCurrRing() != red.GetLmCurrRing());
+  ret = ksReducePolyLC(Red, &red, NULL, &mult);
+  red.Delete();
+  red.Clear();
+
+  return ret;
+}
+
+KINLINE int ksReducePolyTail_Z(LObject* PR, TObject* PW, LObject* Red)
+{
+  BOOLEAN ret;
+  number coef;
+
+  assume(PR->GetLmCurrRing() != PW->GetLmCurrRing());
+  ret = ksReducePoly(Red, PW, NULL, &coef);
+
+  if (!ret)
+  {
+    if (! n_IsOne(coef, currRing->cf))
+    {
+      PR->Mult_nn(coef);
+      // HANNES: mark for Normalize
+    }
+    n_Delete(&coef, currRing->cf);
+  }
+  return ret;
+}
+
 KINLINE int ksReducePolyTail(LObject* PR, TObject* PW, LObject* Red)
 {
   BOOLEAN ret;
@@ -1105,6 +1233,11 @@ KINLINE poly redtailBbaBound (poly p,int pos,kStrategy strat,int bound,BOOLEAN n
 }
 
 #ifdef HAVE_RINGS
+KINLINE poly redtailBba_Ring (poly p,int pos,kStrategy strat)
+{
+  LObject L(p, currRing, strat->tailRing);
+  return redtailBba_Ring(&L, pos, strat);
+}
 KINLINE poly redtailBba_Z (poly p,int pos,kStrategy strat)
 {
   LObject L(p, currRing, strat->tailRing);
