@@ -19,10 +19,13 @@
 #ifdef HAVE_RINGS
 #include "kernel/polys.h"
 #endif
+#ifdef HAVE_SHIFTBBA
+#include "polys/shiftop.h"
+#endif
 
 #ifdef KDEBUG
-int red_count = 0;
-int create_count = 0;
+VAR int red_count = 0;
+VAR int create_count = 0;
 // define this if reductions are reported on TEST_OPT_DEBUG
 #define TEST_OPT_DEBUG_RED
 #endif
@@ -38,7 +41,7 @@ int create_count = 0;
  *         2: cannot change tailRing: strat==NULL
  *
  ***************************************************************/
-int ksReducePoly(LObject* PR,
+int ksReducePolyZ(LObject* PR,
                  TObject* PW,
                  poly spNoether,
                  number *coef,
@@ -58,8 +61,155 @@ int ksReducePoly(LObject* PR,
 #endif
   int ret = 0;
   ring tailRing = PR->tailRing;
-  kTest_L(PR);
-  kTest_T(PW);
+  if (strat!=NULL)
+  {
+    kTest_L(PR,strat);
+    kTest_T(PW,strat);
+  }
+  poly p1 = PR->GetLmTailRing();   // p2 | p1
+  poly p2 = PW->GetLmTailRing();   // i.e. will reduce p1 with p2; lm = LT(p1) / LM(p2)
+  poly t2 = pNext(p2), lm = p1;    // t2 = p2 - LT(p2); really compute P = LC(p2)*p1 - LT(p1)/LM(p2)*p2
+  assume(p1 != NULL && p2 != NULL);// Attention, we have rings and there LC(p2) and LC(p1) are special
+  p_CheckPolyRing(p1, tailRing);
+  p_CheckPolyRing(p2, tailRing);
+
+  pAssume1(p2 != NULL && p1 != NULL &&
+           p_DivisibleBy(p2,  p1, tailRing));
+
+  pAssume1(p_GetComp(p1, tailRing) == p_GetComp(p2, tailRing) ||
+           (p_GetComp(p2, tailRing) == 0 &&
+            p_MaxComp(pNext(p2),tailRing) == 0));
+
+#ifdef HAVE_PLURAL
+  if (rIsPluralRing(currRing))
+  {
+    // for the time being: we know currRing==strat->tailRing
+    // no exp-bound checking needed
+    // (only needed if exp-bound(tailring)<exp-b(currRing))
+    if (PR->bucket!=NULL)  nc_kBucketPolyRed_Z(PR->bucket, p2,coef);
+    else
+    {
+      poly _p = (PR->t_p != NULL ? PR->t_p : PR->p);
+      assume(_p != NULL);
+      nc_PolyPolyRed(_p, p2,coef, currRing);
+      if (PR->t_p!=NULL) PR->t_p=_p; else PR->p=_p;
+      PR->pLength=0; // usually not used, GetpLength re-computes it if needed
+    }
+    return 0;
+  }
+#endif
+
+  if (t2==NULL)           // Divisor is just one term, therefore it will
+  {                       // just cancel the leading term
+    // adjust lead coefficient if needed
+    if (! n_IsOne(pGetCoeff(p2), tailRing->cf))
+    {
+      number bn = pGetCoeff(lm);
+      number an = pGetCoeff(p2);
+      int ct = ksCheckCoeff(&an, &bn, tailRing->cf);    // Calculate special LC
+      p_SetCoeff(lm, bn, tailRing);
+      if ((ct == 0) || (ct == 2))
+      PR->Tail_Mult_nn(an);
+      if (coef != NULL) *coef = an;
+      else n_Delete(&an, tailRing->cf);
+    }
+    PR->LmDeleteAndIter();
+    if (coef != NULL) *coef = n_Init(1, tailRing->cf);
+    return 0;
+  }
+
+  p_ExpVectorSub(lm, p2, tailRing); // Calculate the Monomial we must multiply to p2
+
+  //if (tailRing != currRing)
+  {
+    // check that reduction does not violate exp bound
+    while (PW->max_exp != NULL && !p_LmExpVectorAddIsOk(lm, PW->max_exp, tailRing))
+    {
+      // undo changes of lm
+      p_ExpVectorAdd(lm, p2, tailRing);
+      if (strat == NULL) return 2;
+      if (! kStratChangeTailRing(strat, PR, PW)) return -1;
+      tailRing = strat->tailRing;
+      p1 = PR->GetLmTailRing();
+      p2 = PW->GetLmTailRing();
+      t2 = pNext(p2);
+      lm = p1;
+      p_ExpVectorSub(lm, p2, tailRing);
+      ret = 1;
+    }
+  }
+
+#ifdef HAVE_SHIFTBBA
+  poly lmRight;
+  if (tailRing->isLPring)
+  {
+    assume(PR->shift == 0);
+    assume(PW->shift == si_max(p_mFirstVblock(PW->p, tailRing) - 1, 0));
+    k_SplitFrame(lm, lmRight, PW->shift + 1, tailRing);
+  }
+#endif
+
+  // take care of coef buisness
+  if (! n_IsOne(pGetCoeff(p2), tailRing->cf))
+  {
+    number bn = pGetCoeff(lm);
+    number an = pGetCoeff(p2);
+    int ct = ksCheckCoeff(&an, &bn, tailRing->cf);    // Calculate special LC
+    p_SetCoeff(lm, bn, tailRing);
+    if ((ct == 0) || (ct == 2))
+      PR->Tail_Mult_nn(an);
+    if (coef != NULL) *coef = an;
+    else n_Delete(&an, tailRing->cf);
+  }
+  else
+  {
+    if (coef != NULL) *coef = n_Init(1, tailRing->cf);
+  }
+
+
+  // and finally,
+#ifdef HAVE_SHIFTBBA
+  if (tailRing->isLPring)
+  {
+    PR->Tail_Minus_mm_Mult_qq(lm, tailRing->p_Procs->pp_Mult_mm(t2, lmRight, tailRing), pLength(t2), spNoether);
+  }
+  else
+#endif
+  {
+    PR->Tail_Minus_mm_Mult_qq(lm, t2, pLength(t2) /*PW->GetpLength() - 1*/, spNoether);
+  }
+  assume(PW->GetpLength() == pLength(PW->p != NULL ? PW->p : PW->t_p));
+  PR->LmDeleteAndIter();
+
+  return ret;
+}
+
+int ksReducePoly(LObject* PR,
+                 TObject* PW,
+                 poly spNoether,
+                 number *coef,
+                 poly *mon,
+                 kStrategy strat)
+{
+#ifdef KDEBUG
+  red_count++;
+#ifdef TEST_OPT_DEBUG_RED
+//  if (TEST_OPT_DEBUG)
+//  {
+//    Print("Red %d:", red_count); PR->wrp(); Print(" with:");
+//    PW->wrp();
+//    //printf("\necart(PR)-ecart(PW): %i\n",PR->ecart-PW->ecart);
+//    //pWrite(PR->p);
+//  }
+#endif
+#endif
+  int ret = 0;
+  ring tailRing = PR->tailRing;
+  if (strat!=NULL)
+  {
+    kTest_L(PR,strat);
+    kTest_T(PW,strat);
+  }
 
   poly p1 = PR->GetLmTailRing();   // p2 | p1
   poly p2 = PW->GetLmTailRing();   // i.e. will reduce p1 with p2; lm = LT(p1) / LM(p2)
@@ -81,7 +231,7 @@ int ksReducePoly(LObject* PR,
     // for the time being: we know currRing==strat->tailRing
     // no exp-bound checking needed
     // (only needed if exp-bound(tailring)<exp-b(currRing))
-    if (PR->bucket!=NULL)  nc_kBucketPolyRed(PR->bucket, p2,coef);
+    if (PR->bucket!=NULL)  nc_kBucketPolyRed_Z(PR->bucket, p2,coef);
     else
     {
       poly _p = (PR->t_p != NULL ? PR->t_p : PR->p);
@@ -94,7 +244,7 @@ int ksReducePoly(LObject* PR,
   }
 #endif
 
-  if (t2==NULL)           // Divisor is just one term, therefore it will
+  if ((t2==NULL)&&(mon==NULL)) // Divisor is just one term, therefore it will
   {                       // just cancel the leading term
     PR->LmDeleteAndIter();
     if (coef != NULL) *coef = n_Init(1, tailRing->cf);
@@ -122,6 +272,16 @@ int ksReducePoly(LObject* PR,
     }
   }
 
+#ifdef HAVE_SHIFTBBA
+  poly lmRight=NULL;
+  if (tailRing->isLPring)
+  {
+    assume(PR->shift == 0);
+    assume(PW->shift == si_max(p_mFirstVblock(PW->p, tailRing) - 1, 0));
+    k_SplitFrame(lm, lmRight, PW->shift + 1, tailRing);
+  }
+#endif
+
   // take care of coef buisness
   if (! n_IsOne(pGetCoeff(p2), tailRing->cf))
   {
@@ -138,40 +298,32 @@ int ksReducePoly(LObject* PR,
   {
     if (coef != NULL) *coef = n_Init(1, tailRing->cf);
   }
-
+  if(mon!=NULL) *mon=pHead(lm);
 
   // and finally,
-  PR->Tail_Minus_mm_Mult_qq(lm, t2, pLength(t2) /*PW->GetpLength() - 1*/, spNoether);
+#ifdef HAVE_SHIFTBBA
+  if (tailRing->isLPring)
+  {
+    poly tmp=tailRing->p_Procs->pp_Mult_mm(t2, lmRight, tailRing);
+    PR->Tail_Minus_mm_Mult_qq(lm, tmp, pLength(t2), spNoether);
+    p_Delete(&tmp,tailRing);
+    p_Delete(&lm,tailRing);
+    p_Delete(&lmRight,tailRing);
+  }
+  else
+#endif
+  {
+    PR->Tail_Minus_mm_Mult_qq(lm, t2, pLength(t2) /*PW->GetpLength() - 1*/, spNoether);
+  }
   assume(PW->GetpLength() == pLength(PW->p != NULL ? PW->p : PW->t_p));
   PR->LmDeleteAndIter();
 
-  // the following is commented out: shrinking
-#ifdef HAVE_SHIFTBBA_NONEXISTENT
-  if ( (currRing->isLPring) && (!strat->homog) )
-  {
-    // assume? h->p in currRing
-    PR->GetP();
-    poly qq = p_Shrink(PR->p, currRing->isLPring, currRing);
-    PR->Clear(); // does the right things
-    PR->p = qq;
-    PR->t_p = NULL;
-    PR->SetShortExpVector();
-  }
-#endif
-
-#if defined(KDEBUG) && defined(TEST_OPT_DEBUG_RED)
-  if (TEST_OPT_DEBUG)
-  {
-    Print(" to: "); PR->wrp(); Print("\n");
-    //printf("\nt^%i ", PR->ecart);pWrite(pHead(PR->p));
-  }
-#endif
   return ret;
 }
 
-int ksReducePolyBound(LObject* PR,
+#ifdef HAVE_RINGS
+int ksReducePolyGCD(LObject* PR,
                  TObject* PW,
-                 int bound,
                  poly spNoether,
                  number *coef,
                  kStrategy strat)
@@ -179,19 +331,157 @@ int ksReducePolyBound(LObject* PR,
 #ifdef KDEBUG
   red_count++;
 #ifdef TEST_OPT_DEBUG_RED
-  if (TEST_OPT_DEBUG)
-  {
-    Print("Red %d:", red_count); PR->wrp(); Print(" with:");
-    PW->wrp();
-    //printf("\necart(PR)-ecart(PW): %i\n",PR->ecart-PW->ecart);
-    //pWrite(PR->p);
-  }
+//  if (TEST_OPT_DEBUG)
+//  {
+//    Print("Red %d:", red_count); PR->wrp(); Print(" with:");
+//    PW->wrp();
+//    //printf("\necart(PR)-ecart(PW): %i\n",PR->ecart-PW->ecart);
+//    //pWrite(PR->p);
+//  }
 #endif
 #endif
   int ret = 0;
   ring tailRing = PR->tailRing;
-  kTest_L(PR);
-  kTest_T(PW);
+  if (strat!=NULL)
+  {
+    kTest_L(PR,strat);
+    kTest_T(PW,strat);
+  }
+
+  poly p1 = PR->GetLmTailRing();
+  poly p2 = PW->GetLmTailRing();
+  poly t2 = pNext(p2), lm = pOne();
+  assume(p1 != NULL && p2 != NULL);// Attention, we have rings and there LC(p2) and LC(p1) are special
+  p_CheckPolyRing(p1, tailRing);
+  p_CheckPolyRing(p2, tailRing);
+
+  pAssume1(p2 != NULL && p1 != NULL &&
+           p_DivisibleBy(p2,  p1, tailRing));
+
+  pAssume1(p_GetComp(p1, tailRing) == p_GetComp(p2, tailRing) ||
+           (p_GetComp(p2, tailRing) == 0 &&
+            p_MaxComp(pNext(p2),tailRing) == 0));
+
+#ifdef HAVE_PLURAL
+  if (rIsPluralRing(currRing))
+  {
+    // for the time being: we know currRing==strat->tailRing
+    // no exp-bound checking needed
+    // (only needed if exp-bound(tailring)<exp-b(currRing))
+    if (PR->bucket!=NULL)  nc_kBucketPolyRed_Z(PR->bucket, p2,coef);
+    else
+    {
+      poly _p = (PR->t_p != NULL ? PR->t_p : PR->p);
+      assume(_p != NULL);
+      nc_PolyPolyRed(_p, p2,coef, currRing);
+      if (PR->t_p!=NULL) PR->t_p=_p; else PR->p=_p;
+      PR->pLength=0; // usually not used, GetpLength re-computes it if needed
+    }
+    return 0;
+  }
+#endif
+  // check that reduction does not violate exp bound
+  while (PW->max_exp != NULL && !p_LmExpVectorAddIsOk(lm, PW->max_exp, tailRing))
+  {
+    // undo changes of lm
+    p_ExpVectorAdd(lm, p2, tailRing);
+    if (strat == NULL) return 2;
+    if (! kStratChangeTailRing(strat, PR, PW)) return -1;
+    tailRing = strat->tailRing;
+    p1 = PR->GetLmTailRing();
+    p2 = PW->GetLmTailRing();
+    t2 = pNext(p2);
+    lm = p1;
+    p_ExpVectorSub(lm, p2, tailRing);
+    ret = 1;
+  }
+
+#ifdef HAVE_SHIFTBBA
+  poly lmRight;
+  if (tailRing->isLPring)
+  {
+    assume(PR->shift == 0);
+    assume(PW->shift == si_max(p_mFirstVblock(PW->p, tailRing) - 1, 0));
+    k_SplitFrame(lm, lmRight, PW->shift + 1, tailRing);
+  }
+#endif
+
+  number ct, an, bn;
+  // take care of coef buisness
+  if (! n_IsOne(pGetCoeff(p2), tailRing->cf))
+  {
+    ct = n_ExtGcd(pGetCoeff(p1), pGetCoeff(p2), &an, &bn, tailRing->cf);    // Calculate GCD
+    if (n_IsZero(an, tailRing->cf) || n_IsZero(bn, tailRing->cf))
+    {
+      n_Delete(&an, tailRing->cf);
+      n_Delete(&bn, tailRing->cf);
+      n_Delete(&ct, tailRing->cf);
+      return ret;
+    }
+    /* negate bn since we subtract in Tail_Minus_mm_Mult_qq */
+    bn  = n_InpNeg(bn, tailRing->cf);
+    p_SetCoeff(lm, bn, tailRing);
+    p_Test(lm,tailRing);
+    PR->Tail_Mult_nn(an);
+  }
+  else
+  {
+    if (coef != NULL) *coef = n_Init(1, tailRing->cf);
+  }
+
+
+  // and finally,
+#ifdef HAVE_SHIFTBBA
+  if (tailRing->isLPring)
+  {
+    PR->Tail_Minus_mm_Mult_qq(lm, tailRing->p_Procs->pp_Mult_mm(t2, lmRight, tailRing), pLength(t2), spNoether);
+  }
+  else
+#endif
+  {
+    PR->Tail_Minus_mm_Mult_qq(lm, t2, pLength(t2) /*PW->GetpLength() - 1*/, spNoether);
+  }
+  assume(PW->GetpLength() == pLength(PW->p != NULL ? PW->p : PW->t_p));
+  pSetCoeff(PR->p, ct);
+
+  return ret;
+}
+#endif
+
+/* Computes a reduction of the lead coefficient only. We have already tested
+ * that lm(PW) divides lm(PR), but lc(PW) does not divide lc(PR). We have
+ * computed division with remainder on the lead coefficients, parameter
+ * coef is the corresponding multiple for PW we need. The new lead
+ * coefficient, i.e. the remainder of lc division has already been
+ * set before calling this function. We do not drop the lead term at
+ * the end, but keep the adjusted, correct lead term. */
+int ksReducePolyLC(LObject* PR,
+                 TObject* PW,
+                 poly spNoether,
+                 number *coef,
+                 kStrategy strat)
+{
+#ifdef KDEBUG
+  red_count++;
+#ifdef TEST_OPT_DEBUG_RED
+//  if (TEST_OPT_DEBUG)
+//  {
+//    Print("Red %d:", red_count); PR->wrp(); Print(" with:");
+//    PW->wrp();
+//    //printf("\necart(PR)-ecart(PW): %i\n",PR->ecart-PW->ecart);
+//    //pWrite(PR->p);
+//  }
+#endif
+#endif
+  /* printf("PR->P: ");
+   * p_Write(PR->p, currRing, PR->tailRing); */
+  int ret = 0;
+  ring tailRing = PR->tailRing;
+  if (strat!=NULL)
+  {
+    kTest_L(PR,strat);
+    kTest_T(PW,strat);
+  }
 
   poly p1 = PR->GetLmTailRing();   // p2 | p1
   poly p2 = PW->GetLmTailRing();   // i.e. will reduce p1 with p2; lm = LT(p1) / LM(p2)
@@ -213,7 +503,120 @@ int ksReducePolyBound(LObject* PR,
     // for the time being: we know currRing==strat->tailRing
     // no exp-bound checking needed
     // (only needed if exp-bound(tailring)<exp-b(currRing))
-    if (PR->bucket!=NULL)  nc_kBucketPolyRed(PR->bucket, p2,coef);
+    if (PR->bucket!=NULL)  nc_kBucketPolyRed_Z(PR->bucket, p2,coef);
+    else
+    {
+      poly _p = (PR->t_p != NULL ? PR->t_p : PR->p);
+      assume(_p != NULL);
+      nc_PolyPolyRed(_p, p2,coef, currRing);
+      if (PR->t_p!=NULL) PR->t_p=_p; else PR->p=_p;
+      PR->pLength=0; // usually not used, GetpLength re-computes it if needed
+    }
+    return 0;
+  }
+#endif
+
+  p_ExpVectorSub(lm, p2, tailRing); // Calculate the Monomial we must multiply to p2
+  p_SetCoeff(lm, n_Init(1, tailRing->cf), tailRing);
+  while (PW->max_exp != NULL && !p_LmExpVectorAddIsOk(lm, PW->max_exp, tailRing))
+  {
+    // undo changes of lm
+    p_ExpVectorAdd(lm, p2, tailRing);
+    if (strat == NULL) return 2;
+    /* if (! kStratChangeTailRing(strat, PR, PW)) return -1; */
+    tailRing = strat->tailRing;
+    p1 = PR->GetLmTailRing();
+    p2 = PW->GetLmTailRing();
+    t2 = pNext(p2);
+    lm = p1;
+    p_ExpVectorSub(lm, p2, tailRing);
+    ret = 1;
+  }
+
+#ifdef HAVE_SHIFTBBA
+  poly lmRight;
+  if (tailRing->isLPring)
+  {
+    assume(PR->shift == 0);
+    assume(PW->shift == si_max(p_mFirstVblock(PW->p, tailRing) - 1, 0));
+    k_SplitFrame(lm, lmRight, PW->shift + 1, tailRing);
+  }
+#endif
+
+  // and finally,
+#ifdef HAVE_SHIFTBBA
+  if (tailRing->isLPring)
+  {
+    PR->Tail_Minus_mm_Mult_qq(lm, tailRing->p_Procs->pp_Mult_mm(p2, lmRight, tailRing), pLength(p2), spNoether);
+  }
+  else
+#endif
+  {
+    PR->Tail_Minus_mm_Mult_qq(lm, p2, pLength(p2) /*PW->GetpLength() - 1*/, spNoether);
+  }
+  assume(PW->GetpLength() == pLength(PW->p != NULL ? PW->p : PW->t_p));
+
+  PR->LmDeleteAndIter();
+  p_SetCoeff(PR->p, *coef, currRing);
+
+#if defined(KDEBUG) && defined(TEST_OPT_DEBUG_RED)
+  if (TEST_OPT_DEBUG)
+  {
+    Print(" to: "); PR->wrp(); Print("\n");
+    //printf("\nt^%i ", PR->ecart);pWrite(pHead(PR->p));
+  }
+#endif
+  return ret;
+}
+
+int ksReducePolyBound(LObject* PR,
+                 TObject* PW,
+                 int /*bound*/,
+                 poly spNoether,
+                 number *coef,
+                 kStrategy strat)
+{
+#ifdef KDEBUG
+  red_count++;
+#ifdef TEST_OPT_DEBUG_RED
+  if (TEST_OPT_DEBUG)
+  {
+    Print("Red %d:", red_count); PR->wrp(); Print(" with:");
+    PW->wrp();
+    //printf("\necart(PR)-ecart(PW): %i\n",PR->ecart-PW->ecart);
+    //pWrite(PR->p);
+  }
+#endif
+#endif
+  int ret = 0;
+  ring tailRing = PR->tailRing;
+  if (strat!=NULL)
+  {
+    kTest_L(PR,strat);
+    kTest_T(PW,strat);
+  }
+
+  poly p1 = PR->GetLmTailRing();   // p2 | p1
+  poly p2 = PW->GetLmTailRing();   // i.e. will reduce p1 with p2; lm = LT(p1) / LM(p2)
+  poly t2 = pNext(p2), lm = p1;    // t2 = p2 - LT(p2); really compute P = LC(p2)*p1 - LT(p1)/LM(p2)*p2
+  assume(p1 != NULL && p2 != NULL);// Attention, we have rings and there LC(p2) and LC(p1) are special
+  p_CheckPolyRing(p1, tailRing);
+  p_CheckPolyRing(p2, tailRing);
+
+  pAssume1(p2 != NULL && p1 != NULL &&
+           p_DivisibleBy(p2,  p1, tailRing));
+
+  pAssume1(p_GetComp(p1, tailRing) == p_GetComp(p2, tailRing) ||
+           (p_GetComp(p2, tailRing) == 0 &&
+            p_MaxComp(pNext(p2),tailRing) == 0));
+
+#ifdef HAVE_PLURAL
+  if (rIsPluralRing(currRing))
+  {
+    // for the time being: we know currRing==strat->tailRing
+    // no exp-bound checking needed
+    // (only needed if exp-bound(tailring)<exp-b(currRing))
+    if (PR->bucket!=NULL)  nc_kBucketPolyRed_Z(PR->bucket, p2,coef);
     else
     {
       poly _p = (PR->t_p != NULL ? PR->t_p : PR->p);
@@ -229,7 +632,7 @@ int ksReducePolyBound(LObject* PR,
   if (t2==NULL)           // Divisor is just one term, therefore it will
   {                       // just cancel the leading term
     PR->LmDeleteAndIter();
-    if (coef != NULL) *coef = n_Init(1, tailRing);
+    if (coef != NULL) *coef = n_Init(1, tailRing->cf);
     return 0;
   }
 
@@ -254,8 +657,18 @@ int ksReducePolyBound(LObject* PR,
     }
   }
 
+#ifdef HAVE_SHIFTBBA
+  poly lmRight;
+  if (tailRing->isLPring)
+  {
+    assume(PR->shift == 0);
+    assume(PW->shift == si_max(p_mFirstVblock(PW->p, tailRing) - 1, 0));
+    k_SplitFrame(lm, lmRight, PW->shift + 1, tailRing);
+  }
+#endif
+
   // take care of coef buisness
-  if (! n_IsOne(pGetCoeff(p2), tailRing))
+  if (! n_IsOne(pGetCoeff(p2), tailRing->cf))
   {
     number bn = pGetCoeff(lm);
     number an = pGetCoeff(p2);
@@ -264,33 +677,28 @@ int ksReducePolyBound(LObject* PR,
     if ((ct == 0) || (ct == 2))
       PR->Tail_Mult_nn(an);
     if (coef != NULL) *coef = an;
-    else n_Delete(&an, tailRing);
+    else n_Delete(&an, tailRing->cf);
   }
   else
   {
-    if (coef != NULL) *coef = n_Init(1, tailRing);
+    if (coef != NULL) *coef = n_Init(1, tailRing->cf);
   }
 
 
   // and finally,
-  PR->Tail_Minus_mm_Mult_qq(lm, t2, pLength(t2) /*PW->GetpLength() - 1*/, spNoether);
+#ifdef HAVE_SHIFTBBA
+  if (tailRing->isLPring)
+  {
+    PR->Tail_Minus_mm_Mult_qq(lm, tailRing->p_Procs->pp_Mult_mm(t2, lmRight, tailRing), pLength(t2), spNoether);
+  }
+  else
+#endif
+  {
+    PR->Tail_Minus_mm_Mult_qq(lm, t2, pLength(t2) /*PW->GetpLength() - 1*/, spNoether);
+  }
   assume(PW->GetpLength() == pLength(PW->p != NULL ? PW->p : PW->t_p));
   PR->LmDeleteAndIter();
 
-  // the following is commented out: shrinking
-#ifdef HAVE_SHIFTBBA_NONEXISTENT
-  if ( (currRing->isLPring) && (!strat->homog) )
-  {
-    // assume? h->p in currRing
-    PR->GetP();
-    poly qq = p_Shrink(PR->p, currRing->isLPring, currRing);
-    PR->Clear(); // does the right things
-    PR->p = qq;
-    PR->t_p = NULL;
-    PR->SetShortExpVector();
-  }
-#endif
-
 #if defined(KDEBUG) && defined(TEST_OPT_DEBUG_RED)
   if (TEST_OPT_DEBUG)
   {
@@ -327,8 +735,11 @@ int ksReducePolySig(LObject* PR,
 #endif
   int ret = 0;
   ring tailRing = PR->tailRing;
-  kTest_L(PR);
-  kTest_T(PW);
+  if (strat!=NULL)
+  {
+    kTest_L(PR,strat);
+    kTest_T(PW,strat);
+  }
 
   // signature-based stuff:
   // checking for sig-safeness first
@@ -419,7 +830,7 @@ int ksReducePolySig(LObject* PR,
     // for the time being: we know currRing==strat->tailRing
     // no exp-bound checking needed
     // (only needed if exp-bound(tailring)<exp-b(currRing))
-    if (PR->bucket!=NULL)  nc_kBucketPolyRed(PR->bucket, p2,coef);
+    if (PR->bucket!=NULL)  nc_kBucketPolyRed_Z(PR->bucket, p2,coef);
     else
     {
       poly _p = (PR->t_p != NULL ? PR->t_p : PR->p);
@@ -460,6 +871,16 @@ int ksReducePolySig(LObject* PR,
     }
   }
 
+#ifdef HAVE_SHIFTBBA
+  poly lmRight;
+  if (tailRing->isLPring)
+  {
+    assume(PR->shift == 0);
+    assume(PW->shift == si_max(p_mFirstVblock(PW->p, tailRing) - 1, 0));
+    k_SplitFrame(lm, lmRight, PW->shift + 1, tailRing);
+  }
+#endif
+
   // take care of coef buisness
   if (! n_IsOne(pGetCoeff(p2), tailRing->cf))
   {
@@ -479,23 +900,18 @@ int ksReducePolySig(LObject* PR,
 
 
   // and finally,
-  PR->Tail_Minus_mm_Mult_qq(lm, t2, PW->GetpLength() - 1, spNoether);
-  assume(PW->GetpLength() == pLength(PW->p != NULL ? PW->p : PW->t_p));
-  PR->LmDeleteAndIter();
-
-  // the following is commented out: shrinking
-#ifdef HAVE_SHIFTBBA_NONEXISTENT
-  if ( (currRing->isLPring) && (!strat->homog) )
+#ifdef HAVE_SHIFTBBA
+  if (tailRing->isLPring)
   {
-    // assume? h->p in currRing
-    PR->GetP();
-    poly qq = p_Shrink(PR->p, currRing->isLPring, currRing);
-    PR->Clear(); // does the right things
-    PR->p = qq;
-    PR->t_p = NULL;
-    PR->SetShortExpVector();
+    PR->Tail_Minus_mm_Mult_qq(lm, tailRing->p_Procs->pp_Mult_mm(t2, lmRight, tailRing), pLength(t2), spNoether);
   }
+  else
 #endif
+  {
+    PR->Tail_Minus_mm_Mult_qq(lm, t2, PW->GetpLength() - 1, spNoether);
+  }
+  assume(PW->GetpLength() == (int)pLength(PW->p != NULL ? PW->p : PW->t_p));
+  PR->LmDeleteAndIter();
 
 #if defined(KDEBUG) && defined(TEST_OPT_DEBUG_RED)
   if (TEST_OPT_DEBUG)
@@ -525,8 +941,11 @@ int ksReducePolySigRing(LObject* PR,
 #endif
   int ret = 0;
   ring tailRing = PR->tailRing;
-  kTest_L(PR);
-  kTest_T(PW);
+  if (strat!=NULL)
+  {
+    kTest_L(PR,strat);
+    kTest_T(PW,strat);
+  }
 
   // signature-based stuff:
   // checking for sig-safeness first
@@ -658,7 +1077,7 @@ int ksReducePolySigRing(LObject* PR,
     // for the time being: we know currRing==strat->tailRing
     // no exp-bound checking needed
     // (only needed if exp-bound(tailring)<exp-b(currRing))
-    if (PR->bucket!=NULL)  nc_kBucketPolyRed(PR->bucket, p2,coef);
+    if (PR->bucket!=NULL)  nc_kBucketPolyRed_Z(PR->bucket, p2,coef);
     else
     {
       poly _p = (PR->t_p != NULL ? PR->t_p : PR->p);
@@ -698,6 +1117,17 @@ int ksReducePolySigRing(LObject* PR,
       ret = 1;
     }
   }
+
+#ifdef HAVE_SHIFTBBA
+  poly lmRight;
+  if (tailRing->isLPring)
+  {
+    assume(PR->shift == 0);
+    assume(PW->shift == si_max(p_mFirstVblock(PW->p, tailRing) - 1, 0));
+    k_SplitFrame(lm, lmRight, PW->shift + 1, tailRing);
+  }
+#endif
+
   // take care of coef buisness
   if(rField_is_Ring(currRing))
   {
@@ -724,23 +1154,19 @@ int ksReducePolySigRing(LObject* PR,
   }
 
   // and finally,
-  PR->Tail_Minus_mm_Mult_qq(lm, t2, PW->GetpLength() - 1, spNoether);
-  assume(PW->GetpLength() == pLength(PW->p != NULL ? PW->p : PW->t_p));
+#ifdef HAVE_SHIFTBBA
+  if (tailRing->isLPring)
+  {
+    PR->Tail_Minus_mm_Mult_qq(lm, tailRing->p_Procs->pp_Mult_mm(t2, lmRight, tailRing), pLength(t2), spNoether);
+  }
+  else
+#endif
+  {
+    PR->Tail_Minus_mm_Mult_qq(lm, t2, PW->GetpLength() - 1, spNoether);
+  }
+  assume(PW->GetpLength() == (int)pLength(PW->p != NULL ? PW->p : PW->t_p));
   PR->LmDeleteAndIter();
 
-  // the following is commented out: shrinking
-#ifdef HAVE_SHIFTBBA_NONEXISTENT
-  if ( (currRing->isLPring) && (!strat->homog) )
-  {
-    // assume? h->p in currRing
-    PR->GetP();
-    poly qq = p_Shrink(PR->p, currRing->isLPring, currRing);
-    PR->Clear(); // does the right things
-    PR->p = qq;
-    PR->t_p = NULL;
-    PR->SetShortExpVector();
-  }
-#endif
 #if defined(KDEBUG) && defined(TEST_OPT_DEBUG_RED)
   if (TEST_OPT_DEBUG)
   {
@@ -763,7 +1189,6 @@ void ksCreateSpoly(LObject* Pair,   poly spNoether,
 #ifdef KDEBUG
   create_count++;
 #endif
-  kTest_L(Pair);
   poly p1 = Pair->p1;
   poly p2 = Pair->p2;
   Pair->tailRing = tailRing;
@@ -801,6 +1226,17 @@ void ksCreateSpoly(LObject* Pair,   poly spNoether,
   if (m1 == NULL)
     k_GetLeadTerms(p1, p2, currRing, m1, m2, tailRing);
 
+#ifdef HAVE_SHIFTBBA
+  poly m12, m22;
+  if (tailRing->isLPring)
+  {
+    assume(p_mFirstVblock(p1, tailRing) <= 1 || p_mFirstVblock(p2, tailRing) <= 1);
+    k_SplitFrame(m1, m12, si_max(p_mFirstVblock(p1, tailRing), 1), tailRing);
+    k_SplitFrame(m2, m22, si_max(p_mFirstVblock(p2, tailRing), 1), tailRing);
+    // coeffs of m1,m2 are NULL here
+  }
+#endif
+
   pSetCoeff0(m1, lc2);
   pSetCoeff0(m2, lc1);  // and now, m1 * LT(p1) == m2 * LT(p2)
 
@@ -825,26 +1261,62 @@ void ksCreateSpoly(LObject* Pair,   poly spNoether,
   }
 
   // get m2 * a2
+#ifdef HAVE_SHIFTBBA
+  if (tailRing->isLPring)
+  {
+    // m2*a2*m22
+    poly tmp= tailRing->p_Procs->pp_mm_Mult(a2, m2, tailRing);
+    a2 = tailRing->p_Procs->pp_Mult_mm(tmp, m22, tailRing);
+    p_Delete(&tmp,tailRing);
+  }
+  else
+#endif
   if (spNoether != NULL)
   {
     l2 = -1;
     a2 = tailRing->p_Procs->pp_Mult_mm_Noether(a2, m2, spNoether, l2, tailRing);
-    assume(l2 == pLength(a2));
+    assume(l2 == (int)pLength(a2));
   }
   else
+  {
     a2 = tailRing->p_Procs->pp_Mult_mm(a2, m2, tailRing);
+  }
 #ifdef HAVE_RINGS
   if (!(rField_is_Domain(currRing))) l2 = pLength(a2);
 #endif
 
   Pair->SetLmTail(m2, a2, l2, use_buckets, tailRing);
 
-  // get m2*a2 - m1*a1
-  Pair->Tail_Minus_mm_Mult_qq(m1, a1, l1, spNoether);
+#ifdef HAVE_SHIFTBBA
+  if (tailRing->isLPring)
+  {
+    // get m2*a2*m22 - m1*a1*m12
+    poly tmp=tailRing->p_Procs->pp_Mult_mm(a1, m12, tailRing);
+    Pair->Tail_Minus_mm_Mult_qq(m1, tmp, l1, spNoether);
+    p_Delete(&tmp,tailRing);
+  }
+  else
+#endif
+  {
+    // get m2*a2 - m1*a1
+    Pair->Tail_Minus_mm_Mult_qq(m1, a1, l1, spNoether);
+  }
 
   // Clean-up time
   Pair->LmDeleteAndIter();
   p_LmDelete(m1, tailRing);
+#ifdef HAVE_SHIFTBBA
+  if (tailRing->isLPring)
+  {
+    // just to be sure, check that the shift is correct
+    assume(Pair->shift == 0);
+    assume(si_max(p_mFirstVblock(Pair->p, tailRing) - 1, 0) == Pair->shift); // == 0
+
+    p_LmDelete(m12, tailRing);
+    p_LmDelete(m22, tailRing);
+    // m2 is already deleted
+  }
+#endif
 
   if (co != 0)
   {
@@ -857,21 +1329,6 @@ void ksCreateSpoly(LObject* Pair,   poly spNoether,
       p_SetCompP(p2,0, currRing, tailRing);
     }
   }
-
-  // the following is commented out: shrinking
-#ifdef HAVE_SHIFTBBA_NONEXISTENT
-  if (currRing->isLPring)
-  {
-    // assume? h->p in currRing
-    Pair->GetP();
-    poly qq = p_Shrink(Pair->p, currRing->isLPring, currRing);
-    Pair->Clear(); // does the right things
-    Pair->p = qq;
-    Pair->t_p = NULL;
-    Pair->SetShortExpVector();
-  }
-#endif
-
 }
 
 int ksReducePolyTail(LObject* PR, TObject* PW, poly Current, poly spNoether)
@@ -881,8 +1338,6 @@ int ksReducePolyTail(LObject* PR, TObject* PW, poly Current, poly spNoether)
   poly Lp =     PR->GetLmCurrRing();
   poly Save =   PW->GetLmCurrRing();
 
-  kTest_L(PR);
-  kTest_T(PW);
   pAssume(pIsMonomOf(Lp, Current));
 
   assume(Lp != NULL && Current != NULL && pNext(Current) != NULL);
@@ -913,20 +1368,6 @@ int ksReducePolyTail(LObject* PR, TObject* PW, poly Current, poly spNoether)
   if (Lp == Save)
     With.Delete();
 
-  // the following is commented out: shrinking
-#ifdef HAVE_SHIFTBBA_NONEXISTENT
-  if (currRing->isLPring)
-  {
-    // assume? h->p in currRing
-    PR->GetP();
-    poly qq = p_Shrink(PR->p, currRing->isLPring, currRing);
-    PR->Clear(); // does the right things
-    PR->p = qq;
-    PR->t_p = NULL;
-    PR->SetShortExpVector();
-  }
-#endif
-
   return ret;
 }
 
@@ -937,8 +1378,6 @@ int ksReducePolyTailBound(LObject* PR, TObject* PW, int bound, poly Current, pol
   poly Lp =     PR->GetLmCurrRing();
   poly Save =   PW->GetLmCurrRing();
 
-  kTest_L(PR);
-  kTest_T(PW);
   pAssume(pIsMonomOf(Lp, Current));
 
   assume(Lp != NULL && Current != NULL && pNext(Current) != NULL);
@@ -952,7 +1391,7 @@ int ksReducePolyTailBound(LObject* PR, TObject* PW, int bound, poly Current, pol
 
   if (!ret)
   {
-    if (! n_IsOne(coef, currRing))
+    if (! n_IsOne(coef, currRing->cf))
     {
       pNext(Current) = NULL;
       if (Current == PR->p && PR->t_p != NULL)
@@ -960,7 +1399,7 @@ int ksReducePolyTailBound(LObject* PR, TObject* PW, int bound, poly Current, pol
       PR->Mult_nn(coef);
     }
 
-    n_Delete(&coef, currRing);
+    n_Delete(&coef, currRing->cf);
     pNext(Current) = Red.GetLmTailRing();
     if (Current == PR->p && PR->t_p != NULL)
       pNext(PR->t_p) = pNext(Current);
@@ -969,20 +1408,6 @@ int ksReducePolyTailBound(LObject* PR, TObject* PW, int bound, poly Current, pol
   if (Lp == Save)
     With.Delete();
 
-  // the following is commented out: shrinking
-#ifdef HAVE_SHIFTBBA_NONEXISTENT
-  if (currRing->isLPring)
-  {
-    // assume? h->p in currRing
-    PR->GetP();
-    poly qq = p_Shrink(PR->p, currRing->isLPring, currRing);
-    PR->Clear(); // does the right things
-    PR->p = qq;
-    PR->t_p = NULL;
-    PR->SetShortExpVector();
-  }
-#endif
-
   return ret;
 }
 
@@ -1005,6 +1430,18 @@ int ksReducePolyTailBound(LObject* PR, TObject* PW, int bound, poly Current, pol
 poly ksCreateShortSpoly(poly p1, poly p2, ring tailRing)
 {
   poly a1 = pNext(p1), a2 = pNext(p2);
+#ifdef HAVE_SHIFTBBA
+  int shift1, shift2;
+  if (tailRing->isLPring)
+  {
+    // assume: LM is shifted, tail unshifted
+    assume(p_FirstVblock(a1, tailRing) <= 1);
+    assume(p_FirstVblock(a2, tailRing) <= 1);
+    // save the shift of the LM so we can shift the other monomials on demand
+    shift1 = p_mFirstVblock(p1, tailRing) - 1;
+    shift2 = p_mFirstVblock(p2, tailRing) - 1;
+  }
+#endif
   long c1=p_GetComp(p1, currRing),c2=p_GetComp(p2, currRing);
   long c;
   poly m1,m2;
@@ -1035,6 +1472,14 @@ poly ksCreateShortSpoly(poly p1, poly p2, ring tailRing)
   }
 #endif
 
+#ifdef HAVE_SHIFTBBA
+  // shift the next monomial on demand
+  if (tailRing->isLPring)
+  {
+    a1 = p_LPCopyAndShiftLM(a1, shift1, tailRing);
+    a2 = p_LPCopyAndShiftLM(a2, shift2, tailRing);
+  }
+#endif
   if (a1==NULL)
   {
     if(a2!=NULL)
@@ -1070,6 +1515,12 @@ x2:
           nDelete(&t2);
           pSetCoeff0(m2, t1);
       }
+#endif
+#ifdef HAVE_SHIFTBBA
+      if (tailRing->isLPring && (shift2!=0)) /*a1==NULL*/
+      {
+        p_LmDelete(a2, tailRing);
+      }
 #endif
       return m2;
     }
@@ -1120,6 +1571,12 @@ x1:
       nDelete(&lc2);
       nDelete(&t1);
     }
+#endif
+#ifdef HAVE_SHIFTBBA
+    if (tailRing->isLPring && (shift1!=0)) /*a2==NULL*/
+    {
+      p_LmDelete(a1, tailRing);
+    }
 #endif
     return m1;
   }
@@ -1175,6 +1632,13 @@ x1:
           nDelete(&lc2);
           nDelete(&t1);
         }
+#endif
+#ifdef HAVE_SHIFTBBA
+       if (tailRing->isLPring)
+       {
+         if (shift1!=0) p_LmDelete(a1, tailRing);
+         if (shift2!=0) p_LmDelete(a2, tailRing);
+       }
 #endif
         return m1;
       }
@@ -1189,6 +1653,13 @@ x1:
           nDelete(&lc2);
           nDelete(&t2);
         }
+#endif
+#ifdef HAVE_SHIFTBBA
+       if (tailRing->isLPring)
+       {
+         if (shift1!=0) p_LmDelete(a1, tailRing);
+         if (shift2!=0) p_LmDelete(a2, tailRing);
+       }
 #endif
         return m2;
       }
@@ -1219,6 +1690,13 @@ x1:
           nDelete(&t1);
           nDelete(&t2);
       }
+#endif
+#ifdef HAVE_SHIFTBBA
+      if (tailRing->isLPring)
+      {
+        if (shift1!=0) p_LmDelete(a1, tailRing);
+        if (shift2!=0) p_LmDelete(a2, tailRing);
+      }
 #endif
       return m1;
     }
@@ -1256,6 +1734,13 @@ x1:
         }
       }
     }
+#endif
+#ifdef HAVE_SHIFTBBA
+    if (tailRing->isLPring)
+    {
+      a1 = p_LPCopyAndShiftLM(a1, shift1, tailRing);
+      a2 = p_LPCopyAndShiftLM(a2, shift2, tailRing);
+    }
 #endif
     if (a2==NULL)
     {
