@@ -38,17 +38,19 @@ ideal kSba(ideal F,ideal Q, tHomog h, intvec ** mw, int incremental=0, int arri=
 ideal kStd(ideal F, ideal Q, tHomog h, intvec ** mw,intvec *hilb=NULL,
           int syzComp=0,int newIdeal=0, intvec *vw=NULL, s_poly_proc_t sp=NULL);
 
-ideal kStdShift(ideal F, ideal Q, tHomog h,intvec ** w, intvec *hilb,int syzComp,
-                int newIdeal, intvec *vw, int uptodeg, int lVblock);
+ideal kStdShift(ideal F, ideal Q, tHomog h,intvec ** mw, intvec *hilb=NULL,
+    int syzComp=0, int newIdeal=0, intvec *vw=NULL, BOOLEAN rightGB=FALSE);
+
+ideal rightgb(ideal F, ideal Q);
 
 /* the following global data are defined in kutil.cc */
 //extern int syzComp;
   /*stop building pairs after that component --> ideals.cc, syz.cc */
-extern int LazyPass,LazyDegree,Kstd1_mu,Kstd1_deg;
-  /*parameters for Lazy or global stops --> ipshell.cc, grammar.y*/
-extern BITSET kOptions;
+EXTERN_VAR int Kstd1_mu,Kstd1_deg;
+  /*parameters for global stops --> ipshell.cc, grammar.y*/
+EXTERN_VAR BITSET kOptions;
   /*the known test options (a constant)*/
-extern BITSET validOpts;
+EXTERN_VAR BITSET validOpts;
 
 void initMora(ideal F,kStrategy strat);
 
@@ -62,9 +64,10 @@ ideal stdred(ideal F, ideal Q, tHomog h,intvec ** w);
 ideal kMin_std(ideal F, ideal Q, tHomog h,intvec ** w, ideal &M,
               intvec *hilb=NULL, int syzComp=0,int reduced=0);
 
+BOOLEAN kVerify(ideal F,ideal Q);
 
-extern intvec * kModW;
-extern intvec * kHomW;
+EXTERN_VAR intvec * kModW;
+EXTERN_VAR intvec * kHomW;
 
 
 /* options:
