@@ -11,11 +11,11 @@
 #include <string.h>
 
 #include "omalloc/omalloc.h"
-#ifndef XMEMORY_H
+#ifdef HAVE_OMALLOC
 #include "omalloc/omallocClass.h"
 #endif
-#include "misc/mylimits.h"
 
+#include "misc/mylimits.h"
 
 #include "kernel/polys.h"
 #include "polys/operations/pShallowCopyDelete.h"
@@ -33,6 +33,10 @@
 #define setmaxT ((4096-12)/sizeof(TObject))
 #define setmaxTinc ((4096)/sizeof(TObject))
 
+#define RED_CANONICALIZE 200
+#define REDNF_CANONICALIZE 60
+#define REDTAIL_CANONICALIZE 100
+
 // if you want std computations as in Singular version < 2:
 // This disables RedThrough, tailReductions against T (bba),
 // sets posInT = posInT15 (bba, strat->honey), and enables redFirst with LDeg
@@ -59,7 +63,7 @@ typedef struct denominator_list_s denominator_list_s;
 typedef denominator_list_s *denominator_list;
 
 struct denominator_list_s{number n; denominator_list next;};
-extern denominator_list DENOMINATOR_LIST;
+EXTERN_VAR denominator_list DENOMINATOR_LIST;
 
 class sTObject
 {
@@ -75,6 +79,11 @@ public:
     length,     // as of pLDeg
     pLength,    // either == 0, or == pLength(p)
     i_r;        // index of TObject in R set, or -1 if not in T
+
+#ifdef HAVE_SHIFTBBA
+  int shift;
+#endif
+
   /*BOOLEAN*/ char is_normalized; // true, if pNorm was called on p, false otherwise
   // used in incremental sba() with F5C:
   // we know some of the redundant elements in
@@ -162,13 +171,14 @@ public:
   // manipulations
   KINLINE void pNorm();
   KINLINE void pCleardenom();
+  KINLINE void pContent();
 
 #ifdef KDEBUG
   void wrp();
 #endif
 };
 
-extern int strat_nr;
+EXTERN_VAR int strat_nr;
 
 class sLObject : public sTObject
 {
@@ -219,7 +229,7 @@ public:
 
   // does not delete bucket, just canonicalizes it
   // returned poly is such that Lm(p) \in currRing, Tail(p) \in tailRing
-  KINLINE poly CanonicalizeP();
+  KINLINE void CanonicalizeP();
 
   // makes a copy of the poly of L
   KINLINE void Copy();
@@ -256,11 +266,10 @@ public:
   KINLINE void HeadNormalize();
 };
 
-
-extern int HCord;
+EXTERN_VAR int HCord;
 
 class skStrategy
-#ifndef XMEMORY_H
+#ifdef HAVE_OMALLOC
                  : public omallocClass
 #endif
 {
@@ -317,11 +326,9 @@ public:
   TSet T;
   LSet L;
   LSet    B;
-  poly    kHEdge;
   poly    kNoether;
-  poly    t_kHEdge; // same polys in tailring
+  poly    t_kNoether; // same polys in tailring
   KINLINE poly    kNoetherTail();
-  poly    t_kNoether;
   BOOLEAN * NotUsedAxis;
   BOOLEAN * pairtest;/*used for enterOnePair*/
   poly tail;
@@ -345,7 +352,6 @@ public:
   int Bl,Bmax;
   int ak,LazyDegree,LazyPass;
   int syzComp;
-  int HCord;
   int lastAxis;
   int newIdeal;
   int minim;
@@ -359,15 +365,15 @@ public:
   int blockredmax;
   #endif
   #ifdef HAVE_SHIFTBBA
-  int lV;
   int cv; // in shift bases: counting V criterion
+  /*BOOLEAN*/ char rightGB;
   #endif
   /*BOOLEAN*/ char interpt;
   /*BOOLEAN*/ char homog;
 #ifdef HAVE_PLURAL
   /*BOOLEAN*/ char z2homog; // Z_2 - homogeneous input allows product criterion in commutative and SCA cases!
 #endif
-  /*BOOLEAN*/ char kHEdgeFound;
+  /*BOOLEAN*/ char kAllAxis; // all axis are used -> (re)compute noether
   /*BOOLEAN*/ char honey,sugarCrit;
   /*BOOLEAN*/ char Gebauer,noTailReduction;
   /*BOOLEAN*/ char fromT;
@@ -417,6 +423,7 @@ static inline LSet initL (int nr=setmaxL)
 void deleteInL(LSet set, int *length, int j,kStrategy strat);
 void enterL (LSet *set,int *length, int *LSetmax, LObject p,int at);
 void enterSBba (LObject &p,int atS,kStrategy strat, int atR = -1);
+void enterSBbaShift (LObject &p,int atS,kStrategy strat, int atR = -1);
 void enterSSba (LObject &p,int atS,kStrategy strat, int atR = -1);
 void initEcartPairBba (LObject* Lp,poly f,poly g,int ecartF,int ecartG);
 void initEcartPairMora (LObject* Lp,poly f,poly g,int ecartF,int ecartG);
@@ -484,8 +491,11 @@ int posInL110 (const LSet set, const int length,
 KINLINE poly redtailBba (poly p,int end_pos,kStrategy strat,BOOLEAN normalize=FALSE);
 KINLINE poly redtailBbaBound (poly p,int end_pos,kStrategy strat,int bound,BOOLEAN normalize=FALSE);
 #ifdef HAVE_RINGS
+KINLINE poly redtailBba_Ring (poly p,int end_pos,kStrategy strat);
 KINLINE poly redtailBba_Z (poly p,int end_pos,kStrategy strat);
+poly redtailBba_Ring (LObject* L, int end_pos, kStrategy strat );
 poly redtailBba_Z (LObject* L, int end_pos, kStrategy strat );
+void redtailBbaAlsoLC_Z (LObject* L, int end_pos, kStrategy strat );
 #endif
 poly redtailBba (LObject *L, int end_pos,kStrategy strat,
                  BOOLEAN withT = FALSE,BOOLEAN normalize=FALSE);
@@ -500,19 +510,15 @@ poly redNF (poly h,int & max_ind,int nonorm,kStrategy strat);
 int redNF0 (LObject *P,kStrategy strat);
 poly redNFTail (poly h,const int sl,kStrategy strat);
 int redHoney (LObject* h, kStrategy strat);
+int redLiftstd (LObject* h, kStrategy strat);
 #ifdef HAVE_RINGS
 int redRing (LObject* h,kStrategy strat);
+int redRing_Z (LObject* h,kStrategy strat);
 int redRiloc (LObject* h,kStrategy strat);
 void enterExtendedSpoly(poly h,kStrategy strat);
 void enterExtendedSpolySig(poly h,poly hSig,kStrategy strat);
 void superenterpairs (poly h,int k,int ecart,int pos,kStrategy strat, int atR = -1);
 void superenterpairsSig (poly h,poly hSig,int hFrom,int k,int ecart,int pos,kStrategy strat, int atR = -1);
-poly kCreateZeroPoly(long exp[], long cabsind, poly* t_p, ring leadRing, ring tailRing);
-long ind2(long arg);
-
-long ind_fact_2(long arg);
-long twoPow(long arg);
-ideal createG0();
 #endif
 int redLazy (LObject* h,kStrategy strat);
 int redHomog (LObject* h,kStrategy strat);
@@ -551,6 +557,7 @@ void initSyzRules (kStrategy strat);
 void updateS(BOOLEAN toT,kStrategy strat);
 void enterSyz (LObject &p,kStrategy strat, int atT);
 void enterT (LObject &p,kStrategy strat, int atT = -1);
+void replaceInLAndSAndT(LObject &p, int tj, kStrategy strat);
 #ifdef HAVE_RINGS
 void enterT_strong (LObject &p,kStrategy strat, int atT = -1);
 #endif
@@ -585,11 +592,18 @@ BOOLEAN findMinLMPair(poly sig, unsigned long not_sevSig, kStrategy strat, int s
 
 /// returns index of p in TSet, or -1 if not found
 int kFindInT(poly p, TSet T, int tlength);
+#ifdef HAVE_SHIFTBBA
+int kFindInTShift(poly p, TSet T, int tlength);
+#endif
 
 /// return -1 if no divisor is found
 ///        number of first divisor in T, otherwise
 int kFindDivisibleByInT(const kStrategy strat, const LObject* L, const int start=0);
+int kFindDivisibleByInT_Z(const kStrategy strat, const LObject* L, const int start=0);
+int kFindSameLMInT_Z(const kStrategy strat, const LObject* L, const int start=0);
 
+/// tests if T[0] divides the leading monomial of L, returns -1 if not
+int kTestDivisibleByT0_Z(const kStrategy strat, const LObject* L);
 /// return -1 if no divisor is found
 ///        number of first divisor in S, otherwise
 int kFindDivisibleByInS(const kStrategy strat, int *max_ind, LObject* L);
@@ -632,19 +646,19 @@ BOOLEAN kTest(kStrategy strat);
 // test strat, and test that S is contained in T
 BOOLEAN kTest_TS(kStrategy strat);
 // test LObject
-BOOLEAN kTest_L(LObject* L, ring tailRing = NULL,
+BOOLEAN kTest_L(LObject* L, kStrategy strat,
                  BOOLEAN testp = FALSE, int lpos = -1,
                  TSet T = NULL, int tlength = -1);
 // test TObject
-BOOLEAN kTest_T(TObject* T, ring tailRing = NULL, int tpos = -1, char TN = '?');
+BOOLEAN kTest_T(TObject* T, kStrategy strat, int tpos = -1, char TN = '?');
 // test set strat->SevS
 BOOLEAN kTest_S(kStrategy strat);
 #else
 #define kTest(A)        (TRUE)
 #define kTest_TS(A)     (TRUE)
-#define kTest_T(T)      (TRUE)
+#define kTest_T(T,S)    (TRUE)
 #define kTest_S(T)      (TRUE)
-#define kTest_L(T)      (TRUE)
+#define kTest_L(T,R)    (TRUE)
 #endif
 
 
@@ -685,6 +699,29 @@ void f5c (kStrategy strat, int& olddeg, int& minimcnt, int& hilbeledeg,
 //         -1 tailRing change could not be performed due to exceeding exp
 //            bound of currRing
 int ksReducePoly(LObject* PR,
+                 TObject* PW,
+                 poly spNoether = NULL,
+                 number *coef = NULL,
+                 poly *mon =NULL,
+                 kStrategy strat = NULL);
+
+/* like ksReducePoly, but if the reducer has only 1 term we still
+ * compute a possible coefficient multiplier for PR. this comes from
+ * a special situation in redRing_Z and it is used only there. */
+int ksReducePolyZ(LObject* PR,
+                 TObject* PW,
+                 poly spNoether = NULL,
+                 number *coef = NULL,
+                 kStrategy strat = NULL);
+
+int ksReducePolyLC(LObject* PR,
+                 TObject* PW,
+                 poly spNoether = NULL,
+                 number *coef = NULL,
+                 kStrategy strat = NULL);
+
+
+int ksReducePolyGCD(LObject* PR,
                  TObject* PW,
                  poly spNoether = NULL,
                  number *coef = NULL,
@@ -809,35 +846,55 @@ KINLINE void clearS (poly p, unsigned long p_sev, int* at, int* k,
 /* shiftgb stuff */
 #include "kernel/GBEngine/shiftgb.h"
 
+#ifdef HAVE_SHIFTBBA
+static inline int kFindInL1(const poly p, const kStrategy strat)
+{
+  for(int i=strat->Ll;i>=0;i--)
+  {
+    if (p==strat->L[i].p1) return i;
+  }
+  return -1;
+}
+
 poly pMove2CurrTail(poly p, kStrategy strat);
 
 poly pMoveCurrTail2poly(poly p, kStrategy strat);
 
 poly pCopyL2p(LObject h, kStrategy strat);
 
-void enterTShift(LObject p, kStrategy strat, int atT, int uptodeg, int lV);
+void enterTShift(LObject p, kStrategy strat, int atT = -1);
 
-void initBuchMoraShift (ideal F,ideal Q,kStrategy strat);
+BOOLEAN enterOnePairShift (poly q, poly p, int ecart, int isFromQ, kStrategy strat, int atR, int ecartq, int qisFromQ, int shiftcount, int ifromS);
 
-void enterOnePairSelfShifts (poly qq, poly p, int ecart, int isFromQ, kStrategy strat, int atR, int uptodeg, int lV);
+void enterpairsShift (poly h,int k,int ecart,int pos,kStrategy strat, int atR);
 
-void enterOnePairShift (poly q, poly p, int ecart, int isFromQ, kStrategy strat, int atR, int ecartq, int qisFromQ, int shiftcount, int ifromS, int uptodeg, int lV); // ok
-
-void enterpairsShift (poly h,int k,int ecart,int pos,kStrategy strat, int atR,int uptodeg, int lV);
-
-void updateSShift(kStrategy strat,int uptodeg,int lV);
-
-void initBbaShift(kStrategy strat);
+void superenterpairsShift (poly h,int k,int ecart,int pos,kStrategy strat, int atR);
 
 poly redtailBbaShift (LObject* L, int pos, kStrategy strat, BOOLEAN withT, BOOLEAN normalize);
 
 int redFirstShift (LObject* h,kStrategy strat); // ok
 
-ideal freegb(ideal I, int uptodeg, int lVblock);
+ideal bbaShift(ideal F, ideal Q,intvec *w,intvec *hilb,kStrategy strat);
+#endif
 
-ideal bbaShift(ideal F, ideal Q,intvec *w,intvec *hilb,kStrategy strat, int uptodeg, int lV);
 // test syz strategy: // will be removed soon
-extern  int (*test_PosInT)(const TSet T,const int tl,LObject &h);
-extern  int (*test_PosInL)(const LSet set, const int length,
+EXTERN_VAR int (*test_PosInT)(const TSet T,const int tl,LObject &h);
+EXTERN_VAR int (*test_PosInL)(const LSet set, const int length,
                 LObject* L,const kStrategy strat);
+
+static inline void kDeleteLcm(LObject *P)
+{
+ if (P->lcm!=NULL)
+ {
+ #ifdef HAVE_RINGS
+   if (rField_is_Ring(currRing))
+     pLmDelete(P->lcm);
+   else
+ #endif
+     pLmFree(P->lcm);
+   P->lcm=NULL;
+ }
+}
+
+void initenterpairs (poly h,int k,int ecart,int isFromQ,kStrategy strat, int atR = -1);
 #endif
