@@ -8,7 +8,6 @@
 #include "kernel/mod2.h"
 
 #include "misc/mylimits.h"
-#include "omalloc/omalloc.h"
 
 #include "misc/options.h"
 #include "misc/intvec.h"
@@ -31,8 +30,8 @@ extern void p_Setm_Syz(poly p, ring r,
 
 /*--------------static variables------------------------*/
 /*---points to the real components, shifted of the actual module-*/
-int *  currcomponents=NULL;
-long *  currShiftedComponents=NULL;
+VAR int *  currcomponents=NULL;
+VAR long *  currShiftedComponents=NULL;
 
 
 /*---counts number of applications of GM-criteria-------*/
@@ -2011,8 +2010,7 @@ void syPrint(syStrategy syzstr, const char *sn)
     syPrintEmptySpaces((*resolution)[k]);
     k++;
   }
-  PrintLn();
-  PrintLn();
+  PrintS("\n\n");
   k = 0;
   loop
   {
