@@ -7,8 +7,6 @@
 
 #include "kernel/mod2.h"
 
-#include "omalloc/omalloc.h"
-
 #include "misc/mylimits.h"
 #include "misc/options.h"
 #include "misc/intvec.h"
@@ -46,11 +44,11 @@
 #define USE_HEURISTIC2
 
 #ifdef SHOW_CRIT
-static int crit;
-static int crit1;
-static int spfl;
-static int cons_pairs;
-static int crit_fails;
+STATIC_VAR int crit;
+STATIC_VAR int crit1;
+STATIC_VAR int spfl;
+STATIC_VAR int cons_pairs;
+STATIC_VAR int crit_fails;
 #endif
 typedef struct sopen_pairs open_pairs;
 typedef open_pairs* crit_pairs;
@@ -1032,7 +1030,7 @@ Print("compute %d Paare im Module %d im Grad %d \n",howmuch,index,actdeg+index);
     if (index>0)
     {
       syRedNextPairs_Hilb(nextPairs,syzstr,howmuch,index,actdeg,&toSub,&maxindex,&maxdeg);
-      syzstr->res[index+1]->rank=id_RankFreeModule(syzstr->res[index+1],currRing);
+      syzstr->res[index+1]->rank=idElem(syzstr->res[index]);
       sySetNewHilb(syzstr,toSub,index,actdeg);
       toSub = 0;
       syCompactifyPairSet(syzstr->resPairs[index],(*syzstr->Tl)[index],0);
