@@ -6,7 +6,6 @@
 */
 
 #include "kernel/mod2.h"
-#include "omalloc/omalloc.h"
 
 #include "misc/mylimits.h"
 #include "misc/options.h"
@@ -46,8 +45,8 @@
 #define USE_REGULARITY
 #define WITH_SORT
 //#define FULL_TOTAKE
-int discard_pairs;
-int short_pairs;
+VAR int discard_pairs;
+VAR int short_pairs;
 
 /*3
 * assumes the ideals old_ideal and new_ideal to be homogeneous
