@@ -66,20 +66,7 @@ using std::vector;
 #include "kernel/GBEngine/kInline.h"
 #include "kernel/GBEngine/kstd1.h"
 
-
-#if 1
-
-#define npInit n_Init
-#define npNeg n_InpNeg
-#define npInvers n_Invers
-#define npMult n_Mult
-#define npIsOne n_IsOne
-#define npIsZero n_IsZero
-
-#else
-#error Please do NOT call internal functions directly!
-#endif
-
+#include "coeffs/modulop_inl.h" // npInit, npMult
 
 class PolySimple
 {
@@ -127,14 +114,17 @@ public:
   BOOLEAN changed;
   int len;
   BOOLEAN onlyBorrowed;
-  bool operator<(const MonRedRes& other) const{
+  bool operator<(const MonRedRes& other) const
+  {
     int cmp=p_LmCmp(p,other.p,currRing);
-    if ((cmp<0)||((cmp==0)&&((onlyBorrowed)&&(!(other.onlyBorrowed))))){
+    if ((cmp<0)||((cmp==0)&&((onlyBorrowed)&&(!(other.onlyBorrowed)))))
+    {
       return true;
     } else return false;
   }
   DataNoroCacheNode* ref;
-  MonRedRes(){
+  MonRedRes()
+  {
     ref=NULL;
     p=NULL;
   }
@@ -282,7 +272,7 @@ class slimgb_alg
   #ifdef TGB_RESORT_PAIRS
   BOOLEAN used_b;
   #endif
-  unsigned long pTotaldegree(poly p)
+  inline unsigned long pTotaldegree(poly p)
   {
       pTest(p);
       //assume(pDeg(p,r)==::p_Totaldegree(p,r));
@@ -290,13 +280,13 @@ class slimgb_alg
       return p->exp[deg_pos];
       //return ::pTotaldegree(p,this->r);
   }
-  int pTotaldegree_full(poly p)
+  inline int pTotaldegree_full(poly p)
   {
     int rr=0;
-    while(p)
+    while(p!=NULL)
     {
       int d=this->pTotaldegree(p);
-      rr=si_max(rr,d);
+      if (d>rr) rr=d;
       pIter(p);
     }
     return rr;
@@ -364,7 +354,7 @@ class simple_reducer:public reduction_step
   {
     this->p=pp;
     this->reducer_deg=pp_reducer_deg;
-    assume(pp_len==pLength(pp));
+    assume(pp_len==(int)pLength(pp));
     this->p_len=pp_len;
     this->c=pp_c;
   }
@@ -779,7 +769,7 @@ template<class number_type> MonRedResNP<number_type> noro_red_mon_to_non_poly(po
 
     poly exp_diff=cache->temp_term;
     p_ExpVectorDiff(exp_diff,t,c->strat->S[i],c->r);
-    p_SetCoeff(exp_diff,npNeg(npInvers(coefstrat,c->r->cf),c->r->cf),c->r);
+    p_SetCoeff(exp_diff,npNegM(npInversM(coefstrat,c->r->cf),c->r->cf),c->r);
     p_Setm(exp_diff,c->r);
     assume(c->strat->S[i]!=NULL);
 
@@ -830,15 +820,18 @@ poly tree_add(poly* a,int begin, int end,ring r)
 }
 */
 
-template<class number_type> SparseRow<number_type>* convert_to_sparse_row(number_type* temp_array,int temp_size,int non_zeros){
+template<class number_type> SparseRow<number_type>* convert_to_sparse_row(number_type* temp_array,int temp_size,int non_zeros)
+{
 SparseRow<number_type>* res=new SparseRow<number_type>(non_zeros);
 //int pos=0;
 //Print("denseness:%f\n",((double) non_zeros/(double) temp_size));
 number_type* it_coef=res->coef_array;
 int* it_idx=res->idx_array;
 #if 0
-for(i=0;i<cache->nIrreducibleMonomials;i++){
-  if (!(0==temp_array[i])){
+for(i=0;i<cache->nIrreducibleMonomials;i++)
+{
+  if (!(0==temp_array[i]))
+  {
 
     res->idx_array[pos]=i;
     res->coef_array[pos]=temp_array[i];
@@ -934,7 +927,7 @@ int temp_size,SparseRow<number_type>* row, number coef)
       int idx=idx_array[i];
       assume(bpos<256);
       assume(!(npIsZero((number)(long) buffer[bpos],currRing->cf)));
-      STATISTIC(n_Add); temp_array[idx]=F4mat_to_number_type(npAddM((number)(long) temp_array[idx], (number)(long) buffer[bpos++],currRing->cf));
+      temp_array[idx]=F4mat_to_number_type(npAddM((number)(long) temp_array[idx], (number)(long) buffer[bpos++],currRing->cf));
       #ifndef SING_NDEBUG
       assume(idx<temp_size);
       #endif
@@ -982,7 +975,7 @@ int temp_size,const number_type* row, int len,number coef)
       //int idx=idx_array[i];
       assume(bpos<256);
       //assume(!(npIsZero((number) buffer[bpos])));
-      STATISTIC(n_Add); temp_array[i]=F4mat_to_number_type(npAddM((number)(long) temp_array[i], (number)(long) buffer[bpos++],currRing->cf));
+      temp_array[i]=F4mat_to_number_type(npAddM((number)(long) temp_array[i], (number)(long) buffer[bpos++],currRing->cf));
       #ifndef SING_NDEBUG
       assume(i<temp_size);
       #endif
@@ -1009,10 +1002,10 @@ int temp_size,const number_type* row, int len)
   int i;
   for(i=0;i<len;i++)
   {
-      STATISTIC(n_Add); temp_array[i]=F4mat_to_number_type(npAddM((number)(long) temp_array[i], (number)(long) row[i],currRing->cf));
-      #ifndef SING_NDEBUG
-      assume(i<temp_size);
-      #endif
+    temp_array[i]=F4mat_to_number_type(npAddM((number)(long) temp_array[i], (number)(long) row[i],currRing->cf));
+    #ifndef SING_NDEBUG
+    assume(i<temp_size);
+    #endif
   }
 
 }
@@ -1035,13 +1028,11 @@ int temp_size,const number_type* row, int len)
   int i;
   for(i=0;i<len;i++)
   {
-
-      STATISTIC(n_Sub); temp_array[i]=F4mat_to_number_type(npSubM((number)(long) temp_array[i], (number)(long) row[i],currRing->cf));
-      #ifndef SING_NDEBUG
-      assume(i<temp_size);
-      #endif
+    temp_array[i]=F4mat_to_number_type(npSubM((number)(long) temp_array[i], (number)(long) row[i],currRing->cf));
+    #ifndef SING_NDEBUG
+    assume(i<temp_size);
+    #endif
   }
-
 }
 
 #ifdef SING_NDEBUG
@@ -1058,10 +1049,10 @@ template <class number_type> void add_sparse(number_type* const temp_array,int t
         for(j=0;j<len;j++)
         {
           int idx=idx_array[j];
-          STATISTIC(n_Add); temp_array[idx]=F4mat_to_number_type(   (number_type)(long)npAddM((number) (long)temp_array[idx],(number)(long) coef_array[j],currRing->cf));
+          temp_array[idx]=F4mat_to_number_type(   (number_type)(long)npAddM((number) (long)temp_array[idx],(number)(long) coef_array[j],currRing->cf));
           #ifndef SING_NDEBUG
           assume(idx<temp_size);
-	  #endif
+          #endif
         }
 }
 #ifdef SING_NDEBUG
@@ -1078,7 +1069,7 @@ template <class number_type> void sub_sparse(number_type* const temp_array,int t
         for(j=0;j<len;j++)
         {
           int idx=idx_array[j];
-          STATISTIC(n_Sub); temp_array[idx]=F4mat_to_number_type(  (number_type)(long) npSubM((number) (long)temp_array[idx],(number)(long) coef_array[j],currRing->cf));
+          temp_array[idx]=F4mat_to_number_type(  (number_type)(long) npSubM((number) (long)temp_array[idx],(number)(long) coef_array[j],currRing->cf));
           #ifndef SING_NDEBUG
           assume(idx<temp_size);
           #endif
@@ -1143,7 +1134,7 @@ template <class number_type> SparseRow<number_type>* noro_red_to_non_poly_dense(
        {
          if (red.ref->value_len==NoroCache<number_type>::backLinkCode)
          {
-           STATISTIC(n_Add); temp_array[red.ref->term_index]=F4mat_to_number_type( npAddM((number)(long) temp_array[red.ref->term_index],red.coef,currRing->cf));
+           temp_array[red.ref->term_index]=F4mat_to_number_type( npAddM((number)(long) temp_array[red.ref->term_index],red.coef,currRing->cf));
          }
          else
          {
@@ -1155,7 +1146,8 @@ template <class number_type> SparseRow<number_type>* noro_red_to_non_poly_dense(
    int non_zeros=0;
    for(i=0;i<cache->nIrreducibleMonomials;i++)
    {
-     //if (!(temp_array[i]==0)){
+     //if (!(temp_array[i]==0))
+     //{
      //  non_zeros++;
      //}
      assume(((temp_array[i]!=0)==0)|| (((temp_array[i]!=0)==1)));
@@ -1191,7 +1183,7 @@ template<class number_type> void write_coef_times_xx_idx_to_buffer(CoefIdx<numbe
   {
     assume(coef_array[j]!=0);
     CoefIdx<number_type> ci;
-    STATISTIC(n_Mult); ci.coef=F4mat_to_number_type(npMultM((number)(long) coef,(number)(long) coef_array[j],currRing->cf));
+    ci.coef=F4mat_to_number_type(npMultM((number)(long) coef,(number)(long) coef_array[j],currRing->cf));
     ci.idx=idx_array[j];
     pairs[pos++]=ci;
   }
@@ -1206,7 +1198,7 @@ template<class number_type> void write_coef_times_xx_idx_to_buffer_dense(CoefIdx
     {
       assume(coef_array[j]!=0);
       CoefIdx<number_type> ci;
-      STATISTIC(n_Mult); ci.coef=F4mat_to_number_type(npMultM((number)(long) coef,(number)(long) coef_array[j],currRing->cf));
+      ci.coef=F4mat_to_number_type(npMultM((number)(long) coef,(number)(long) coef_array[j],currRing->cf));
       assume(ci.coef!=0);
       ci.idx=j;
       pairs[pos++]=ci;
@@ -1241,7 +1233,7 @@ template<class number_type> void write_minus_coef_idx_to_buffer_dense(CoefIdx<nu
     {
       assume(coef_array[j]!=0);
       CoefIdx<number_type> ci;
-      STATISTIC(n_InpNeg); ci.coef=F4mat_to_number_type(npNegM((number)(long) coef_array[j],currRing->cf)); // FIXME: inplace negation! // TODO: check if this is not a bug!?
+      ci.coef=F4mat_to_number_type(npNegM((number)(long) coef_array[j],currRing->cf)); // FIXME: inplace negation! // TODO: check if this is not a bug!?
       assume(ci.coef!=0);
       ci.idx=j;
       pairs[pos++]=ci;
@@ -1268,7 +1260,7 @@ template<class number_type> void write_minus_coef_idx_to_buffer(CoefIdx<number_t
   {
     assume(coef_array[j]!=0);
     CoefIdx<number_type> ci;
-    STATISTIC(n_InpNeg); ci.coef=F4mat_to_number_type(npNegM((number)(unsigned long)coef_array[j],currRing->cf));  // FIXME: inplace negation! // TODO: check if this is not a bug!?
+    ci.coef=F4mat_to_number_type(npNegM((number)(unsigned long)coef_array[j],currRing->cf));  // FIXME: inplace negation! // TODO: check if this is not a bug!?
     ci.idx=idx_array[j];
     pairs[pos++]=ci;
   }
@@ -1376,7 +1368,7 @@ template <class number_type> SparseRow<number_type>* noro_red_to_non_poly_sparse
     }
     else
     {
-      STATISTIC(n_Add); pairs[act].coef=F4mat_to_number_type(npAddM((number)(long)pairs[act].coef,(number)(long)pairs[i].coef,currRing->cf));
+      pairs[act].coef=F4mat_to_number_type(npAddM((number)(long)pairs[act].coef,(number)(long)pairs[i].coef,currRing->cf));
     }
   }
 
@@ -1401,8 +1393,9 @@ template <class number_type> SparseRow<number_type>* noro_red_to_non_poly_sparse
 
   return res;
 }
-template<class number_type> SparseRow<number_type> * noro_red_to_non_poly_t(poly p, int &len, NoroCache<number_type>* cache,slimgb_alg* c){
-  assume(len==pLength(p));
+template<class number_type> SparseRow<number_type> * noro_red_to_non_poly_t(poly p, int &len, NoroCache<number_type>* cache,slimgb_alg* c)
+{
+  assume(len==(int)pLength(p));
   if (p==NULL)
   {
     len=0;
@@ -1449,7 +1442,6 @@ template<class number_type> SparseRow<number_type> * noro_red_to_non_poly_t(poly
 
 }
 #endif
-wlen_type pELength(poly p, ring r);
 int terms_sort_crit(const void* a, const void* b);
 //void simplest_gauss_modp(number* a, int nrows,int ncols);
 // a: a[0,0],a[0,1]....a[nrows-1,ncols-1]
@@ -1457,11 +1449,13 @@ int terms_sort_crit(const void* a, const void* b);
 #ifdef USE_NORO
 
 
-template <class number_type > void write_poly_to_row(number_type* row, poly h, poly*terms, int tn, ring r){
+template <class number_type > void write_poly_to_row(number_type* row, poly h, poly*terms, int tn)
+{
   //poly* base=row;
-  while(h!=NULL){
+  while(h!=NULL)
+  {
     //Print("h:%i\n",h);
-    number coef=p_GetCoeff(h,r);
+    number coef=pGetCoeff(h);
     poly* ptr_to_h=(poly*) bsearch(&h,terms,tn,sizeof(poly),terms_sort_crit);
     assume(ptr_to_h!=NULL);
     int pos=ptr_to_h-terms;
@@ -1470,12 +1464,15 @@ template <class number_type > void write_poly_to_row(number_type* row, poly h, p
     pIter(h);
   }
 }
-template <class number_type > poly row_to_poly(number_type* row, poly* terms, int tn, ring r){
+template <class number_type > poly row_to_poly(number_type* row, poly* terms, int tn, ring r)
+{
   poly h=NULL;
   int j;
   number_type zero=0;//;npInit(0);
-  for(j=tn-1;j>=0;j--){
-    if (!(zero==(row[j]))){
+  for(j=tn-1;j>=0;j--)
+  {
+    if (!(zero==(row[j])))
+    {
       poly t=terms[j];
       t=p_LmInit(t,r);
       p_SetCoeff(t,(number)(long) row[j],r);
@@ -1515,12 +1512,12 @@ public:
   {
     this->ncols=nncols;
     this->nrows=nnrows;
-    rows=(number_type**) omalloc(nnrows*sizeof(number_type*));
-    startIndices=(int*)omalloc(nnrows*sizeof(int));
+    rows=(number_type**) omalloc((size_t)nnrows*sizeof(number_type*));
+    startIndices=(int*)omalloc((size_t)nnrows*sizeof(int));
     int i;
     for(i=0;i<nnrows;i++)
     {
-      rows[i]=array+(i*nncols);
+      rows[i]=array+((long)i*(long)nncols);
       updateStartIndex(i,-1);
     }
   }
@@ -1543,9 +1540,19 @@ public:
   {
     int i;
     number_type* row_array=rows[row];
-    for(i=startIndices[row];i<ncols;i++)
+    if(currRing->cf->ch<=NV_MAX_PRIME)
     {
-      row_array[i]=F4mat_to_number_type(npMult((number)(long) row_array[i],(number)(long) coef,currRing->cf));
+      for(i=startIndices[row];i<ncols;i++)
+      {
+        row_array[i]=F4mat_to_number_type(npMult((number)(long) row_array[i],(number)(long) coef,currRing->cf));
+      }
+    }
+    else
+    {
+      for(i=startIndices[row];i<ncols;i++)
+      {
+        row_array[i]=F4mat_to_number_type(nvMult((number)(long) row_array[i],(number)(long) coef,currRing->cf));
+      }
     }
   }
   void reduceOtherRowsForward(int r)
@@ -1559,85 +1566,120 @@ public:
     int other_row;
     assume(!(npIsZero((number)(long) row_array[start],currRing->cf)));
     if (!(npIsOne((number)(long) coef,currRing->cf)))
-      multiplyRow(r,F4mat_to_number_type(npInvers((number)(long) coef,currRing->cf)));
+      multiplyRow(r,F4mat_to_number_type(npInversM((number)(long) coef,currRing->cf)));
     assume(npIsOne((number)(long) row_array[start],currRing->cf));
     int lastIndex=modP_lastIndexRow(row_array, ncols);
     number minus_one=npInit(-1, currRing->cf);
-    for (other_row=r+1;other_row<nrows;other_row++)
+    if(currRing->cf->ch<=NV_MAX_PRIME)
     {
-      assume(startIndices[other_row]>=start);
-      if (startIndices[other_row]==start)
+      for (other_row=r+1;other_row<nrows;other_row++)
       {
-        int i;
-        number_type* other_row_array=rows[other_row];
-        number coef2=npNeg((number)(long) other_row_array[start],currRing->cf);
-        if (coef2==minus_one)
+        assume(startIndices[other_row]>=start);
+        if (startIndices[other_row]==start)
         {
-          for(i=start;i<=lastIndex;i++)
+          int i;
+          number_type* other_row_array=rows[other_row];
+          number coef2=npNegM((number)(long) other_row_array[start],currRing->cf);
+          if (coef2==minus_one)
           {
-            if (row_array[i]!=zero)
-            { STATISTIC(n_Sub);
-              other_row_array[i]=F4mat_to_number_type(npSubM((number)(long) other_row_array[i], (number)(long) row_array[i],currRing->cf));
+            for(i=start;i<=lastIndex;i++)
+            {
+              if (row_array[i]!=zero)
+              {
+                other_row_array[i]=F4mat_to_number_type(npSubM((number)(long) other_row_array[i], (number)(long) row_array[i],currRing->cf));
+              }
             }
-
           }
-      }
-      else
-      {
-          //assume(FALSE);
-          for(i=start;i<=lastIndex;i++)
+          else
           {
-            if (row_array[i]!=zero)
-            { STATISTIC(n_Add);
-              other_row_array[i]=F4mat_to_number_type(npAddM(npMult(coef2,(number)(long) row_array[i],currRing->cf),(number)(long) other_row_array[i],currRing->cf));
+            for(i=start;i<=lastIndex;i++)
+            {
+              if (row_array[i]!=zero)
+              {
+                other_row_array[i]=F4mat_to_number_type(npAddM(npMult(coef2,(number)(long) row_array[i],currRing->cf),(number)(long) other_row_array[i],currRing->cf));
+              }
             }
-
           }
+          updateStartIndex(other_row,start);
+          assume(npIsZero((number)(long) other_row_array[start],currRing->cf));
         }
-        updateStartIndex(other_row,start);
-        assume(npIsZero((number)(long) other_row_array[start],currRing->cf));
       }
     }
+    else /* ch>NV_MAX_PRIME*/
+    {
+      for (other_row=r+1;other_row<nrows;other_row++)
+      {
+        assume(startIndices[other_row]>=start);
+        if (startIndices[other_row]==start)
+        {
+          int i;
+          number_type* other_row_array=rows[other_row];
+          number coef2=npNegM((number)(long) other_row_array[start],currRing->cf);
+          if (coef2==minus_one)
+          {
+            for(i=start;i<=lastIndex;i++)
+            {
+              if (row_array[i]!=zero)
+              {
+                other_row_array[i]=F4mat_to_number_type(npSubM((number)(long) other_row_array[i], (number)(long) row_array[i],currRing->cf));
+              }
+            }
+          }
+          else
+          {
+            for(i=start;i<=lastIndex;i++)
+            {
+              if (row_array[i]!=zero)
+              {
+                other_row_array[i]=F4mat_to_number_type(npAddM(nvMult(coef2,(number)(long) row_array[i],currRing->cf),(number)(long) other_row_array[i],currRing->cf));
+              }
+            }
+          }
+          updateStartIndex(other_row,start);
+          assume(npIsZero((number)(long) other_row_array[start],currRing->cf));
+        }
+      }
+    }
+}
+void updateStartIndex(int row,int lower_bound)
+{
+  number_type* row_array=rows[row];
+  assume((lower_bound<0)||(npIsZero((number)(long) row_array[lower_bound],currRing->cf)));
+  int i;
+  //number_type zero=npInit(0);
+  for(i=lower_bound+1;i<ncols;i++)
+  {
+    if (!(row_array[i]==0))
+      break;
   }
-  void updateStartIndex(int row,int lower_bound)
+  startIndices[row]=i;
+}
+int getStartIndex(int row)
+{
+  return startIndices[row];
+}
+BOOLEAN findPivot(int &r, int &c)
+{
+  //row>=r, col>=c
+
+  while(c<ncols)
   {
-    number_type* row_array=rows[row];
-    assume((lower_bound<0)||(npIsZero((number)(long) row_array[lower_bound],currRing->cf)));
     int i;
-    //number_type zero=npInit(0);
-    for(i=lower_bound+1;i<ncols;i++)
+    for(i=r;i<nrows;i++)
     {
-      if (!(row_array[i]==0))
-        break;
-    }
-    startIndices[row]=i;
-  }
-  int getStartIndex(int row)
-  {
-    return startIndices[row];
-  }
-  BOOLEAN findPivot(int &r, int &c)
-  {
-    //row>=r, col>=c
-
-    while(c<ncols)
-    {
-      int i;
-      for(i=r;i<nrows;i++)
+      assume(startIndices[i]>=c);
+      if (startIndices[i]==c)
       {
-        assume(startIndices[i]>=c);
-        if (startIndices[i]==c)
-        {
-          //r=i;
-          if (r!=i)
-            permRows(r,i);
-          return TRUE;
-        }
+        //r=i;
+        if (r!=i)
+          permRows(r,i);
+        return TRUE;
       }
-      c++;
     }
-    return FALSE;
+    c++;
   }
+  return FALSE;
+}
 protected:
   number_type** rows;
   int* startIndices;
@@ -1655,9 +1697,19 @@ public:
   {
     int i;
     number_type* row_array=rows[row];
-    for(i=startIndices[row];i<ncols;i++)
+    if (currRing->cf->ch<=NV_MAX_PRIME)
     {
-      row_array[i]=F4mat_to_number_type(npMult((number)(long) row_array[i],(number)(long) coef,currRing->cf));
+      for(i=startIndices[row];i<ncols;i++)
+      {
+        row_array[i]=F4mat_to_number_type(npMult((number)(long) row_array[i],(number)(long) coef,currRing->cf));
+      }
+    }
+    else
+    {
+      for(i=startIndices[row];i<ncols;i++)
+      {
+        row_array[i]=F4mat_to_number_type(nvMult((number)(long) row_array[i],(number)(long) coef,currRing->cf));
+      }
     }
   }
   ModPMatrixBackSubstProxyOnArray<number_type> (ModPMatrixProxyOnArray<number_type> & p)
@@ -1721,30 +1773,55 @@ public:
     if (!(npIsOne((number)(long) row_array[r],currRing->cf)))
     {
       //it should be one, but this safety is not expensive
-      multiplyRow(r, F4mat_to_number_type(npInvers((number)(long) row_array[start],currRing->cf)));
+      multiplyRow(r, F4mat_to_number_type(npInversM((number)(long) row_array[start],currRing->cf)));
     }
     int lastIndex=modP_lastIndexRow(row_array, ncols);
     assume(lastIndex<ncols);
     assume(lastIndex>=0);
-    for(other_row=r-1;other_row>=0;other_row--)
+    if(currRing->cf->ch<=NV_MAX_PRIME)
     {
-      assume(lastReducibleIndices[other_row]<=start);
-      if (lastReducibleIndices[other_row]==start)
+      for(other_row=r-1;other_row>=0;other_row--)
       {
-        number_type* other_row_array=rows[other_row];
-        number coef=npNeg((number)(long) other_row_array[start],currRing->cf);
-        assume(!(npIsZero(coef,currRing->cf)));
-        int i;
-        assume(start>startIndices[other_row]);
-        for(i=start;i<=lastIndex;i++)
+        assume(lastReducibleIndices[other_row]<=start);
+        if (lastReducibleIndices[other_row]==start)
         {
-          if (row_array[i]!=zero)
+          number_type* other_row_array=rows[other_row];
+          number coef=npNegM((number)(long) other_row_array[start],currRing->cf);
+          assume(!(npIsZero(coef,currRing->cf)));
+          int i;
+          assume(start>startIndices[other_row]);
+          for(i=start;i<=lastIndex;i++)
           {
-            STATISTIC(n_Add);
-            other_row_array[i]=F4mat_to_number_type(npAddM(npMult(coef,(number)(long)row_array[i],currRing->cf),(number)(long)other_row_array[i],currRing->cf));
+            if (row_array[i]!=zero)
+            {
+              other_row_array[i]=F4mat_to_number_type(npAddM(npMult(coef,(number)(long)row_array[i],currRing->cf),(number)(long)other_row_array[i],currRing->cf));
+            }
           }
+          updateLastReducibleIndex(other_row,r);
+        }
+      }
+    }
+    else
+    {
+      for(other_row=r-1;other_row>=0;other_row--)
+      {
+        assume(lastReducibleIndices[other_row]<=start);
+        if (lastReducibleIndices[other_row]==start)
+        {
+          number_type* other_row_array=rows[other_row];
+          number coef=npNegM((number)(long) other_row_array[start],currRing->cf);
+          assume(!(npIsZero(coef,currRing->cf)));
+          int i;
+          assume(start>startIndices[other_row]);
+          for(i=start;i<=lastIndex;i++)
+          {
+            if (row_array[i]!=zero)
+            {
+              other_row_array[i]=F4mat_to_number_type(npAddM(nvMult(coef,(number)(long)row_array[i],currRing->cf),(number)(long)other_row_array[i],currRing->cf));
+            }
+          }
+          updateLastReducibleIndex(other_row,r);
         }
-        updateLastReducibleIndex(other_row,r);
       }
     }
   }
@@ -1770,7 +1847,8 @@ template <class number_type > void simplest_gauss_modp(number_type* a, int nrows
 
   int c=0;
   int r=0;
-  while(mat.findPivot(r,c)){
+  while(mat.findPivot(r,c))
+  {
     //int pivot=find_pivot()
       mat.reduceOtherRowsForward(r);
     r++;
@@ -1783,7 +1861,8 @@ template <class number_type > void simplest_gauss_modp(number_type* a, int nrows
   //PrintS("StopGauss\n");
 }
 //int term_nodes_sort_crit(const void* a, const void* b);
-template <class number_type> void noro_step(poly*p,int &pn,slimgb_alg* c){
+template <class number_type> void noro_step(poly*p,int &pn,slimgb_alg* c)
+{
   //Print("Input rows %d\n",pn);
   int j;
   if (TEST_OPT_PROT)
@@ -1839,12 +1918,12 @@ template <class number_type> void noro_step(poly*p,int &pn,slimgb_alg* c){
   //if (TEST_OPT_PROT)
   //  Print("Evaluate Rows \n");
   pn=non_zeros;
-  number_type* number_array=(number_type*) omalloc0(n*pn*sizeof(number_type));
+  number_type* number_array=(number_type*) omalloc0(((size_t)n)*pn*sizeof(number_type));
 
   for(j=0;j<pn;j++)
   {
     int i;
-    number_type* row=number_array+n*j;
+    number_type* row=number_array+((long)n)*(long)j;
     /*for(i=0;i<n;i++)
     {
       row[i]=zero;
@@ -1881,9 +1960,11 @@ template <class number_type> void noro_step(poly*p,int &pn,slimgb_alg* c){
   simplest_gauss_modp(number_array,pn,n);
 
   int p_pos=0;
-  for(j=0;j<pn;j++){
-    poly h=row_to_poly(number_array+j*n,terms,n,c->r);
-    if(h!=NULL){
+  for(j=0;j<pn;j++)
+  {
+    poly h=row_to_poly(number_array+((long)j)*((long)n),terms,n,c->r);
+    if(h!=NULL)
+    {
       p[p_pos++]=h;
     }
   }
@@ -1899,13 +1980,16 @@ template <class number_type> void noro_step(poly*p,int &pn,slimgb_alg* c){
 
 }
 
-template <class number_type> void NoroCache<number_type>::collectIrreducibleMonomials( std::vector<DataNoroCacheNode<number_type> *>& res){
+template <class number_type> void NoroCache<number_type>::collectIrreducibleMonomials( std::vector<DataNoroCacheNode<number_type> *>& res)
+{
   int i;
-  for(i=0;i<root.branches_len;i++){
+  for(i=0;i<root.branches_len;i++)
+  {
     collectIrreducibleMonomials(1,root.branches[i],res);
   }
 }
-template <class number_type> void NoroCache<number_type>::collectIrreducibleMonomials(int level, NoroCacheNode* node, std::vector<DataNoroCacheNode<number_type>*>& res){
+template <class number_type> void NoroCache<number_type>::collectIrreducibleMonomials(int level, NoroCacheNode* node, std::vector<DataNoroCacheNode<number_type>*>& res)
+{
   assume(level>=0);
   if (node==NULL) return;
   if (level<(currRing->N))
@@ -1926,32 +2010,40 @@ template <class number_type> void NoroCache<number_type>::collectIrreducibleMono
   }
 }
 
-template<class number_type> DataNoroCacheNode<number_type>* NoroCache<number_type>::getCacheReference(poly term){
+template<class number_type> DataNoroCacheNode<number_type>* NoroCache<number_type>::getCacheReference(poly term)
+{
   int i;
   NoroCacheNode* parent=&root;
-  for(i=1;i<(currRing->N);i++){
+  for(i=1;i<(currRing->N);i++)
+  {
     parent=parent->getBranch(p_GetExp(term,i,currRing));
-    if (!(parent)){
+    if (!(parent))
+    {
       return NULL;
     }
   }
   DataNoroCacheNode<number_type>* res_holder=(DataNoroCacheNode<number_type>*) parent->getBranch(p_GetExp(term,i,currRing));
   return res_holder;
 }
-template<class number_type> poly NoroCache<number_type>::lookup(poly term, BOOLEAN& succ, int & len){
+template<class number_type> poly NoroCache<number_type>::lookup(poly term, BOOLEAN& succ, int & len)
+{
   int i;
   NoroCacheNode* parent=&root;
-  for(i=1;i<(currRing->N);i++){
+  for(i=1;i<(currRing->N);i++)
+  {
     parent=parent->getBranch(p_GetExp(term,i,currRing));
-    if (!(parent)){
+    if (!(parent))
+    {
       succ=FALSE;
       return NULL;
     }
   }
   DataNoroCacheNode<number_type>* res_holder=(DataNoroCacheNode<number_type>*) parent->getBranch(p_GetExp(term,i,currRing));
-  if (res_holder){
+  if (res_holder)
+  {
     succ=TRUE;
-    if ( /*(*/ res_holder->value_len==backLinkCode /*)*/ ){
+    if ( /*(*/ res_holder->value_len==backLinkCode /*)*/ )
+    {
       len=1;
       return term;
     }
