@@ -7,7 +7,6 @@
 
 #include "kernel/mod2.h"
 
-#include "omalloc/omalloc.h"
 #include "misc/mylimits.h"
 #include "misc/intvec.h"
 
@@ -19,6 +18,13 @@
 #include "polys/monomials/p_polys.h"
 #include "polys/simpleideals.h"
 
+#if SIZEOF_LONG == 8
+#define OVERFLOW_MAX LONG_MAX
+#define OVERFLOW_MIN LONG_MIN
+#else
+#define OVERFLOW_MAX (((int64)LONG_MAX)<<30)
+#define OVERFLOW_MIN (-OVERFLOW_MAX)
+#endif
 
 // #include "kernel/structs.h"
 // #include "kernel/polys.h"
@@ -42,9 +48,9 @@
 #include <iostream>
 #endif
 
-static int  **Qpol;
-static int  *Q0, *Ql;
-static int  hLength;
+STATIC_VAR int64  **Qpol;
+STATIC_VAR int64  *Q0, *Ql;
+STATIC_VAR int  hLength;
 
 
 static int hMinModulweight(intvec *modulweight)
@@ -65,7 +71,7 @@ static void hHilbEst(scfmon stc, int Nstc, varset var, int Nvar)
 {
   int  i, j;
   int  x, y, z = 1;
-  int  *p;
+  int64  *p;
   for (i = Nvar; i>0; i--)
   {
     x = 0;
@@ -84,12 +90,12 @@ static void hHilbEst(scfmon stc, int Nstc, varset var, int Nvar)
        WerrorS("internal arrays too big");
        return;
       }
-      p = (int *)omAlloc((unsigned long)z * sizeof(int));
+      p = (int64 *)omAlloc((unsigned long)z * sizeof(int64));
       if (Ql[j]!=0)
       {
         if (j==0)
-          memcpy(p, Qpol[j], Ql[j] * sizeof(int));
-        omFreeSize((ADDRESS)Qpol[j], Ql[j] * sizeof(int));
+          memcpy(p, Qpol[j], Ql[j] * sizeof(int64));
+        omFreeSize((ADDRESS)Qpol[j], Ql[j] * sizeof(int64));
       }
       if (j==0)
       {
@@ -102,27 +108,42 @@ static void hHilbEst(scfmon stc, int Nstc, varset var, int Nvar)
   }
 }
 
-static int  *hAddHilb(int Nv, int x, int *pol, int *lp)
+static int64 *hAddHilb(int Nv, int x, int64 *pol, int *lp)
 {
   int  l = *lp, ln, i;
-  int  *pon;
+  int64  *pon;
   *lp = ln = l + x;
   pon = Qpol[Nv];
-  memcpy(pon, pol, l * sizeof(int));
+  memcpy(pon, pol, l * sizeof(int64));
   if (l > x)
   {/*pon[i] -= pol[i - x];*/
     for (i = x; i < l; i++)
-    { int64 t=pon[i];
+    {
+      #ifndef __SIZEOF_INT128__
+      int64 t=pon[i];
       int64 t2=pol[i - x];
       t-=t2;
-      if ((t>=INT_MIN)&&(t<=INT_MAX)) pon[i]=t;
+      if ((t>=OVERFLOW_MIN)&&(t<=OVERFLOW_MAX)) pon[i]=t;
       else if (!errorreported) WerrorS("int overflow in hilb 1");
+      #else
+      __int128 t=pon[i];
+      __int128 t2=pol[i - x];
+      t-=t2;
+      if ((t>=LONG_MIN)&&(t<=LONG_MAX)) pon[i]=t;
+      else if (!errorreported) WerrorS("long int overflow in hilb 1");
+      #endif
     }
     for (i = l; i < ln; i++)
     { /*pon[i] = -pol[i - x];*/
+      #ifndef __SIZEOF_INT128__
       int64 t= -pol[i - x];
-      if ((t>=INT_MIN)&&(t<=INT_MAX)) pon[i]=t;
+      if ((t>=OVERFLOW_MIN)&&(t<=OVERFLOW_MAX)) pon[i]=t;
       else if (!errorreported) WerrorS("int overflow in hilb 2");
+      #else
+      __int128 t= -pol[i - x];
+      if ((t>=LONG_MIN)&&(t<=LONG_MAX)) pon[i]=t;
+      else if (!errorreported) WerrorS("long int overflow in hilb 2");
+      #endif
     }
   }
   else
@@ -135,11 +156,11 @@ static int  *hAddHilb(int Nv, int x, int *pol, int *lp)
   return pon;
 }
 
-static void hLastHilb(scmon pure, int Nv, varset var, int *pol, int lp)
+static void hLastHilb(scmon pure, int Nv, varset var, int64 *pol, int lp)
 {
   int  l = lp, x, i, j;
-  int  *pl;
-  int  *p;
+  int64  *pl;
+  int64  *p;
   p = pol;
   for (i = Nv; i>0; i--)
   {
@@ -151,11 +172,19 @@ static void hLastHilb(scmon pure, int Nv, varset var, int *pol, int lp)
   j = Q0[Nv + 1];
   for (i = 0; i < l; i++)
   { /* pl[i + j] += p[i];*/
+    #ifndef __SIZEOF_INT128__
     int64 t=pl[i+j];
     int64 t2=p[i];
     t+=t2;
-    if ((t>=INT_MIN)&&(t<=INT_MAX)) pl[i+j]=t;
+    if ((t>=OVERFLOW_MIN)&&(t<=OVERFLOW_MAX)) pl[i+j]=t;
     else if (!errorreported) WerrorS("int overflow in hilb 3");
+    #else
+    __int128 t=pl[i+j];
+    __int128 t2=p[i];
+    t+=t2;
+    if ((t>=LONG_MIN)&&(t<=LONG_MAX)) pl[i+j]=t;
+    else if (!errorreported) WerrorS("long int overflow in hilb 3");
+    #endif
   }
   x = pure[var[1]];
   if (x!=0)
@@ -163,11 +192,19 @@ static void hLastHilb(scmon pure, int Nv, varset var, int *pol, int lp)
     j += x;
     for (i = 0; i < l; i++)
     { /* pl[i + j] -= p[i];*/
+      #ifndef __SIZEOF_INT128__
       int64 t=pl[i+j];
       int64 t2=p[i];
       t-=t2;
-      if ((t>=INT_MIN)&&(t<=INT_MAX)) pl[i+j]=t;
+      if ((t>=OVERFLOW_MIN)&&(t<=OVERFLOW_MAX)) pl[i+j]=t;
       else if (!errorreported) WerrorS("int overflow in hilb 4");
+      #else
+      __int128 t=pl[i+j];
+      __int128 t2=p[i];
+      t-=t2;
+      if ((t>=LONG_MIN)&&(t<=LONG_MAX)) pl[i+j]=t;
+      else if (!errorreported) WerrorS("long int overflow in hilb 4");
+      #endif
     }
   }
   j += l;
@@ -176,13 +213,13 @@ static void hLastHilb(scmon pure, int Nv, varset var, int *pol, int lp)
 }
 
 static void hHilbStep(scmon pure, scfmon stc, int Nstc, varset var,
- int Nvar, int *pol, int Lpol)
+ int Nvar, int64 *pol, int Lpol)
 {
   int  iv = Nvar -1, ln, a, a0, a1, b, i;
   int  x, x0;
   scmon pn;
   scfmon sn;
-  int  *pon;
+  int64  *pon;
   if (Nstc==0)
   {
     hLastHilb(pure, iv, var, pol, Lpol);
@@ -1177,7 +1214,7 @@ static intvec * hSeries(ideal S, intvec *modulweight,
 
   intvec *work, *hseries1=NULL;
   int  mc;
-  int  p0;
+  int64  p0;
   int  i, j, k, l, ii, mw;
   hexist = hInit(S, Q, &hNexist, tailRing);
   if (hNexist==0)
@@ -1202,9 +1239,9 @@ static intvec * hSeries(ideal S, intvec *modulweight,
   hvar = (varset)omAlloc(((currRing->N) + 1) * sizeof(int));
   hpure = (scmon)omAlloc((1 + ((currRing->N) * (currRing->N))) * sizeof(int));
   stcmem = hCreate((currRing->N) - 1);
-  Qpol = (int **)omAlloc(((currRing->N) + 1) * sizeof(int *));
-  Ql = (int *)omAlloc0(((currRing->N) + 1) * sizeof(int));
-  Q0 = (int *)omAlloc(((currRing->N) + 1) * sizeof(int));
+  Qpol = (int64 **)omAlloc(((currRing->N) + 1) * sizeof(int64 *));
+  Ql = (int64 *)omAlloc0(((currRing->N) + 1) * sizeof(int64));
+  Q0 = (int64 *)omAlloc(((currRing->N) + 1) * sizeof(int64));
   *Qpol = NULL;
   hLength = k = j = 0;
   mc = hisModule;
@@ -1253,7 +1290,7 @@ static intvec * hSeries(ideal S, intvec *modulweight,
         (**Qpol)++;
       else
       {
-        *Qpol = (int *)omAlloc(sizeof(int));
+        *Qpol = (int64 *)omAlloc(sizeof(int64));
         hLength = *Ql = **Qpol = 1;
       }
     }
@@ -1310,11 +1347,11 @@ static intvec * hSeries(ideal S, intvec *modulweight,
   for (i = 0; i <= (currRing->N); i++)
   {
     if (Ql[i]!=0)
-      omFreeSize((ADDRESS)Qpol[i], Ql[i] * sizeof(int));
+      omFreeSize((ADDRESS)Qpol[i], Ql[i] * sizeof(int64));
   }
-  omFreeSize((ADDRESS)Q0, ((currRing->N) + 1) * sizeof(int));
-  omFreeSize((ADDRESS)Ql, ((currRing->N) + 1) * sizeof(int));
-  omFreeSize((ADDRESS)Qpol, ((currRing->N) + 1) * sizeof(int *));
+  omFreeSize((ADDRESS)Q0, ((currRing->N) + 1) * sizeof(int64));
+  omFreeSize((ADDRESS)Ql, ((currRing->N) + 1) * sizeof(int64));
+  omFreeSize((ADDRESS)Qpol, ((currRing->N) + 1) * sizeof(int64 *));
   hKill(stcmem, (currRing->N) - 1);
   omFreeSize((ADDRESS)hpure, (1 + ((currRing->N) * (currRing->N))) * sizeof(int));
   omFreeSize((ADDRESS)hvar, ((currRing->N) + 1) * sizeof(int));
@@ -1396,13 +1433,19 @@ void hDegreeSeries(intvec *s1, intvec *s2, int *co, int *mu)
   *co = i - j;
 }
 
-static void hPrintHilb(intvec *hseries)
+static void hPrintHilb(intvec *hseries,intvec *modul_weight)
 {
   int  i, j, l, k;
   if (hseries == NULL)
     return;
   l = hseries->length()-1;
   k = (*hseries)[l];
+  if ((modul_weight!=NULL)&&(modul_weight->compare(0)!=0))
+  {
+    char *s=modul_weight->ivString(1,0,1);
+    Print("module weights:%s\n",s);
+    omFree(s);
+  }
   for (i = 0; i < l; i++)
   {
     j = (*hseries)[i];
@@ -1423,7 +1466,7 @@ void hLookSeries(ideal S, intvec *modulweight, ideal Q, intvec *wdegree, ring ta
   intvec *hseries1 = hFirstSeries(S, modulweight, Q, wdegree, tailRing);
   if (errorreported) return;
 
-  hPrintHilb(hseries1);
+  hPrintHilb(hseries1,modulweight);
 
   const int l = hseries1->length()-1;
 
@@ -1433,7 +1476,7 @@ void hLookSeries(ideal S, intvec *modulweight, ideal Q, intvec *wdegree, ring ta
   hDegreeSeries(hseries1, hseries2, &co, &mu);
 
   PrintLn();
-  hPrintHilb(hseries2);
+  hPrintHilb(hseries2,modulweight);
   if ((l == 1) &&(mu == 0))
     scPrintDegree(rVar(currRing)+1, 0);
   else
@@ -1568,7 +1611,7 @@ static int comapreMonoIdBases_IG_Case(ideal J, int JCount, ideal Ob, int ObCount
   return(s);
 }
 
-static int positionInOrbit_IG_Case(ideal I, poly w, std::vector<ideal> idorb, std::vector<poly> polist, int trInd, int trunDegHs)
+static int positionInOrbit_IG_Case(ideal I, poly w, std::vector<ideal> idorb, std::vector<poly> polist, int trInd, int)
 {
   /*
    * It compares the ideal I with ideals in the set 'idorb'
@@ -1968,6 +2011,13 @@ static ideal colonIdeal(ideal S, poly w, int lV, ideal Jwi, int trunDegHs)
 
 void HilbertSeries_OrbitData(ideal S, int lV, bool IG_CASE, bool mgrad, bool odp, int trunDegHs)
 {
+	
+	/* new story: 
+	no lV is needed, i.e.  it is to be determined
+	the rest is extracted from the interface input list in extra.cc and makes the input of this proc
+	called from extra.cc
+	*/
+	
   /*
    * This is based on iterative right colon operations on a
    * two-sided monomial ideal of the free associative algebra.
@@ -2019,7 +2069,7 @@ void HilbertSeries_OrbitData(ideal S, int lV, bool IG_CASE, bool mgrad, bool odp
   std::vector<int> C;
 
   int ds, is, ps;
-  int lpcnt = 0;
+  unsigned long lpcnt = 0;
 
   poly w, wi;
   ideal Jwi;
