@@ -11,7 +11,6 @@
 
 
 #include "kernel/mod2.h"
-#include "omalloc/omalloc.h"
 #include "misc/options.h"
 #include "polys/monomials/p_polys.h"
 #include "polys/prCopy.h"
@@ -25,7 +24,7 @@
 // define if you want to use special src_ring
 #define HAVE_SRC_R 1
 // define if you want to use optimization step
-#define HAVE_MAP_OPTIMIZE 1
+#define HAVE_MAP_OPTIMIZE 0
 
 /*******************************************************************************
 **
@@ -86,7 +85,7 @@ static unsigned long maGetMaxExp(ideal pi_id, ring pi_r, ideal map_id, ring map_
   }
   for (i=0; i<IDELEMS(map_id); i++)
   {
-    p_Delete(&max_map_monomials[i], map_r);
+    p_LmFree(&max_map_monomials[i], map_r);
   }
   omFreeSize(max_map_monomials,IDELEMS(map_id)*sizeof(poly));
 
@@ -132,8 +131,8 @@ void maPoly_Out(mapoly mpoly, ring src_r, ring dest_r)
 **
 *F  mapolyCreate  . . . . . . . . . . . . . . .  Creates mapoly
 */
-static omBin mapolyBin = omGetSpecBin(sizeof(mapoly_s));
-static omBin macoeffBin = omGetSpecBin(sizeof(macoeff_s));
+STATIC_VAR omBin mapolyBin = omGetSpecBin(sizeof(mapoly_s));
+STATIC_VAR omBin macoeffBin = omGetSpecBin(sizeof(macoeff_s));
 
 mapoly maMonomial_Create(poly p, ring /*r_p*/, sBucket_pt bucket)
 {
@@ -167,11 +166,12 @@ void maMonomial_Destroy(mapoly mp, ring src_r, ring dest_r)
         omFreeBin(coeff, macoeffBin);
       }
       while (next != NULL);
-      if (mp->dest != NULL)
-      {
-        assume(dest_r != NULL);
-        p_Delete(&(mp->dest), dest_r);
-      }
+      mp->coeff=NULL;
+    }
+    if (mp->dest != NULL)
+    {
+      assume(dest_r != NULL);
+      p_Delete(&(mp->dest), dest_r);
     }
   }
   omFreeBin(mp, mapolyBin);
@@ -396,7 +396,7 @@ ideal fast_map_common_subexp(const ideal map_id,const ring map_r,const ideal ima
   maPoly_Eval(mp, src_r, dest_id, dest_r, length);
   if (TEST_OPT_PROT) PrintS(".");
 
-  // collect the results back into an ideal
+  // collect the results back into an ideal, clean up mideal
   ideal res_dest_id = maIdeal_2_Ideal(mideal, dest_r);
   if (TEST_OPT_PROT) PrintS(".");
 
