@@ -1,6 +1,5 @@
 #include "kernel/mod2.h"
 
-#include "omalloc/omalloc.h"
 #include "misc/options.h"
 #include "misc/intvec.h"
 
@@ -12,9 +11,6 @@
 #include "kernel/GBEngine/kstd1.h"
 #include "kernel/GBEngine/khstd.h"
 
-#include "kernel/GBEngine/kutil.h"
-
-
 #ifdef HAVE_PLURAL
 #include "polys/nc/nc.h"
 #endif
@@ -88,8 +84,6 @@ ideal maGetPreimage(ring theImageRing, map theMap, ideal id, const ring dst_r)
      return NULL;
   }
 
-  assume(n_SetMap(theImageRing->cf, dst_r->cf) == ndCopyMap);
-
   if (theImageRing->cf != dst_r->cf)
   {
     /// TODO: there might be extreme cases where this doesn't hold...
