@@ -7,7 +7,6 @@
 * ngc == number gnu complex
 */
 #include "misc/auxiliary.h"
-#include "omalloc/omalloc.h"
 
 #include "misc/mylimits.h"
 #include "reporter/reporter.h"
@@ -142,6 +141,7 @@ static number ngcInvers(number a, const coeffs R)
   if (((gmp_complex*)a)->isZero())
   {
     WerrorS(nDivBy0);
+    r = new gmp_complex( 0 );
   }
   else
   {
@@ -194,7 +194,7 @@ static number ngcDiv (number a, number b, const coeffs r)
   {
     // a/0 = error
     WerrorS(nDivBy0);
-    return NULL;
+    return (number)new gmp_complex( 0 );
   }
   gmp_complex* res = new gmp_complex( (*(gmp_complex*)a) / (*(gmp_complex*)b) );
   return (number)res;
@@ -410,19 +410,10 @@ static void ngcKillChar(coeffs r)
   omFreeSize((ADDRESS)p, P * sizeof(char*));
 }
 
-static char* ngcCoeffString(const coeffs r)
-{
-  const char *p=n_ParameterNames(r)[0];
-  char *s=(char*)omAlloc(31+strlen(p));
-  sprintf(s,"complex,%d,%d,%s",r->float_len,r->float_len2,p);
-  return s;
-}
-
 static char* ngcCoeffName(const coeffs r)
 {
-  static char ngcCoeffName_buf[40];
+  STATIC_VAR char ngcCoeffName_buf[40];
   const char *p=n_ParameterNames(r)[0];
-  if (ngcCoeffName_buf!=NULL) omFree(ngcCoeffName_buf);
   sprintf(ngcCoeffName_buf,"complex,%d,%d,%s",r->float_len,r->float_len2,p);
   return ngcCoeffName_buf;
 }
@@ -521,6 +512,14 @@ static number ngcCopyMap(number from, const coeffs aRing, const coeffs r)
   return (number)b;
 }
 
+static number ngcInitMPZ(mpz_t m, const coeffs)
+{
+  gmp_float mm(m);
+  gmp_complex* res = new gmp_complex(mm);
+  return (number)res;
+}
+
+
 static nMapFunc ngcSetMap(const coeffs src, const coeffs dst)
 {
   assume( getCoeffType(dst) == n_long_C );
@@ -561,13 +560,14 @@ BOOLEAN ngcInitChar(coeffs n, void* parameter)
 
   n->cfKillChar = ngcKillChar;
   n->ch = 0;
-  n->cfCoeffString=ngcCoeffString;
   n->cfCoeffName=ngcCoeffName;
+  n->cfCoeffWrite = ngcCoeffWrite;
 
   n->cfDelete  = ngcDelete;
   //n->cfNormalize=ndNormalize;
-  n->cfInit   = ngcInit;
-  n->cfInt    = ngcInt;
+  n->cfInit    = ngcInit;
+  n->cfInitMPZ = ngcInitMPZ;
+  n->cfInt     = ngcInt;
   n->cfAdd     = ngcAdd;
   n->cfSub     = ngcSub;
   n->cfMult    = ngcMult;
@@ -591,7 +591,6 @@ BOOLEAN ngcInitChar(coeffs n, void* parameter)
   n->cfSetMap = ngcSetMap;
   n->cfRePart  = ngcRePart;
   n->cfImPart  = ngcImPart;
-  n->cfCoeffWrite = ngcCoeffWrite;
     // cfSize  = ndSize;
 #ifdef LDEBUG
   //n->cfDBTest  = ndDBTest; // not yet implemented: ngcDBTest
