@@ -6,7 +6,6 @@
 */
 
 #include "misc/auxiliary.h"
-#include "omalloc/omalloc.h"
 
 #include "factory/factory.h"
 
@@ -63,10 +62,6 @@ BOOLEAN  nlIsMOne(number a, const coeffs r);
 long     nlInt(number &n, const coeffs r);
 number   nlBigInt(number &n);
 
-#ifdef HAVE_RINGS
-number nlMapGMP(number from, const coeffs src, const coeffs dst);
-#endif
-
 BOOLEAN  nlGreaterZero(number za, const coeffs r);
 number   nlInvers(number a, const coeffs r);
 number   nlDiv(number a, number b, const coeffs r);
@@ -77,7 +72,6 @@ void     nlPower(number x, int exp, number *lu, const coeffs r);
 const char *   nlRead (const char *s, number *a, const coeffs r);
 void     nlWrite(number a, const coeffs r);
 
-void     nlCoeffWrite(const coeffs r, BOOLEAN details);
 number   nlFarey(number nN, number nP, const coeffs CF);
 
 #ifdef LDEBUG
@@ -167,6 +161,17 @@ number nlShort3_noinline(number x) // assume x->s==3
   return nlShort3(x);
 }
 
+static number nlInitMPZ(mpz_t m, const coeffs)
+{
+  number z = ALLOC_RNUMBER();
+  z->s = 3;
+  #ifdef LDEBUG
+  z->debug=123456;
+  #endif
+  mpz_init_set(z->z, m);
+  z=nlShort3(z);
+  return z;
+}
 
 #if (__GNU_MP_VERSION*10+__GNU_MP_VERSION_MINOR < 31)
 void mpz_mul_si (mpz_ptr r, mpz_srcptr s, long int si)
@@ -198,16 +203,9 @@ static number nlMapR(number from, const coeffs src, const coeffs dst);
 /*2
 * convert from a GMP integer
 */
-number nlMapGMP(number from, const coeffs /*src*/, const coeffs /*dst*/)
+static inline number nlMapGMP(number from, const coeffs /*src*/, const coeffs dst)
 {
-  number z=ALLOC_RNUMBER();
-#if defined(LDEBUG)
-  z->debug=123456;
-#endif
-  mpz_init_set(z->z,(mpz_ptr) from);
-  z->s = 3;
-  z=nlShort3(z);
-  return z;
+  return nlInitMPZ((mpz_ptr)from,dst);
 }
 
 number nlMapZ(number from, const coeffs src, const coeffs dst)
@@ -216,7 +214,7 @@ number nlMapZ(number from, const coeffs src, const coeffs dst)
   {
     return from;
   }
-  return nlMapGMP(from,src,dst);
+  return nlInitMPZ((mpz_ptr)from,dst);
 }
 
 /*2
@@ -424,6 +422,16 @@ static number nlMapR(number from, const coeffs src, const coeffs dst)
   return re;
 }
 
+static number nlMapR_BI(number from, const coeffs src, const coeffs dst)
+{
+  assume( getCoeffType(src) == n_R );
+
+  double f=nrFloat(from); // FIXME? TODO? // extern float   nrFloat(number n);
+  if (f==0.0) return INT_TO_SR(0);
+  long l=long(f);
+  return nlInit(l,dst);
+}
+
 static number nlMapLongR(number from, const coeffs src, const coeffs dst)
 {
   assume( getCoeffType(src) == n_long_R );
@@ -471,6 +479,7 @@ static number nlMapLongR(number from, const coeffs src, const coeffs dst)
     for (i=0;i<size;i++) dd[i] = qp[i];
     bl = 1-e;
     nn = (mp_ptr)allocfunc(sizeof(mp_limb_t)*bl);
+    memset(nn,0,sizeof(mp_limb_t)*bl);
     nn[bl-1] = 1;
     ndest = res->n;
     ndest->_mp_d = nn;
@@ -482,6 +491,141 @@ static number nlMapLongR(number from, const coeffs src, const coeffs dst)
     al = dest->_mp_size = size+e;
     if (al<2) al = 2;
     dd = (mp_ptr)allocfunc(sizeof(mp_limb_t)*al);
+    memset(dd,0,sizeof(mp_limb_t)*al);
+    for (i=0;i<size;i++) dd[i+e] = qp[i];
+    for (i=0;i<e;i++) dd[i] = 0;
+    res->s = 3;
+  }
+
+  dest->_mp_d = dd;
+  dest->_mp_alloc = al;
+  if (negative) mpz_neg(dest,dest);
+
+  if (res->s==0)
+    nlNormalize(res,dst);
+  else if (mpz_size1(res->z)<=MP_SMALL)
+  {
+    // res is new, res->ref is 1
+    res=nlShort3(res);
+  }
+  nlTest(res, dst);
+  return res;
+}
+
+static number nlMapLongR_BI(number from, const coeffs src, const coeffs dst)
+{
+  assume( getCoeffType(src) == n_long_R );
+
+  gmp_float *ff=(gmp_float*)from;
+  if (mpf_fits_slong_p(ff->t))
+  {
+    long l=mpf_get_si(ff->t);
+    return nlInit(l,dst);
+  }
+  char *out=floatToStr(*(gmp_float*)from, src->float_len);
+  char *p=strchr(out,'.');
+  *p='\0';
+  number res;
+  res = ALLOC_RNUMBER();
+#if defined(LDEBUG)
+  res->debug=123456;
+#endif
+  res->s=3;
+  mpz_init(res->z);
+  if (out[0]=='-')
+  {
+    mpz_set_str(res->z,out+1,10);
+    res=nlNeg(res,dst);
+  }
+  else
+  {
+    mpz_set_str(res->z,out,10);
+  }
+  omFree( (void *)out );
+  return res;
+}
+
+static number nlMapC(number from, const coeffs src, const coeffs dst)
+{
+  assume( getCoeffType(src) == n_long_C );
+  if ( ! ((gmp_complex*)from)->imag().isZero() )
+    return INT_TO_SR(0);
+
+  if (dst->is_field==FALSE) /* ->ZZ */
+  {
+    char *s=floatToStr(((gmp_complex*)from)->real(),src->float_len);
+    mpz_t z;
+    mpz_init(z);
+    char *ss=nEatLong(s,z);
+    if (*ss=='\0')
+    {
+      omFree(s);
+      number n=nlInitMPZ(z,dst);
+      mpz_clear(z);
+      return n;
+    }
+    omFree(s);
+    mpz_clear(z);
+    WarnS("conversion problem in CC -> ZZ mapping");
+    return INT_TO_SR(0);
+  }
+      
+  mpf_t *f = ((gmp_complex*)from)->real()._mpfp();
+
+  number res;
+  mpz_ptr dest,ndest;
+  int size, i,negative;
+  int e,al,bl;
+  mp_ptr qp,dd,nn;
+
+  size = (*f)[0]._mp_size;
+  if (size == 0)
+    return INT_TO_SR(0);
+  if(size<0)
+  {
+    negative = 1;
+    size = -size;
+  }
+  else
+    negative = 0;
+
+  qp = (*f)[0]._mp_d;
+  while(qp[0]==0)
+  {
+    qp++;
+    size--;
+  }
+
+  e=(*f)[0]._mp_exp-size;
+  res = ALLOC_RNUMBER();
+#if defined(LDEBUG)
+  res->debug=123456;
+#endif
+  dest = res->z;
+
+  void* (*allocfunc) (size_t);
+  mp_get_memory_functions (&allocfunc,NULL, NULL);
+  if (e<0)
+  {
+    al = dest->_mp_size = size;
+    if (al<2) al = 2;
+    dd = (mp_ptr)allocfunc(sizeof(mp_limb_t)*al);
+    for (i=0;i<size;i++) dd[i] = qp[i];
+    bl = 1-e;
+    nn = (mp_ptr)allocfunc(sizeof(mp_limb_t)*bl);
+    memset(nn,0,sizeof(mp_limb_t)*bl);
+    nn[bl-1] = 1;
+    ndest = res->n;
+    ndest->_mp_d = nn;
+    ndest->_mp_alloc = ndest->_mp_size = bl;
+    res->s = 0;
+  }
+  else
+  {
+    al = dest->_mp_size = size+e;
+    if (al<2) al = 2;
+    dd = (mp_ptr)allocfunc(sizeof(mp_limb_t)*al);
+    memset(dd,0,sizeof(mp_limb_t)*al);
     for (i=0;i<size;i++) dd[i+e] = qp[i];
     for (i=0;i<e;i++) dd[i] = 0;
     res->s = 3;
@@ -733,8 +877,6 @@ number   nlExactDiv(number a, number b, const coeffs r)
     WerrorS(nDivBy0);
     return INT_TO_SR(0);
   }
-  if (a==INT_TO_SR(0))
-    return INT_TO_SR(0);
   number u;
   if (SR_HDL(a) & SR_HDL(b) & SR_INT)
   {
@@ -800,8 +942,6 @@ number nlIntDiv (number a, number b, const coeffs r)
     WerrorS(nDivBy0);
     return INT_TO_SR(0);
   }
-  if (a==INT_TO_SR(0))
-    return INT_TO_SR(0);
   number u;
   if (SR_HDL(a) & SR_HDL(b) & SR_INT)
   {
@@ -843,12 +983,14 @@ number nlIntDiv (number a, number b, const coeffs r)
 #endif
   assume(a->s==3);
   assume(b->s==3);
-  mpz_init_set(u->z,a->z);
   /* u=u/b */
+  mpz_t rr;
+  mpz_init(rr);
+  mpz_mod(rr,a->z,b->z);
   u->s = 3;
-  number rr=nlIntMod(a,b,r);
-  if (SR_HDL(rr) &  SR_INT) mpz_sub_ui(u->z,u->z,SR_TO_INT(rr));
-  else                      mpz_sub(u->z,u->z,rr->z);
+  mpz_init(u->z);
+  mpz_sub(u->z,a->z,rr);
+  mpz_clear(rr);
   mpz_divexact(u->z,u->z,b->z);
   if (aa!=NULL)
   {
@@ -1218,8 +1360,6 @@ number nlGcd(number a, number b, const coeffs r)
   {
     long i=SR_TO_INT(a);
     long j=SR_TO_INT(b);
-    if((i==0L)||(j==0L))
-      return INT_TO_SR(1);
     long l;
     i=ABS(i);
     j=ABS(j);
@@ -1271,6 +1411,7 @@ number nlGcd(number a, number b, const coeffs r)
   nlTest(result, r);
   return result;
 }
+
 static int int_extgcd(int a, int b, int * u, int* x, int * v, int* y)
 {
   int q, r;
@@ -1487,7 +1628,7 @@ void nlGMP(number &i, mpz_t n, const coeffs r)
   }
   if (i->s!=3)
   {
-     WarnS("Omitted denominator during coefficient mapping !");
+    WarnS("Omitted denominator during coefficient mapping !");
   }
   mpz_set(n, i->z);
 }
@@ -1496,7 +1637,7 @@ void nlGMP(number &i, mpz_t n, const coeffs r)
 /*2
 * acces to denominator, other 1 for integers
 */
-number   nlGetDenom(number &n, const coeffs r)
+number nlGetDenom(number &n, const coeffs r)
 {
   if (!(SR_HDL(n) & SR_INT))
   {
@@ -1525,7 +1666,7 @@ number   nlGetDenom(number &n, const coeffs r)
 /*2
 * acces to Nominator, nlCopy(n) for integers
 */
-number   nlGetNumerator(number &n, const coeffs r)
+number nlGetNumerator(number &n, const coeffs r)
 {
   if (!(SR_HDL(n) & SR_INT))
   {
@@ -1605,7 +1746,7 @@ BOOLEAN _nlEqual_aNoImm_OR_bNoImm(number a, number b)
 // copy not immediate number a
 number _nlCopy_NoImm(number a)
 {
-  assume(!((SR_HDL(a) & SR_INT)||(a==NULL)));
+  assume(!(SR_HDL(a) & SR_INT));
   //nlTest(a, r);
   number b=ALLOC_RNUMBER();
 #if defined(LDEBUG)
@@ -1634,22 +1775,20 @@ void _nlDelete_NoImm(number *a)
         mpz_clear((*a)->n);
       case 3:
         mpz_clear((*a)->z);
-#ifdef LDEBUG
-        (*a)->s=2;
-#endif
     }
+    #ifdef LDEBUG
+    memset(*a,0,sizeof(**a));
+    #endif
     FREE_RNUMBER(*a); // omFreeBin((void *) *a, rnumber_bin);
   }
 }
 
 number _nlNeg_NoImm(number a)
 {
+  mpz_neg(a->z,a->z);
+  if (a->s==3)
   {
-    mpz_neg(a->z,a->z);
-    if (a->s==3)
-    {
-      a=nlShort3(a);
-    }
+    a=nlShort3(a);
   }
   return a;
 }
@@ -2319,11 +2458,33 @@ number nlCopyMap(number a, const coeffs /*src*/, const coeffs /*dst*/)
   return _nlCopy_NoImm(a);
 }
 
-nMapFunc nlSetMap(const coeffs src, const coeffs /*dst*/)
+number nlMapQtoZ(number a, const coeffs src, const coeffs dst)
+{
+  if ((SR_HDL(a) & SR_INT)||(a==NULL))
+  {
+    return a;
+  }
+  if (a->s==3) return _nlCopy_NoImm(a);
+  number a0=a;
+  BOOLEAN a1=FALSE;
+  if (a->s==0) { a0=_nlCopy_NoImm(a); a1=TRUE; }
+  number b1=nlGetNumerator(a0,src);
+  number b2=nlGetDenom(a0,src);
+  number b=nlIntDiv(b1,b2,dst);
+  nlDelete(&b1,src);
+  nlDelete(&b2,src);
+  if (a1)  _nlDelete_NoImm(&a0);
+  return b;
+}
+
+nMapFunc nlSetMap(const coeffs src, const coeffs dst)
 {
   if (src->rep==n_rep_gap_rat)  /*Q, coeffs_BIGINT */
   {
-    return ndCopyMap;
+    if ((src->is_field==dst->is_field) /* Q->Q, Z->Z*/
+    || (src->is_field==FALSE))         /* Z->Q */
+      return nlCopyMap;
+    return nlMapQtoZ;        /* Q->Z */
   }
   if ((src->rep==n_rep_int) && nCoeff_is_Zp(src))
   {
@@ -2331,14 +2492,24 @@ nMapFunc nlSetMap(const coeffs src, const coeffs /*dst*/)
   }
   if ((src->rep==n_rep_float) && nCoeff_is_R(src))
   {
-    return nlMapR;
+    if (dst->is_field) /* R -> Q */
+      return nlMapR;
+    else
+      return nlMapR_BI; /* R -> bigint */
   }
   if ((src->rep==n_rep_gmp_float) && nCoeff_is_long_R(src))
   {
-    return nlMapLongR; /* long R -> Q */
+    if (dst->is_field)
+      return nlMapLongR; /* long R -> Q */
+    else
+      return nlMapLongR_BI;
+  }
+  if (nCoeff_is_long_C(src))
+  {
+    return nlMapC; /* C -> Q */
   }
 #ifdef HAVE_RINGS
-  if (src->rep==n_rep_gmp) // nCoeff_is_Ring_Z(src) || nCoeff_is_Ring_PtoM(src) || nCoeff_is_Ring_ModN(src))
+  if (src->rep==n_rep_gmp) // nCoeff_is_Z(src) || nCoeff_is_Ring_PtoM(src) || nCoeff_is_Zn(src))
   {
     return nlMapGMP;
   }
@@ -2481,7 +2652,7 @@ LINLINE BOOLEAN nlIsZero (number a, const coeffs)
 */
 LINLINE number nlCopy(number a, const coeffs)
 {
-  if ((SR_HDL(a) & SR_INT)||(a==NULL))
+  if (SR_HDL(a) & SR_INT)
   {
     return a;
   }
@@ -2645,7 +2816,7 @@ LINLINE void nlInpMult(number &a, number b, const coeffs r)
 
 #ifndef P_NUMBERS_H
 
-static void nlMPZ(mpz_t m, number &n, const coeffs r)
+void nlMPZ(mpz_t m, number &n, const coeffs r)
 {
   nlTest(n, r);
   nlNormalize(n, r);
@@ -2654,18 +2825,6 @@ static void nlMPZ(mpz_t m, number &n, const coeffs r)
 }
 
 
-static number nlInitMPZ(mpz_t m, const coeffs)
-{
-  number z = ALLOC_RNUMBER();
-  z->s = 3;
-  #ifdef LDEBUG
-  z->debug=123456;
-  #endif
-  mpz_init_set(z->z, m);
-  z=nlShort3(z);
-  return z;
-}
-
 number  nlXExtGcd (number a, number b, number *s, number *t, number *u, number *v, const coeffs r)
 {
   if (SR_HDL(a) & SR_HDL(b) & SR_INT)
@@ -2796,9 +2955,11 @@ void nlInpIntDiv(number &a, number b, const coeffs r)
   }
   else
   {
-    number rr=nlIntMod(a,b,r);
-    if (SR_HDL(rr) &  SR_INT) mpz_sub_ui(a->z,a->z,SR_TO_INT(rr));
-    else                      mpz_sub(a->z,a->z,rr->z);
+    mpz_t rr;
+    mpz_init(rr);
+    mpz_mod(rr,a->z,b->z);
+    mpz_sub(a->z,a->z,rr);
+    mpz_clear(rr);
     mpz_divexact(a->z,a->z,b->z);
     a=nlShort3_noinline(a);
   }
@@ -2842,8 +3003,8 @@ number nlFarey(number nN, number nP, const coeffs r)
          #ifdef LDEBUG
          z->debug=123456;
          #endif
-         mpz_init_set(z->z,N);
-         mpz_init_set(z->n,B);
+         memcpy(z->z,N,sizeof(mpz_t));
+         memcpy(z->n,B,sizeof(mpz_t));
          z->s = 0;
          nlNormalize(z,r);
        }
@@ -2851,6 +3012,8 @@ number nlFarey(number nN, number nP, const coeffs r)
        {
          // return nN (the input) instead of "fail"
          z=nlCopy(nN,r);
+         mpz_clear(B);
+         mpz_clear(N);
        }
        break;
     }
@@ -2866,11 +3029,9 @@ number nlFarey(number nN, number nP, const coeffs r)
   }
   mpz_clear(tmp);
   mpz_clear(A);
-  mpz_clear(B);
   mpz_clear(C);
   mpz_clear(D);
   mpz_clear(E);
-  mpz_clear(N);
   mpz_clear(P);
   return z;
 }
@@ -2924,15 +3085,13 @@ number nlExtGcd(number a, number b, number *s, number *t, const coeffs)
   return g;
 }
 
-void    nlCoeffWrite  (const coeffs r, BOOLEAN /*details*/)
-{
-  if (r->is_field)
-  PrintS("QQ");
-  else
-  PrintS("ZZ");
-}
+//void    nlCoeffWrite  (const coeffs r, BOOLEAN /*details*/)
+//{
+//  if (r->is_field)  PrintS("QQ");
+//  else  PrintS("ZZ");
+//}
 
-int n_SwitchChinRem=0;
+VAR int n_SwitchChinRem=0;
 number   nlChineseRemainderSym(number *x, number *q,int rl, BOOLEAN sym, CFArray &inv_cache,const coeffs CF)
 // elemenst in the array are x[0..(rl-1)], q[0..(rl-1)]
 {
@@ -3062,7 +3221,7 @@ static void nlClearContent(ICoeffsEnumerator& numberCollectionEnumerator, number
   while (numberCollectionEnumerator.MoveNext() )
   {
     number& n = numberCollectionEnumerator.Current();
-    number t=nlIntDiv(n, cand, cf); // simple integer exact division, no ratios to remain
+    number t=nlExactDiv(n, cand, cf); // simple integer exact division, no ratios to remain
     nlDelete(&n, cf);
     n = t;
   }
@@ -3168,33 +3327,26 @@ char * nlCoeffName(const coeffs r)
   else                 return (char*)"ZZ";
 }
 
-static char* nlCoeffString(const coeffs r)
-{
-  //return omStrDup(nlCoeffName(r));
-  if (r->cfDiv==nlDiv) return omStrDup("QQ");
-  else                 return omStrDup("ZZ");
-}
-
-static void nlWriteFd(number n,FILE* f, const coeffs)
+void nlWriteFd(number n, const ssiInfo* d, const coeffs)
 {
   if(SR_HDL(n) & SR_INT)
   {
     #if SIZEOF_LONG == 4
-    fprintf(f,"4 %ld ",SR_TO_INT(n));
+    fprintf(d->f_write,"4 %ld ",SR_TO_INT(n));
     #else
     long nn=SR_TO_INT(n);
     if ((nn<POW_2_28_32)&&(nn>= -POW_2_28_32))
     {
       int nnn=(int)nn;
-      fprintf(f,"4 %d ",nnn);
+      fprintf(d->f_write,"4 %d ",nnn);
     }
     else
     {
       mpz_t tmp;
       mpz_init_set_si(tmp,nn);
-      fputs("8 ",f);
-      mpz_out_str (f,SSI_BASE, tmp);
-      fputc(' ',f);
+      fputs("8 ",d->f_write);
+      mpz_out_str (d->f_write,SSI_BASE, tmp);
+      fputc(' ',d->f_write);
       mpz_clear(tmp);
     }
     #endif
@@ -3202,29 +3354,29 @@ static void nlWriteFd(number n,FILE* f, const coeffs)
   else if (n->s<2)
   {
     //gmp_fprintf(f,"%d %Zd %Zd ",n->s,n->z,n->n);
-    fprintf(f,"%d ",n->s+5);
-    mpz_out_str (f,SSI_BASE, n->z);
-    fputc(' ',f);
-    mpz_out_str (f,SSI_BASE, n->n);
-    fputc(' ',f);
+    fprintf(d->f_write,"%d ",n->s+5);
+    mpz_out_str (d->f_write,SSI_BASE, n->z);
+    fputc(' ',d->f_write);
+    mpz_out_str (d->f_write,SSI_BASE, n->n);
+    fputc(' ',d->f_write);
 
     //if (d->f_debug!=NULL) gmp_fprintf(d->f_debug,"number: s=%d gmp/gmp \"%Zd %Zd\" ",n->s,n->z,n->n);
   }
   else /*n->s==3*/
   {
     //gmp_fprintf(d->f_write,"3 %Zd ",n->z);
-    fputs("8 ",f);
-    mpz_out_str (f,SSI_BASE, n->z);
-    fputc(' ',f);
+    fputs("8 ",d->f_write);
+    mpz_out_str (d->f_write,SSI_BASE, n->z);
+    fputc(' ',d->f_write);
 
     //if (d->f_debug!=NULL) gmp_fprintf(d->f_debug,"number: gmp \"%Zd\" ",n->z);
   }
 }
 
-static number nlReadFd(s_buff f, const coeffs)
+number nlReadFd(const ssiInfo *d, const coeffs)
 {
   int sub_type=-1;
-  sub_type=s_readint(f);
+  sub_type=s_readint(d->f_read);
   switch(sub_type)
   {
      case 0:
@@ -3232,8 +3384,8 @@ static number nlReadFd(s_buff f, const coeffs)
        {// read mpz_t, mpz_t
          number n=nlRInit(0);
          mpz_init(n->n);
-         s_readmpz(f,n->z);
-         s_readmpz(f,n->n);
+         s_readmpz(d->f_read,n->z);
+         s_readmpz(d->f_read,n->n);
          n->s=sub_type;
          return n;
        }
@@ -3241,7 +3393,7 @@ static number nlReadFd(s_buff f, const coeffs)
      case 3:
        {// read mpz_t
          number n=nlRInit(0);
-         s_readmpz(f,n->z);
+         s_readmpz(d->f_read,n->z);
          n->s=3; /*sub_type*/
          #if SIZEOF_LONG == 8
          n=nlShort3(n);
@@ -3250,7 +3402,7 @@ static number nlReadFd(s_buff f, const coeffs)
        }
      case 4:
        {
-         LONG dd=s_readlong(f);
+         LONG dd=s_readlong(d->f_read);
          //#if SIZEOF_LONG == 8
          return INT_TO_SR(dd);
          //#else
@@ -3262,15 +3414,15 @@ static number nlReadFd(s_buff f, const coeffs)
        {// read raw mpz_t, mpz_t
          number n=nlRInit(0);
          mpz_init(n->n);
-         s_readmpz_base (f,n->z, SSI_BASE);
-         s_readmpz_base (f,n->n, SSI_BASE);
+         s_readmpz_base (d->f_read,n->z, SSI_BASE);
+         s_readmpz_base (d->f_read,n->n, SSI_BASE);
          n->s=sub_type-5;
          return n;
        }
      case 8:
        {// read raw mpz_t
          number n=nlRInit(0);
-         s_readmpz_base (f,n->z, SSI_BASE);
+         s_readmpz_base (d->f_read,n->z, SSI_BASE);
          n->s=sub_type=3; /*subtype-5*/
          #if SIZEOF_LONG == 8
          n=nlShort3(n);
@@ -3283,6 +3435,7 @@ static number nlReadFd(s_buff f, const coeffs)
   }
   return NULL;
 }
+
 BOOLEAN nlCoeffIsEqual(const coeffs r, n_coeffType n, void *p)
 {
   /* test, if r is an instance of nInitCoeffs(n,parameter) */
@@ -3299,7 +3452,7 @@ static number nlLcm(number a,number b,const coeffs r)
 {
   number g=nlGcd(a,b,r);
   number n1=nlMult(a,b,r);
-  number n2=nlIntDiv(n1,g,r);
+  number n2=nlExactDiv(n1,g,r);
   nlDelete(&g,r);
   nlDelete(&n1,r);
   return n2;
@@ -3326,7 +3479,7 @@ BOOLEAN nlInitChar(coeffs r, void*p)
 
   r->nCoeffIsEqual=nlCoeffIsEqual;
   //r->cfKillChar = ndKillChar; /* dummy */
-  r->cfCoeffString=nlCoeffString;
+  //r->cfCoeffString=nlCoeffString;
   r->cfCoeffName=nlCoeffName;
 
   r->cfInitMPZ = nlInitMPZ;
@@ -3388,7 +3541,7 @@ BOOLEAN nlInitChar(coeffs r, void*p)
   //r->cfName = ndName;
   r->cfInpMult=nlInpMult;
   r->cfInpAdd=nlInpAdd;
-  r->cfCoeffWrite=nlCoeffWrite;
+  //r->cfCoeffWrite=nlCoeffWrite;
 
   r->cfClearContent = nlClearContent;
   r->cfClearDenominators = nlClearDenominators;
