@@ -6,8 +6,6 @@
 */
 #include "misc/auxiliary.h"
 
-#include "omalloc/omalloc.h"
-
 #include "misc/mylimits.h"
 #include "reporter/reporter.h"
 
@@ -27,9 +25,10 @@
 #ifdef LDEBUG
 BOOLEAN nr2mDBTest(number a, const char *f, const int l, const coeffs r)
 {
-  if (((long)a<0L) || ((long)a>(long)r->mod2mMask))
+  if ((((long)a<0L) || ((long)a>(long)r->mod2mMask))
+  && (r->mod2mMask!= ~0UL))
   {
-    Print("wrong mod 2^n number %ld at %s,%d\n",(long)a,f,l);
+    Print("wrong mod 2^n number %ld (m:%ld) at %s,%d\n",(long)a,(long)r->mod2mMask,f,l);
     return FALSE;
   }
   return TRUE;
@@ -59,20 +58,18 @@ static inline number nr2mSubM(number a, number b, const coeffs r)
 #define nr2mNegM(A,r) (number)((r->mod2mMask+1 - (unsigned long)(A)) & r->mod2mMask)
 #define nr2mEqualM(A,B)  ((A)==(B))
 
-extern omBin gmp_nrz_bin; /* init in rintegers*/
+EXTERN_VAR omBin gmp_nrz_bin; /* init in rintegers*/
 
 static char* nr2mCoeffName(const coeffs cf)
 {
-  static char n2mCoeffName_buf[22];
-  snprintf(n2mCoeffName_buf,21,"ZZ/(2^%lu)",cf->modExponent);
+  STATIC_VAR char n2mCoeffName_buf[30];
+  if (cf->modExponent>32) /* for 32/64bit arch.*/
+    snprintf(n2mCoeffName_buf,21,"ZZ/(bigint(2)^%lu)",cf->modExponent);
+  else
+    snprintf(n2mCoeffName_buf,21,"ZZ/(2^%lu)",cf->modExponent);
   return n2mCoeffName_buf;
 }
 
-static void    nr2mCoeffWrite  (const coeffs r, BOOLEAN /*details*/)
-{
-  Print("ZZ/(2^%lu)", r->modExponent);
-}
-
 static BOOLEAN nr2mCoeffIsEqual(const coeffs r, n_coeffType n, void * p)
 {
   if (n==n_Z2m)
@@ -84,14 +81,6 @@ static BOOLEAN nr2mCoeffIsEqual(const coeffs r, n_coeffType n, void * p)
   return FALSE;
 }
 
-static char* nr2mCoeffString(const coeffs r)
-{
-  // r->modExponent <=bitsize(long)
-  char* s = (char*) omAlloc(11+11);
-  sprintf(s,"ZZ/(2^%lu)",r->modExponent);
-  return s;
-}
-
 static coeffs nr2mQuot1(number c, const coeffs r)
 {
   coeffs rr;
@@ -423,7 +412,12 @@ static number nr2mDiv(number a, number b, const coeffs r)
         b = (number)((unsigned long)b / 2);
       }
     }
-    if ((unsigned long)b % 2 == 0)
+    if ((long)b==0L)
+    {
+      WerrorS(nDivBy0);
+      return (number)0L;
+    }
+    else if ((unsigned long)b % 2 == 0)
     {
       WerrorS("Division not possible, even by cancelling zero divisors.");
       WerrorS("Result is integer division without remainder.");
@@ -646,8 +640,7 @@ static number nr2mMapGMP(number from, const coeffs /*src*/, const coeffs dst)
 static number nr2mMapQ(number from, const coeffs src, const coeffs dst)
 {
   mpz_ptr gmp = (mpz_ptr)omAllocBin(gmp_nrz_bin);
-  mpz_init(gmp);
-  nlGMP(from, gmp, src); // FIXME? TODO? // extern void   nlGMP(number &i, number n, const coeffs r); // to be replaced with n_MPZ(erg, from, src); // ?
+  nlMPZ(gmp, from, src);
   number res=nr2mMapGMP((number)gmp,src,dst);
   mpz_clear(gmp); omFree((ADDRESS)gmp);
   return res;
@@ -665,11 +658,6 @@ static number nr2mMapZ(number from, const coeffs src, const coeffs dst)
 
 static nMapFunc nr2mSetMap(const coeffs src, const coeffs dst)
 {
-  if ((src->rep==n_rep_int) && nCoeff_is_Ring_2toM(src)
-     && (src->mod2mMask == dst->mod2mMask))
-  {
-    return ndCopyMap;
-  }
   if ((src->rep==n_rep_int) && nCoeff_is_Ring_2toM(src)
      && (src->mod2mMask < dst->mod2mMask))
   { /* i.e. map an integer mod 2^s into Z mod 2^t, where t < s */
@@ -681,15 +669,15 @@ static nMapFunc nr2mSetMap(const coeffs src, const coeffs dst)
     // to be done
     return nr2mMapProject;
   }
-  if ((src->rep==n_rep_gmp) && nCoeff_is_Ring_Z(src))
+  if ((src->rep==n_rep_gmp) && nCoeff_is_Z(src))
   {
     return nr2mMapGMP;
   }
-  if ((src->rep==n_rep_gap_gmp) /*&& nCoeff_is_Ring_Z(src)*/)
+  if ((src->rep==n_rep_gap_gmp) /*&& nCoeff_is_Z(src)*/)
   {
     return nr2mMapZ;
   }
-  if ((src->rep==n_rep_gap_rat) && (nCoeff_is_Q(src)||nCoeff_is_Ring_Z(src)))
+  if ((src->rep==n_rep_gap_rat) && (nCoeff_is_Q(src)||nCoeff_is_Z(src)))
   {
     return nr2mMapQ;
   }
@@ -698,7 +686,7 @@ static nMapFunc nr2mSetMap(const coeffs src, const coeffs dst)
     return nr2mMapZp;
   }
   if ((src->rep==n_rep_gmp) &&
-  (nCoeff_is_Ring_PtoM(src) || nCoeff_is_Ring_ModN(src)))
+  (nCoeff_is_Ring_PtoM(src) || nCoeff_is_Zn(src)))
   {
     if (mpz_divisible_2exp_p(src->modNumber,dst->modExponent))
       return nr2mMapGMP;
@@ -790,7 +778,6 @@ BOOLEAN nr2mInitChar (coeffs r, void* p)
 
   //r->cfKillChar    = ndKillChar; /* dummy*/
   r->nCoeffIsEqual = nr2mCoeffIsEqual;
-  r->cfCoeffString = nr2mCoeffString;
 
   r->modBase = (mpz_ptr) omAllocBin (gmp_nrz_bin);
   mpz_init_set_si (r->modBase, 2L);
@@ -831,7 +818,6 @@ BOOLEAN nr2mInitChar (coeffs r, void* p)
   r->cfIsUnit      = nr2mIsUnit;
   r->cfGetUnit     = nr2mGetUnit;
   r->cfExtGcd      = nr2mExtGcd;
-  r->cfCoeffWrite  = nr2mCoeffWrite;
   r->cfCoeffName   = nr2mCoeffName;
   r->cfQuot1       = nr2mQuot1;
 #ifdef LDEBUG
