@@ -12,14 +12,16 @@
 #include "clapsing.h"
 
 #include "factory/factory.h"
+#include "factory/cf_roots.h"
 
-#include "omalloc/omalloc.h"
 #include "coeffs/numbers.h"
 #include "coeffs/coeffs.h"
 #include "coeffs/bigintmat.h"
 
 #include "monomials/ring.h"
 #include "simpleideals.h"
+#include "polys/flintconv.h"
+#include "polys/flint_mpoly.h"
 
 
 //#include "polys.h"
@@ -55,19 +57,59 @@ poly singclap_gcd_r ( poly f, poly g, const ring r )
   {
     return p_GcdMon(g,f,r);
   }
-
-  Off(SW_RATIONAL);
-  if (rField_is_Q(r) || rField_is_Zp(r) || rField_is_Ring_Z(r))
+  #ifdef HAVE_FLINT
+  #if __FLINT_RELEASE >= 20503
+  if (rField_is_Zp(r) && (r->cf->ch>10))
   {
-    setCharacteristic( rChar(r) );
+    nmod_mpoly_ctx_t ctx;
+    if (!convSingRFlintR(ctx,r))
+    {
+      // leading coef. 1
+      return Flint_GCD_MP(f,pLength(f),g,pLength(g),ctx,r);
+    }
+  }
+  else
+  if (rField_is_Q(r))
+  {
+    fmpq_mpoly_ctx_t ctx;
+    if (!convSingRFlintR(ctx,r))
+    {
+      // leading coef. positive, all coeffs in Z
+      poly res=Flint_GCD_MP(f,pLength(f),g,pLength(g),ctx,r);
+      res=p_Cleardenom(res,r);
+      return res;
+    }
+  }
+  else
+  if (rField_is_Z(r))
+  {
+    fmpz_mpoly_ctx_t ctx;
+    if (!convSingRFlintR(ctx,r))
+    {
+      // leading coef. positive, all coeffs in Z
+      poly res=Flint_GCD_MP(f,pLength(f),g,pLength(g),ctx,r);
+      return res;
+    }
+  }
+  #endif
+  #endif
+  Off(SW_RATIONAL);
+  if (rField_is_Q(r) || rField_is_Zp(r) || rField_is_Z(r)
+  || (rField_is_Zn(r)&&(r->cf->convSingNFactoryN!=ndConvSingNFactoryN)))
+  {
+    setCharacteristic( rInternalChar(r) );
     CanonicalForm F( convSingPFactoryP( f,r ) ), G( convSingPFactoryP( g, r ) );
     res=convFactoryPSingP( gcd( F, G ) , r);
+    if ( rField_is_Zp(r))
+      p_Norm(res,r); // leading coef. 1
+    else if (rField_is_Q(r) && (!n_GreaterZero(pGetCoeff(res),r->cf)))
+      res = p_Neg(res,r); // leading coef. positive, all coeffs in Z
   }
   // and over Q(a) / Fp(a)
   else if ( r->cf->extRing!=NULL )
   {
     if ( rField_is_Q_a(r)) setCharacteristic( 0 );
-    else                   setCharacteristic( rChar(r) );
+    else                   setCharacteristic( rInternalChar(r) );
     if (r->cf->extRing->qideal!=NULL)
     {
       bool b1=isOn(SW_USE_QGCD);
@@ -80,15 +122,24 @@ poly singclap_gcd_r ( poly f, poly g, const ring r )
       res= convFactoryAPSingAP( gcd( F, G ),r );
       prune (a);
       if (!b1) Off(SW_USE_QGCD);
+      if ( rField_is_Zp_a(r)) p_Norm(res,r); // leading coef. 1
     }
     else
     {
+      convSingTrP(f,r);
+      convSingTrP(g,r);
       CanonicalForm F( convSingTrPFactoryP( f,r ) ), G( convSingTrPFactoryP( g,r ) );
       res= convFactoryPSingTrP( gcd( F, G ),r );
     }
   }
-  else
+  else if (r->cf->convSingNFactoryN==ndConvSingNFactoryN)
     WerrorS( feNotImplemented );
+  else
+  { // handle user type coeffs:
+    setCharacteristic( rInternalChar(r) );
+    CanonicalForm F( convSingPFactoryP( f,r ) ), G( convSingPFactoryP( g, r ) );
+    res=convFactoryPSingP( gcd( F, G ) , r);
+  }
   Off(SW_RATIONAL);
   return res;
 }
@@ -134,10 +185,11 @@ poly singclap_gcd_and_divide ( poly& f, poly& g, const ring r)
 
   Off(SW_RATIONAL);
   CanonicalForm F,G,GCD;
-  if (rField_is_Q(r) || (rField_is_Zp(r)))
+  if (rField_is_Q(r) || (rField_is_Zp(r))
+  || (rField_is_Zn(r)&&(r->cf->convSingNFactoryN!=ndConvSingNFactoryN)))
   {
     bool b1=isOn(SW_USE_EZGCD_P);
-    setCharacteristic( rChar(r) );
+    setCharacteristic( rInternalChar(r) );
     F=convSingPFactoryP( f,r );
     G=convSingPFactoryP( g,r );
     GCD=gcd(F,G);
@@ -173,7 +225,7 @@ poly singclap_gcd_and_divide ( poly& f, poly& g, const ring r)
   else if ( r->cf->extRing )
   {
     if ( rField_is_Q_a(r)) setCharacteristic( 0 );
-    else                   setCharacteristic( rChar(r) );
+    else                   setCharacteristic( rInternalChar(r) );
     if (r->cf->extRing->qideal!=NULL)
     {
       bool b1=isOn(SW_USE_QGCD);
@@ -252,31 +304,6 @@ poly singclap_gcd_and_divide ( poly& f, poly& g, const ring r)
   return res;
 }
 
-poly singclap_gcd ( poly f, poly g, const ring r)
-{
-  poly res=NULL;
-
-  if (f!=NULL)
-  {
-    //if (r->cf->has_simple_Inverse) p_Norm(f,r);
-    if (rField_is_Zp(r)) p_Norm(f,r);
-    else                 p_Cleardenom(f, r);
-  }
-  if (g!=NULL)
-  {
-    //if (r->cf->has_simple_Inverse) p_Norm(g,r);
-    if (rField_is_Zp(r)) p_Norm(g,r);
-    else                 p_Cleardenom(g, r);
-  }
-  else         return f; // g==0 => gcd=f (but do a p_Cleardenom/pNorm)
-  if (f==NULL) return g; // f==0 => gcd=g (but do a p_Cleardenom/pNorm)
-
-  res=singclap_gcd_r(f,g,r);
-  p_Delete(&f, r);
-  p_Delete(&g, r);
-  return res;
-}
-
 /*2 find the maximal exponent of var(i) in poly p*/
 int pGetExp_Var(poly p, int i, const ring r)
 {
@@ -305,10 +332,11 @@ poly singclap_resultant ( poly f, poly g , poly x, const ring r)
     goto resultant_returns_res;
   // for now there is only the possibility to handle polynomials over
   // Q and Fp ...
-  if (rField_is_Zp(r) || rField_is_Q(r))
+  if (rField_is_Zp(r) || rField_is_Q(r)
+  || (rField_is_Zn(r)&&(r->cf->convSingNFactoryN!=ndConvSingNFactoryN)))
   {
     Variable X(i);
-    setCharacteristic( rChar(r) );
+    setCharacteristic( rInternalChar(r) );
     CanonicalForm F( convSingPFactoryP( f,r ) ), G( convSingPFactoryP( g,r ) );
     res=convFactoryPSingP( resultant( F, G, X),r );
     Off(SW_RATIONAL);
@@ -318,7 +346,7 @@ poly singclap_resultant ( poly f, poly g , poly x, const ring r)
   else if (r->cf->extRing!=NULL)
   {
     if (rField_is_Q_a(r)) setCharacteristic( 0 );
-    else               setCharacteristic( rChar(r) );
+    else               setCharacteristic( rInternalChar(r) );
     Variable X(i+rPar(r));
     if (r->cf->extRing->qideal!=NULL)
     {
@@ -442,9 +470,10 @@ BOOLEAN singclap_extgcd ( poly f, poly g, poly &res, poly &pa, poly &pb , const
   // Q and Fp ...
   res=NULL;pa=NULL;pb=NULL;
   On(SW_SYMMETRIC_FF);
-  if ( rField_is_Q(r) || rField_is_Zp(r) )
+  if ( rField_is_Q(r) || rField_is_Zp(r)
+  || (rField_is_Zn(r)&&(r->cf->convSingNFactoryN!=ndConvSingNFactoryN)))
   {
-    setCharacteristic( rChar(r) );
+    setCharacteristic( rInternalChar(r) );
     CanonicalForm F( convSingPFactoryP( f,r ) ), G( convSingPFactoryP( g,r) );
     CanonicalForm FpG=F+G;
     if (!(FpG.isUnivariate()|| FpG.inCoeffDomain()))
@@ -465,7 +494,7 @@ BOOLEAN singclap_extgcd ( poly f, poly g, poly &res, poly &pa, poly &pb , const
   else if ( r->cf->extRing!=NULL )
   {
     if (rField_is_Q_a(r)) setCharacteristic( 0 );
-    else                 setCharacteristic( rChar(r) );
+    else                 setCharacteristic( rInternalChar(r) );
     CanonicalForm Fa,Gb;
     if (r->cf->extRing->qideal!=NULL)
     {
@@ -526,23 +555,18 @@ poly singclap_pmult ( poly f, poly g, const ring r )
 {
   poly res=NULL;
   On(SW_RATIONAL);
-  if (rField_is_Zp(r) || rField_is_Q(r))
+  if (rField_is_Zp(r) || rField_is_Q(r) || rField_is_Z(r)
+  || (rField_is_Zn(r)&&(r->cf->convSingNFactoryN!=ndConvSingNFactoryN)))
   {
-    setCharacteristic( rChar(r) );
-    CanonicalForm F( convSingPFactoryP( f,r ) ), G( convSingPFactoryP( g,r ) );
-    res = convFactoryPSingP( F * G,r );
-  }
-  else if (rField_is_Ring_Z(r))
-  {
-    Off(SW_RATIONAL);
-    setCharacteristic( rChar(r) );
+    if (rField_is_Z(r)) Off(SW_RATIONAL);
+    setCharacteristic( rInternalChar(r) );
     CanonicalForm F( convSingPFactoryP( f,r ) ), G( convSingPFactoryP( g,r ) );
     res = convFactoryPSingP( F * G,r );
   }
   else if (r->cf->extRing!=NULL)
   {
     if (rField_is_Q_a(r)) setCharacteristic( 0 );
-    else               setCharacteristic( rChar(r) );
+    else               setCharacteristic( rInternalChar(r) );
     if (r->cf->extRing->qideal!=NULL)
     {
       CanonicalForm mipo=convSingPFactoryP(r->cf->extRing->qideal->m[0],
@@ -577,10 +601,42 @@ poly singclap_pmult ( poly f, poly g, const ring r )
 poly singclap_pdivide ( poly f, poly g, const ring r )
 {
   poly res=NULL;
-  On(SW_RATIONAL);
-  if (rField_is_Zp(r) || rField_is_Q(r))
+
+  #ifdef HAVE_FLINT
+  #if __FLINT_RELEASE >= 20503
+  /*
+    If the division is not exact, control will pass to factory where the
+    polynomials can be divided using the ordering that factory chooses.
+  */
+  if (rField_is_Zp(r))
   {
-    setCharacteristic( rChar(r) );
+    nmod_mpoly_ctx_t ctx;
+    if (!convSingRFlintR(ctx,r))
+    {
+      res = Flint_Divide_MP(f,0,g,0,ctx,r);
+      if (res != NULL)
+        return res;
+    }
+  }
+  else
+  if (rField_is_Q(r))
+  {
+    fmpq_mpoly_ctx_t ctx;
+    if (!convSingRFlintR(ctx,r))
+    {
+      res = Flint_Divide_MP(f,0,g,0,ctx,r);
+      if (res != NULL)
+        return res;
+    }
+  }
+  #endif
+  #endif
+
+  On(SW_RATIONAL);
+  if (rField_is_Zp(r) || rField_is_Q(r)
+  || (rField_is_Zn(r)&&(r->cf->convSingNFactoryN!=ndConvSingNFactoryN)))
+  {
+    setCharacteristic( rInternalChar(r) );
     CanonicalForm F( convSingPFactoryP( f,r ) ), G( convSingPFactoryP( g,r ) );
     res = convFactoryPSingP( F / G,r );
   }
@@ -588,7 +644,7 @@ poly singclap_pdivide ( poly f, poly g, const ring r )
   else if (r->cf->extRing!=NULL)
   {
     if (rField_is_Q_a(r)) setCharacteristic( 0 );
-    else               setCharacteristic( rChar(r) );
+    else               setCharacteristic( rInternalChar(r) );
     if (r->cf->extRing->qideal!=NULL)
     {
       CanonicalForm mipo=convSingPFactoryP(r->cf->extRing->qideal->m[0],
@@ -624,17 +680,21 @@ poly singclap_pmod ( poly f, poly g, const ring r )
 {
   poly res=NULL;
   On(SW_RATIONAL);
-  if (rField_is_Zp(r) || rField_is_Q(r))
+  if (rField_is_Zp(r) || rField_is_Q(r)
+  || (rField_is_Zn(r)&&(r->cf->convSingNFactoryN!=ndConvSingNFactoryN)))
   {
-    setCharacteristic( rChar(r) );
+    setCharacteristic( rInternalChar(r) );
     CanonicalForm F( convSingPFactoryP( f,r ) ), G( convSingPFactoryP( g,r ) );
-    res = convFactoryPSingP( F % G,r );
+    CanonicalForm Q,R;
+    divrem(F,G,Q,R);
+    res = convFactoryPSingP(R,r);
+    //res = convFactoryPSingP( F-(F/G)*G,r );
   }
   // mod is not implemented for ZZ coeffs in factory
   else if (r->cf->extRing!=NULL)
   {
     if (rField_is_Q_a(r)) setCharacteristic( 0 );
-    else               setCharacteristic( rChar(r) );
+    else               setCharacteristic( rInternalChar(r) );
     if (r->cf->extRing->qideal!=NULL)
     {
       CanonicalForm mipo=convSingPFactoryP(r->cf->extRing->qideal->m[0],
@@ -642,13 +702,19 @@ poly singclap_pmod ( poly f, poly g, const ring r )
       Variable a=rootOf(mipo);
       CanonicalForm F( convSingAPFactoryAP( f,a,r ) ),
                     G( convSingAPFactoryAP( g,a,r ) );
-      res= convFactoryAPSingAP(  F % G, r  );
+      CanonicalForm Q,R;
+      divrem(F,G,Q,R);
+      res = convFactoryAPSingAP(R,r);
+      //res= convFactoryAPSingAP( F-(F/G)*G, r  );
       prune (a);
     }
     else
     {
       CanonicalForm F( convSingTrPFactoryP( f,r ) ), G( convSingTrPFactoryP( g,r ) );
-      res= convFactoryPSingTrP(  F % G,r  );
+      CanonicalForm Q,R;
+      divrem(F,G,Q,R);
+      res = convFactoryPSingTrP(R,r);
+      //res= convFactoryPSingTrP(  F-(F/G)*G,r  );
     }
   }
 #if 0 // not yet working
@@ -666,6 +732,8 @@ poly singclap_pmod ( poly f, poly g, const ring r )
   return res;
 }
 
+#if 0
+// unused
 void singclap_divide_content ( poly f, const ring r )
 {
   if ( f==NULL )
@@ -762,13 +830,14 @@ void singclap_divide_content ( poly f, const ring r )
     // pTest(f);
   }
 }
+#endif
 
 BOOLEAN count_Factors(ideal I, intvec *v,int j, poly &f, poly fac, const ring r)
 {
   p_Test(f,r);
   p_Test(fac,r);
   int e=0;
-  if (!p_IsConstantPoly(fac,r))
+  if (!p_IsConstant(fac,r))
   {
 #ifdef FACTORIZE2_DEBUG
     printf("start count_Factors(%d), Fdeg=%d, factor deg=%d\n",j,p_Totaldegree(f,r),p_Totaldegree(fac,r));
@@ -777,7 +846,8 @@ BOOLEAN count_Factors(ideal I, intvec *v,int j, poly &f, poly fac, const ring r)
     On(SW_RATIONAL);
     CanonicalForm F, FAC,Q,R;
     Variable a;
-    if (rField_is_Zp(r) || rField_is_Q(r))
+    if (rField_is_Zp(r) || rField_is_Q(r)
+    || (rField_is_Zn(r)&&(r->cf->convSingNFactoryN!=ndConvSingNFactoryN)))
     {
       F=convSingPFactoryP( f,r );
       FAC=convSingPFactoryP( fac,r );
@@ -811,7 +881,8 @@ BOOLEAN count_Factors(ideal I, intvec *v,int j, poly &f, poly fac, const ring r)
       R-=F;
       if (R.isZero())
       {
-        if (rField_is_Zp(r) || rField_is_Q(r))
+        if (rField_is_Zp(r) || rField_is_Q(r)
+        || (rField_is_Zn(r)&&(r->cf->convSingNFactoryN!=ndConvSingNFactoryN)))
         {
           q = convFactoryPSingP( Q,r );
         }
@@ -849,7 +920,7 @@ BOOLEAN count_Factors(ideal I, intvec *v,int j, poly &f, poly fac, const ring r)
   return TRUE;
 }
 
-int singclap_factorize_retry;
+VAR int singclap_factorize_retry;
 
 ideal singclap_factorize ( poly f, intvec ** v , int with_exps, const ring r)
 /* destroys f, sets *v */
@@ -901,7 +972,7 @@ ideal singclap_factorize ( poly f, intvec ** v , int with_exps, const ring r)
     }
     if (n==0)
     {
-      res->m[0]=p_One(r);
+      if (res->m[0]==NULL) res->m[0]=p_One(r);
       // (**v)[0]=1; is already done
     }
     else
@@ -925,6 +996,7 @@ ideal singclap_factorize ( poly f, intvec ** v , int with_exps, const ring r)
   }
   //PrintS("S:");p_Write(f,r);PrintLn();
   // use factory/libfac in general ==============================
+  Variable dummy(-1); prune(dummy); // remove all (tmp.) extensions
   Off(SW_RATIONAL);
   On(SW_SYMMETRIC_FF);
   CFFList L;
@@ -933,81 +1005,88 @@ ideal singclap_factorize ( poly f, intvec ** v , int with_exps, const ring r)
   number old_lead_coeff=n_Copy(pGetCoeff(f), r->cf);
 
   Variable a;
-  if (!rField_is_Zp(r) && !rField_is_Zp_a(r) && !rField_is_Z(r)) /* Q, Q(a) */
+  if (r->cf->convSingNFactoryN!=ndConvSingNFactoryN)
   {
-    //if (f!=NULL) // already tested at start of routine
+    if (rField_is_Q(r) || rField_is_Q_a(r) || rField_is_Z(r)) /* Q, Q(a), Z */
     {
-      number n0=n_Copy(pGetCoeff(f),r->cf);
-      if (with_exps==0)
-        N=n_Copy(n0,r->cf);
-      p_Cleardenom(f, r);
-      //after here f should not have a denominator!!
-      //PrintS("S:");p_Write(f,r);PrintLn();
-      NN=n_Div(n0,pGetCoeff(f),r->cf);
-      n_Delete(&n0,r->cf);
-      if (with_exps==0)
+      //if (f!=NULL) // already tested at start of routine
       {
-        n_Delete(&N,r->cf);
-        N=n_Copy(NN,r->cf);
+        number n0=n_Copy(pGetCoeff(f),r->cf);
+        if (with_exps==0)
+          N=n_Copy(n0,r->cf);
+        if (rField_is_Z(r)) p_Content(f, r);
+        else                p_Cleardenom(f, r);
+        //after here f should not have a denominator!! and no content
+        //PrintS("S:");p_Write(f,r);PrintLn();
+        NN=n_Div(n0,pGetCoeff(f),r->cf);
+        n_Delete(&n0,r->cf);
+        if (with_exps==0)
+        {
+          n_Delete(&N,r->cf);
+          N=n_Copy(NN,r->cf);
+        }
       }
     }
-  }
-  else if (rField_is_Zp_a(r))
-  {
-    //if (f!=NULL) // already tested at start of routine
-    if (singclap_factorize_retry==0)
+    else if (rField_is_Zp_a(r))
     {
-      number n0=n_Copy(pGetCoeff(f),r->cf);
-      if (with_exps==0)
-        N=n_Copy(n0,r->cf);
-      p_Norm(f,r);
-      p_Cleardenom(f, r);
-      NN=n_Div(n0,pGetCoeff(f),r->cf);
-      n_Delete(&n0,r->cf);
-      if (with_exps==0)
+      //if (f!=NULL) // already tested at start of routine
+      if (singclap_factorize_retry==0)
       {
-        n_Delete(&N,r->cf);
-        N=n_Copy(NN,r->cf);
+        number n0=n_Copy(pGetCoeff(f),r->cf);
+        if (with_exps==0)
+          N=n_Copy(n0,r->cf);
+        p_Norm(f,r);
+        p_Cleardenom(f, r);
+        NN=n_Div(n0,pGetCoeff(f),r->cf);
+        n_Delete(&n0,r->cf);
+        if (with_exps==0)
+        {
+          n_Delete(&N,r->cf);
+          N=n_Copy(NN,r->cf);
+        }
       }
     }
-  }
-  if (rField_is_Q(r) || rField_is_Zp(r) || (rField_is_Z(r)))
-  {
-    setCharacteristic( rChar(r) );
-    CanonicalForm F( convSingPFactoryP( f,r ) );
-    L = factorize( F );
-  }
-  // and over Q(a) / Fp(a)
-  else if (r->cf->extRing!=NULL)
-  {
-    if (rField_is_Q_a (r)) setCharacteristic (0);
-    else                   setCharacteristic( rChar(r) );
-    if (r->cf->extRing->qideal!=NULL)
+    if (rField_is_Q(r) || rField_is_Zp(r) || rField_is_Z(r) || rField_is_Zn(r))
     {
-      CanonicalForm mipo=convSingPFactoryP(r->cf->extRing->qideal->m[0],
-                                           r->cf->extRing);
-      a=rootOf(mipo);
-      CanonicalForm F( convSingAPFactoryAP( f, a, r ) );
-      if (rField_is_Zp_a(r))
+      setCharacteristic( rInternalChar(r) );
+      if (errorreported) goto notImpl; // char too large
+      CanonicalForm F( convSingPFactoryP( f,r ) );
+      L = factorize( F );
+    }
+    // and over Q(a) / Fp(a)
+    else if (r->cf->extRing!=NULL)
+    {
+      if (rField_is_Q_a (r)) setCharacteristic (0);
+      else                   setCharacteristic( rInternalChar(r) );
+      if (errorreported) goto notImpl; // char too large
+      if (r->cf->extRing->qideal!=NULL) /*algebraic extension */
       {
+        CanonicalForm mipo=convSingPFactoryP(r->cf->extRing->qideal->m[0],
+                                             r->cf->extRing);
+        a=rootOf(mipo);
+        CanonicalForm F( convSingAPFactoryAP( f, a, r ) );
         L = factorize( F, a );
+        prune(a);
       }
-      else
+      else /* rational functions */
       {
-        //  over Q(a)
-        L= factorize (F, a);
+        CanonicalForm F( convSingTrPFactoryP( f,r ) );
+        L = factorize( F );
       }
     }
     else
     {
-      CanonicalForm F( convSingTrPFactoryP( f,r ) );
-      L = factorize( F );
+      goto notImpl;
     }
   }
   else
   {
     goto notImpl;
   }
+  if (errorreported)
+  {
+    errorreported=FALSE;
+  }
   {
     poly ff=p_Copy(f,r); // a copy for the retry stuff
     // the first factor should be a constant
@@ -1031,7 +1110,8 @@ ideal singclap_factorize ( poly f, intvec ** v , int with_exps, const ring r)
     for ( ; J.hasItem(); J++, j++ )
     {
       if (with_exps!=1) (**v)[j] = J.getItem().exp();
-      if (rField_is_Zp(r) || rField_is_Q(r)||  rField_is_Z(r))           /* Q, Fp, Z */
+      if (rField_is_Zp(r) || rField_is_Q(r)||  rField_is_Z(r)
+      || rField_is_Zn(r))           /* Q, Fp, Z */
       {
         //count_Factors(res,*v,f, j, convFactoryPSingP( J.getItem().factor() );
         res->m[j] = convFactoryPSingP( J.getItem().factor(),r );
@@ -1078,7 +1158,7 @@ ideal singclap_factorize ( poly f, intvec ** v , int with_exps, const ring r)
       if (r->cf->extRing->qideal!=NULL)
         prune (a);
 #ifndef SING_NDEBUG
-    if ((r->cf->extRing!=NULL) && (!p_IsConstantPoly(ff,r)))
+    if ((r->cf->extRing!=NULL) && (!p_IsConstant(ff,r)))
     {
       singclap_factorize_retry++;
       if (singclap_factorize_retry<3)
@@ -1207,8 +1287,20 @@ ideal singclap_factorize ( poly f, intvec ** v , int with_exps, const ring r)
     n_Delete(&old_lead_coeff,r->cf);
   errorreported=save_errorreported;
 notImpl:
+  prune(a);
   if (res==NULL)
+  {
     WerrorS( feNotImplemented );
+    if ((v!=NULL) && ((*v)!=NULL) &&(with_exps==2))
+    {
+       *v = new intvec( 1 );
+       (*v)[0]=1;
+    }
+    res=idInit(2,1);
+    res->m[0]=p_One(r);
+    res->m[1]=f;
+  }
+  else p_Delete(&f,r);
   if (NN!=NULL)
   {
     n_Delete(&NN,r->cf);
@@ -1217,8 +1309,6 @@ notImpl:
   {
     n_Delete(&N,r->cf);
   }
-  if (f!=NULL) p_Delete(&f,r);
-  //PrintS("......S\n");
   return res;
 }
 
@@ -1298,7 +1388,7 @@ ideal singclap_sqrfree ( poly f, intvec ** v , int with_exps, const ring r)
   if (!rField_is_Zp(r) && !rField_is_Zp_a(r)) /* Q, Q(a) */
   {
     //if (f!=NULL) // already tested at start of routine
-    number n0=n_Copy(pGetCoeff(f),r->cf);
+    number n0=n_Copy(old_lead_coeff,r->cf);
     if (with_exps==0 || with_exps==3)
       N=n_Copy(n0,r->cf);
     p_Cleardenom(f, r);
@@ -1317,7 +1407,7 @@ ideal singclap_sqrfree ( poly f, intvec ** v , int with_exps, const ring r)
     //if (f!=NULL) // already tested at start of routine
     if (singclap_factorize_retry==0)
     {
-      number n0=n_Copy(pGetCoeff(f),r->cf);
+      number n0=n_Copy(old_lead_coeff,r->cf);
       if (with_exps==0 || with_exps==3)
         N=n_Copy(n0,r->cf);
       p_Norm(f,r);
@@ -1331,16 +1421,17 @@ ideal singclap_sqrfree ( poly f, intvec ** v , int with_exps, const ring r)
       }
     }
   }
-  if (rField_is_Q(r) || rField_is_Zp(r))
+  if (rField_is_Q(r) || rField_is_Zp(r)
+  || (rField_is_Zn(r)&&(r->cf->convSingNFactoryN!=ndConvSingNFactoryN)))
   {
-    setCharacteristic( rChar(r) );
+    setCharacteristic( rInternalChar(r) );
     CanonicalForm F( convSingPFactoryP( f,r ) );
     L = sqrFree( F );
   }
   else if (r->cf->extRing!=NULL)
   {
     if (rField_is_Q_a (r)) setCharacteristic (0);
-    else                   setCharacteristic( rChar(r) );
+    else                   setCharacteristic( rInternalChar(r) );
     if (r->cf->extRing->qideal!=NULL)
     {
       CanonicalForm mipo=convSingPFactoryP(r->cf->extRing->qideal->m[0],
@@ -1358,7 +1449,7 @@ ideal singclap_sqrfree ( poly f, intvec ** v , int with_exps, const ring r)
   #if 0
   else if (rField_is_GF())
   {
-    int c=rChar(r);
+    int c=rInternalChar(r);
     setCharacteristic( c, primepower(c) );
     CanonicalForm F( convSingGFFactoryGF( f ) );
     if (F.isUnivariate())
@@ -1399,7 +1490,8 @@ ideal singclap_sqrfree ( poly f, intvec ** v , int with_exps, const ring r)
     for ( ; J.hasItem(); J++, j++ )
     {
       if (with_exps!=1 && with_exps!=3) (**v)[j] = J.getItem().exp();
-      if (rField_is_Zp(r) || rField_is_Q(r))
+      if (rField_is_Zp(r) || rField_is_Q(r)
+      || (rField_is_Zn(r)&&(r->cf->convSingNFactoryN!=ndConvSingNFactoryN)))
         res->m[j] = convFactoryPSingP( J.getItem().factor(),r );
       else if (r->cf->extRing!=NULL)     /* Q(a), Fp(a) */
       {
@@ -1465,9 +1557,10 @@ matrix singclap_irrCharSeries ( ideal I, const ring r)
   On(SW_SYMMETRIC_FF);
   CFList L;
   ListCFList LL;
-  if (rField_is_Q(r) || rField_is_Zp(r))
+  if (rField_is_Q(r) || rField_is_Zp(r)
+  || (rField_is_Zn(r)&&(r->cf->convSingNFactoryN!=ndConvSingNFactoryN)))
   {
-    setCharacteristic( rChar(r) );
+    setCharacteristic( rInternalChar(r) );
     for(i=0;i<IDELEMS(I);i++)
     {
       poly p=I->m[i];
@@ -1476,14 +1569,14 @@ matrix singclap_irrCharSeries ( ideal I, const ring r)
         p=p_Copy(p,r);
         p_Cleardenom(p, r);
         L.append(convSingPFactoryP(p,r));
-	p_Delete(&p,r);
+        p_Delete(&p,r);
       }
     }
   }
   // and over Q(a) / Fp(a)
   else if (nCoeff_is_transExt (r->cf))
   {
-    setCharacteristic( rChar(r) );
+    setCharacteristic( rInternalChar(r) );
     for(i=0;i<IDELEMS(I);i++)
     {
       poly p=I->m[i];
@@ -1492,7 +1585,7 @@ matrix singclap_irrCharSeries ( ideal I, const ring r)
         p=p_Copy(p,r);
         p_Cleardenom(p, r);
         L.append(convSingTrPFactoryP(p,r));
-	p_Delete(&p,r);
+        p_Delete(&p,r);
       }
     }
   }
@@ -1534,7 +1627,8 @@ matrix singclap_irrCharSeries ( ideal I, const ring r)
   {
     for (n=1, Li = LLi.getItem(); Li.hasItem(); Li++, n++)
     {
-      if (rField_is_Q(r) || rField_is_Zp(r))
+      if (rField_is_Q(r) || rField_is_Zp(r)
+      || (rField_is_Zn(r)&&(r->cf->convSingNFactoryN!=ndConvSingNFactoryN)))
         MATELEM(res,m,n)=convFactoryPSingP(Li.getItem(),r);
       else
         MATELEM(res,m,n)=convFactoryPSingTrP(Li.getItem(),r);
@@ -1550,9 +1644,10 @@ char* singclap_neworder ( ideal I, const ring r)
   Off(SW_RATIONAL);
   On(SW_SYMMETRIC_FF);
   CFList L;
-  if (rField_is_Q(r) || rField_is_Zp(r))
+  if (rField_is_Q(r) || rField_is_Zp(r)
+  || (rField_is_Zn(r)&&(r->cf->convSingNFactoryN!=ndConvSingNFactoryN)))
   {
-    setCharacteristic( rChar(r) );
+    setCharacteristic( rInternalChar(r) );
     for(i=0;i<IDELEMS(I);i++)
     {
       poly p=I->m[i];
@@ -1567,7 +1662,7 @@ char* singclap_neworder ( ideal I, const ring r)
   // and over Q(a) / Fp(a)
   else if (nCoeff_is_transExt (r->cf))
   {
-    setCharacteristic( rChar(r) );
+    setCharacteristic( rInternalChar(r) );
     for(i=0;i<IDELEMS(I);i++)
     {
       poly p=I->m[i];
@@ -1695,7 +1790,7 @@ number singclap_det_bi( bigintmat * m, const coeffs cf)
   return res;
 }
 
-#ifdef HAVE_NTL
+#if defined(HAVE_NTL) || defined(AHVE_FLINT)
 matrix singntl_HNF(matrix  m, const ring s )
 {
   int r=m->rows();
@@ -1705,7 +1800,7 @@ matrix singntl_HNF(matrix  m, const ring s )
     return NULL;
   }
 
-  matrix res=mp_New(r,r);
+  matrix res=mpNew(r,r);
 
   if (rField_is_Q(s))
   {
@@ -1798,7 +1893,7 @@ matrix singntl_LLL(matrix  m, const ring s )
 {
   int r=m->rows();
   int c=m->cols();
-  matrix res=mp_New(r,c);
+  matrix res=mpNew(r,c);
   if (rField_is_Q(s))
   {
     CFMatrix M(r,c);
@@ -1849,7 +1944,33 @@ intvec* singntl_LLL(intvec*  m)
   delete MM;
   return mm;
 }
+#else
+matrix singntl_HNF(matrix  m, const ring s )
+{
+  WerrorS("NTL/FLINT missing");
+  return NULL;
+}
 
+intvec* singntl_HNF(intvec*  m)
+{
+  WerrorS("NTL/FLINT missing");
+  return NULL;
+}
+
+matrix singntl_LLL(matrix  m, const ring s )
+{
+  WerrorS("NTL/FLINT missing");
+  return NULL;
+}
+
+intvec* singntl_LLL(intvec*  m)
+{
+  WerrorS("NTL/FLINT missing");
+  return NULL;
+}
+#endif
+
+#if defined(HAVE_NTL) || defined(HAVE_FLINT)
 ideal singclap_absFactorize ( poly f, ideal & mipos, intvec ** exps, int & numFactors, const ring r)
 {
   p_Test(f, r);
@@ -1927,35 +2048,16 @@ ideal singclap_absFactorize ( poly f, ideal & mipos, intvec ** exps, int & numFa
 }
 
 #else
-matrix singntl_HNF(matrix  m, const ring s )
-{
-  WerrorS("NTL missing");
-  return NULL;
-}
-
-intvec* singntl_HNF(intvec*  m)
-{
-  WerrorS("NTL missing");
-  return NULL;
-}
-
-matrix singntl_LLL(matrix  m, const ring s )
-{
-  WerrorS("NTL missing");
-  return NULL;
-}
-
-intvec* singntl_LLL(intvec*  m)
-{
-  WerrorS("NTL missing");
-  return NULL;
-}
-
 ideal singclap_absFactorize ( poly f, ideal & mipos, intvec ** exps, int & numFactors, const ring r)
 {
-  WerrorS("NTL missing");
+  WerrorS("NTL/FLINT missing: absFactorize");
   return NULL;
 }
 
 #endif /* HAVE_NTL */
 
+int * Zp_roots(poly p, const ring r)
+{
+  CanonicalForm pp=convSingPFactoryP(p,r);
+  return Zp_roots(pp);
+}
