@@ -9,7 +9,7 @@
   *           IMPORTANT ASSUMPTIONS:
   *           1.) So far we assume that cf->extRing is a valid polynomial
   *               ring in exactly one variable, i.e., K[a], where K is allowed
-  *               to be any field (representable in SINGULAR and which may
+  0*               to be any field (representable in SINGULAR and which may
   *               itself be some extension field, thus allowing for extension
   *               towers).
   *           2.) Moreover, this implementation assumes that
@@ -30,8 +30,6 @@
 
 #include "misc/auxiliary.h"
 
-#include "omalloc/omalloc.h"
-
 #include "reporter/reporter.h"
 
 #include "coeffs/coeffs.h"
@@ -298,7 +296,6 @@ BOOLEAN naEqual(number a, number b, const coeffs cf)
 number naCopy(number a, const coeffs cf)
 {
   naTest(a);
-  if (a == NULL) return NULL;
   if (((poly)a)==naMinpoly) return a;
   return (number)p_Copy((poly)a, naRing);
 }
@@ -344,6 +341,12 @@ number naInit(long i, const coeffs cf)
   else        return (number)p_ISet(i, naRing);
 }
 
+static number naInitMPZ(mpz_t m, const coeffs r)
+{
+  number n=n_InitMPZ(m,r->extRing->cf);
+  return (number)p_NSet(n,r->extRing);
+}
+
 long naInt(number &a, const coeffs cf)
 {
   naTest(a);
@@ -812,7 +815,7 @@ number naGcd(number a, number b, const coeffs cf)
 #if 0
   naTest(a); naTest(b);
   const ring R = naRing;
-  return (number) singclap_gcd(p_Copy((poly)a, R), p_Copy((poly)b, R), R);
+  return (number) singclap_gcd_r((poly)a, (poly)b, R);
 #endif
 //  return (number)p_Gcd((poly)a, (poly)b, naRing);
 }
@@ -1021,8 +1024,6 @@ nMapFunc naSetMap(const coeffs src, const coeffs dst)
   /* dst is expected to be an algebraic field extension */
   assume(getCoeffType(dst) == n_algExt);
 
-  if( src == dst ) return ndCopyMap;
-
   int h = 0; /* the height of the extension tower given by dst */
   coeffs bDst = nCoeff_bottom(dst, h); /* the bottom field in the tower dst */
   coeffs bSrc = nCoeff_bottom(src, h); /* the bottom field in the tower src */
@@ -1103,7 +1104,7 @@ int naIsParam(number m, const coeffs cf)
 }
 
 
-void naClearContent(ICoeffsEnumerator& numberCollectionEnumerator, number& c, const coeffs cf)
+static void naClearContent(ICoeffsEnumerator& numberCollectionEnumerator, number& c, const coeffs cf)
 {
   assume(cf != NULL);
   assume(getCoeffType(cf) == n_algExt);
@@ -1171,7 +1172,11 @@ void naClearContent(ICoeffsEnumerator& numberCollectionEnumerator, number& c, co
 
 //    p_InpGcd(cand, (poly)n, R);
 
-    cand = singclap_gcd(cand, p_Copy((poly)n, R), R);
+    { // R->cf is QQ
+      poly tmp=gcd_over_Q(cand,(poly)n,R);
+      p_Delete(&cand,R);
+      cand=tmp;
+    }
 
 //    cand1 = p_Gcd(cand,(poly)n, R); p_Delete(&cand, R); cand = cand1;
 
@@ -1322,31 +1327,9 @@ void naClearDenominators(ICoeffsEnumerator& numberCollectionEnumerator, number&
 
 void naKillChar(coeffs cf)
 {
-   if ((--cf->extRing->ref) == 0)
-     rDelete(cf->extRing);
-}
-
-char* naCoeffString(const coeffs r) // currently also for tranext.
-{
-  const char* const* p=n_ParameterNames(r);
-  int l=0;
-  int i;
-  for(i=0; i<n_NumberOfParameters(r);i++)
-  {
-    l+=(strlen(p[i])+1);
-  }
-  char *s=(char *)omAlloc(l+10+1);
-  s[0]='\0';
-  snprintf(s,10+1,"%d",r->ch); /* Fp(a) or Q(a) */
-  char tt[2];
-  tt[0]=',';
-  tt[1]='\0';
-  for(i=0; i<n_NumberOfParameters(r);i++)
-  {
-    strcat(s,tt);
-    strcat(s,p[i]);
-  }
-  return s;
+  rDecRefCnt(cf->extRing);
+  if(cf->extRing->ref<0)
+    rDelete(cf->extRing);
 }
 
 char* naCoeffName(const coeffs r) // currently also for tranext.
@@ -1358,7 +1341,7 @@ char* naCoeffName(const coeffs r) // currently also for tranext.
   {
     l+=(strlen(p[i])+1);
   }
-  static char s[200];
+  STATIC_VAR char s[200];
   s[0]='\0';
   snprintf(s,10+1,"%d",r->ch); /* Fp(a) or Q(a) */
   char tt[2];
@@ -1409,7 +1392,7 @@ BOOLEAN naInitChar(coeffs cf, void * infoStruct)
   assume( cf != NULL );
   assume(getCoeffType(cf) == n_algExt);                     // coeff type;
 
-  e->r->ref ++; // increase the ref.counter for the ground poly. ring!
+  rIncRefCnt(e->r); // increase the ref.counter for the ground poly. ring!
   const ring R = e->r; // no copy!
   cf->extRing  = R;
 
@@ -1425,7 +1408,6 @@ BOOLEAN naInitChar(coeffs cf, void * infoStruct)
   p_Test((poly)naMinpoly, naRing);
   #endif
 
-  cf->cfCoeffString  = naCoeffString;
   cf->cfCoeffName    = naCoeffName;
 
   cf->cfGreaterZero  = naGreaterZero;
@@ -1435,6 +1417,7 @@ BOOLEAN naInitChar(coeffs cf, void * infoStruct)
   cf->cfIsOne        = naIsOne;
   cf->cfIsMOne       = naIsMOne;
   cf->cfInit         = naInit;
+  cf->cfInitMPZ      = naInitMPZ;
   cf->cfFarey        = naFarey;
   cf->cfChineseRemainder= naChineseRemainder;
   cf->cfInt          = naInt;
@@ -1525,7 +1508,7 @@ BOOLEAN  n2pDBTest(number a, const char *f, const int l, const coeffs r);
 #define n2pCoeffs cf->extRing->cf
 
 #ifdef LDEBUG
-BOOLEAN n2pDBTest(number a, const char *f, const int l, const coeffs cf)
+BOOLEAN n2pDBTest(number a, const char */*f*/, const int /*l*/, const coeffs cf)
 {
   if (a == NULL) return TRUE;
   return p_Test((poly)a, n2pRing);
@@ -1592,34 +1575,6 @@ static BOOLEAN n2pCoeffIsEqual(const coeffs cf, n_coeffType n, void * param)
   return FALSE;
 }
 
-char* n2pCoeffString(const coeffs cf)
-{
-  const char* const* p=n_ParameterNames(cf);
-  int l=0;
-  int i;
-  for(i=0; i<rVar(n2pRing);i++)
-  {
-    l+=(strlen(p[i])+1);
-  }
-  char *cf_s=nCoeffString(n2pRing->cf);
-  char *s=(char *)omAlloc(l+5+strlen(cf_s));
-  s[0]='\0';
-  snprintf(s,strlen(cf_s)+2,"%s",cf_s);
-  omFree(cf_s);
-  char tt[2];
-  tt[0]='[';
-  tt[1]='\0';
-  strcat(s,tt);
-  tt[0]=',';
-  for(i=0; i<rVar(n2pRing);i++)
-  {
-    strcat(s,p[i]);
-    if (i+1!=rVar(n2pRing)) strcat(s,tt);
-    else { tt[0]=']'; strcat(s,tt); }
-  }
-  return s;
-}
-
 char* n2pCoeffName(const coeffs cf)
 {
   const char* const* p=n_ParameterNames(cf);
@@ -1629,11 +1584,10 @@ char* n2pCoeffName(const coeffs cf)
   {
     l+=(strlen(p[i])+1);
   }
-  char *cf_s=nCoeffString(n2pRing->cf);
-  static char s[200];
+  char *cf_s=nCoeffName(n2pRing->cf);
+  STATIC_VAR char s[200];
   s[0]='\0';
   snprintf(s,strlen(cf_s)+2,"%s",cf_s);
-  omFree(cf_s);
   char tt[2];
   tt[0]='[';
   tt[1]='\0';
@@ -1688,7 +1642,7 @@ BOOLEAN n2pInitChar(coeffs cf, void * infoStruct)
 
   assume( cf != NULL );
 
-  e->r->ref ++; // increase the ref.counter for the ground poly. ring!
+  rIncRefCnt(e->r); // increase the ref.counter for the ground poly. ring!
   const ring R = e->r; // no copy!
   cf->extRing  = R;
 
@@ -1698,7 +1652,6 @@ BOOLEAN n2pInitChar(coeffs cf, void * infoStruct)
   cf->is_field=FALSE;
   cf->is_domain=TRUE;
 
-  cf->cfCoeffString  = n2pCoeffString;
   cf->cfCoeffName    = n2pCoeffName;
 
   cf->cfGreaterZero  = naGreaterZero;
@@ -1708,6 +1661,7 @@ BOOLEAN n2pInitChar(coeffs cf, void * infoStruct)
   cf->cfIsOne        = naIsOne;
   cf->cfIsMOne       = naIsMOne;
   cf->cfInit         = naInit;
+  cf->cfInitMPZ      = naInitMPZ;
   cf->cfFarey        = naFarey;
   cf->cfChineseRemainder= naChineseRemainder;
   cf->cfInt          = naInt;
