@@ -2,7 +2,6 @@
 *  Computer Algebra System SINGULAR     *
 ****************************************/
 
-#include "omalloc/omalloc.h"
 #include "misc/auxiliary.h"
 #include "misc/options.h"
 
@@ -12,6 +11,10 @@
 #include "polys/monomials/ring.h"
 #include "polys/kbuckets.h"
 
+#ifdef HAVE_SHIFTBBA
+#include "polys/shiftop.h"
+#endif
+
 #ifdef HAVE_COEF_BUCKETS
 #define USE_COEF_BUCKETS
 #endif
@@ -39,9 +42,9 @@
 #else
 #define MULTIPLY_BUCKET(B,I)
 #endif
-static omBin kBucket_bin = omGetSpecBin(sizeof(kBucket));
+STATIC_VAR omBin kBucket_bin = omGetSpecBin(sizeof(kBucket));
 #ifdef USE_COEF_BUCKETS
-static int coef_start=1;
+STATIC_VAR int coef_start=1;
 #endif
 //////////////////////////////////////////////////////////////////////////
 ///
@@ -368,6 +371,7 @@ void kBucketInit(kBucket_pt bucket, poly lm, int length)
 
 int kBucketCanonicalize(kBucket_pt bucket)
 {
+#ifndef HAVE_PSEUDO_BUCKETS
   assume(bucket->buckets_used<=MAX_BUCKET);
   MULTIPLY_BUCKET(bucket,1);
   kbTest(bucket);
@@ -437,6 +441,20 @@ int kBucketCanonicalize(kBucket_pt bucket)
   //if (TEST_OPT_PROT) { Print("C(%d)",pl); }
   kbTest(bucket);
   return i;
+#endif
+}
+
+void kBucketNormalize(kBucket_pt bucket)
+{
+#ifdef HAVE_PSEUDO_BUCKETS
+  p_Normalize(bucket->p,bucket->bucket_ring);
+#else
+  MULTIPLY_BUCKET(bucket,1);
+  for (int i=0; i<=bucket->buckets_used; i++)
+  {
+    p_Normalize(bucket->buckets[i],bucket->bucket_ring);
+  }
+#endif
 }
 
 void kBucketClear(kBucket_pt bucket, poly *p, int *length)
@@ -614,12 +632,18 @@ void kBucket_Mult_n(kBucket_pt bucket, number n)
       }
 #else
       bucket->buckets[i] = __p_Mult_nn(bucket->buckets[i], n, r);
-      if (rField_is_Ring(r) && !(rField_is_Domain(r)))
+#endif
+    }
+  }
+  if (rField_is_Ring(r) && !(rField_is_Domain(r)))
+  {
+    for (i=0; i<= bucket->buckets_used; i++)
+    {
+      if (bucket->buckets[i] != NULL)
       {
         bucket->buckets_length[i] = pLength(bucket->buckets[i]);
         kBucketAdjust(bucket, i);
       }
-#endif
     }
   }
   kbTest(bucket);
@@ -723,7 +747,7 @@ void kBucket_Minus_m_Mult_p(kBucket_pt bucket, poly m, poly p, int *l,
   ||(rIsPluralRing(r)))
   {
     pSetCoeff0(m, n_InpNeg(pGetCoeff(m),r->cf));
-    p1=pp_Mult_mm(p,m,r);
+    p1=r->p_Procs->pp_mm_Mult(p,m,r);
     pSetCoeff0(m, n_InpNeg(pGetCoeff(m),r->cf));
     l1=pLength(p1);
     i = pLogLength(l1);
@@ -765,7 +789,7 @@ void kBucket_Minus_m_Mult_p(kBucket_pt bucket, poly m, poly p, int *l,
       }
       else
       {
-        p1 = r->p_Procs->pp_Mult_mm(p1, m, r);
+        p1 = r->p_Procs->pp_mm_Mult(p1, m, r);
       }
       pSetCoeff0(m, n_InpNeg(pGetCoeff(m),r->cf));
     }
@@ -1125,6 +1149,17 @@ number kBucketPolyRed(kBucket_pt bucket,
   l1--;
 
   assume((unsigned)l1==pLength(a1));
+
+#ifdef HAVE_SHIFTBBA
+  poly lmRight;
+  poly lm_org;
+  if (r->isLPring)
+  {
+    int firstBlock = p_mFirstVblock(p1, r);
+    lm_org=lm;
+    k_SplitFrame(lm, lmRight, si_max(firstBlock, 1), r);
+  }
+#endif
 #if 0
   BOOLEAN backuped=FALSE;
   number coef;
@@ -1145,7 +1180,20 @@ number kBucketPolyRed(kBucket_pt bucket,
   }
 #endif
 
-  kBucket_Minus_m_Mult_p(bucket, lm, a1, &l1, spNoether);
+#ifdef HAVE_SHIFTBBA
+  if (r->isLPring)
+  {
+    poly tmp=r->p_Procs->pp_Mult_mm(a1, lmRight, r);
+    kBucket_Minus_m_Mult_p(bucket, lm,tmp, &l1, spNoether);
+    p_Delete(&tmp,r);
+    p_LmDelete(&lmRight,r);
+    p_LmDelete(lm_org,r);
+  }
+  else
+#endif
+  {
+    kBucket_Minus_m_Mult_p(bucket, lm, a1, &l1, spNoether);
+  }
 
 #if 0
   if (backuped)
@@ -1396,8 +1444,8 @@ int ksCheckCoeff(number *a, number *b, const coeffs r)
   }
   else
   {
-    an = n_ExactDiv(an, cn, r); n_Normalize(an,r);
-    bn = n_ExactDiv(bn, cn, r); n_Normalize(bn,r);
+    an = n_ExactDiv(an, cn, r);
+    bn = n_ExactDiv(bn, cn, r);
   }
   n_Delete(&cn, r);
   if (n_IsOne(an, r))
