@@ -8,7 +8,6 @@
 
 #include "misc/auxiliary.h"
 
-#include "omalloc/omalloc.h"
 #include "misc/mylimits.h"
 
 #include "misc/intvec.h"
@@ -23,6 +22,7 @@
 #include "simpleideals.h"
 #include "matpol.h"
 #include "prCopy.h"
+#include "clapsing.h"
 
 #include "sparsmat.h"
 
@@ -31,6 +31,7 @@
 
 static poly mp_Exdiv ( poly m, poly d, poly vars, const ring);
 static poly mp_Select (poly fro, poly what, const ring);
+static poly mp_SelectId (ideal I, poly what, const ring R);
 
 /// create a r x c zero-matrix
 matrix mpNew(int r, int c)
@@ -64,10 +65,10 @@ matrix mp_Copy (matrix a, const ring r)
 {
   id_Test((ideal)a, r);
   poly t;
-  int i, m=MATROWS(a), n=MATCOLS(a);
+  int m=MATROWS(a), n=MATCOLS(a);
   matrix b = mpNew(m, n);
 
-  for (i=m*n-1; i>=0; i--)
+  for (long i=(long)m*(long)n-1; i>=0; i--)
   {
     t = a->m[i];
     if (t!=NULL)
@@ -226,27 +227,24 @@ matrix mp_Mult(matrix a, matrix b, const ring R)
   }
   matrix c = mpNew(m,q);
 
-  for (i=1; i<=m; i++)
+  for (i=0; i<m; i++)
   {
-    for (k=1; k<=p; k++)
+    for (k=0; k<p; k++)
     {
       poly aik;
-      if ((aik=MATELEM(a,i,k))!=NULL)
+      if ((aik=MATELEM0(a,i,k))!=NULL)
       {
-        for (j=1; j<=q; j++)
+        for (j=0; j<q; j++)
         {
           poly bkj;
-          if ((bkj=MATELEM(b,k,j))!=NULL)
+          if ((bkj=MATELEM0(b,k,j))!=NULL)
           {
-            poly *cij=&(MATELEM(c,i,j));
-            poly s = pp_Mult_qq(aik /*MATELEM(a,i,k)*/, bkj/*MATELEM(b,k,j)*/, R);
-            if (/*MATELEM(c,i,j)*/ (*cij)==NULL) (*cij)=s;
-            else (*cij) = p_Add_q((*cij) /*MATELEM(c,i,j)*/ ,s, R);
+            poly *cij=&(MATELEM0(c,i,j));
+            poly s = pp_Mult_qq(aik /*MATELEM0(a,i,k)*/, bkj/*MATELEM0(b,k,j)*/, R);
+            (*cij)/*MATELEM0(c,i,j)*/ = p_Add_q((*cij) /*MATELEM0(c,i,j)*/ ,s, R);
           }
         }
       }
-    //  pNormalize(t);
-    //  MATELEM(c,i,j) = t;
     }
   }
   for(i=m*q-1;i>=0;i--) p_Normalize(c->m[i], R);
@@ -410,7 +408,7 @@ matrix mp_CoeffProc (poly f, poly vars, const ring R)
   {
     co = mpNew(2, 1);
     MATELEM(co,1,1) = p_One(R);
-    MATELEM(co,2,1) = NULL;
+    //MATELEM(co,2,1) = NULL;
     return co;
   }
   sel = mp_Select(f, vars, R);
@@ -425,7 +423,7 @@ matrix mp_CoeffProc (poly f, poly vars, const ring R)
       pIter(sel);
       pNext(h)=NULL;
       MATELEM(co,1,i) = h;
-      MATELEM(co,2,i) = NULL;
+      //MATELEM(co,2,i) = NULL;
       if (p_IsConstant(h, R)) pos_of_1 = i;
     }
   }
@@ -437,7 +435,7 @@ matrix mp_CoeffProc (poly f, poly vars, const ring R)
       pIter(sel);
       pNext(h)=NULL;
       MATELEM(co,1,i) = h;
-      MATELEM(co,2,i) = NULL;
+      //MATELEM(co,2,i) = NULL;
       if (p_IsConstant(h, R)) pos_of_1 = i;
     }
   }
@@ -475,6 +473,86 @@ matrix mp_CoeffProc (poly f, poly vars, const ring R)
   return co;
 }
 
+matrix mp_CoeffProcId (ideal I, poly vars, const ring R)
+{
+  assume(vars!=NULL);
+  poly sel, h;
+  int l, i;
+  int pos_of_1 = -1;
+  matrix co;
+
+  if (idIs0(I))
+  {
+    co = mpNew(IDELEMS(I)+1,1);
+    MATELEM(co,1,1) = p_One(R);
+    return co;
+  }
+  sel = mp_SelectId(I, vars, R);
+  l = pLength(sel);
+  co = mpNew(IDELEMS(I)+1, l);
+
+  if (rHasLocalOrMixedOrdering(R))
+  {
+    for (i=l; i>=1; i--)
+    {
+      h = sel;
+      pIter(sel);
+      pNext(h)=NULL;
+      MATELEM(co,1,i) = h;
+      //MATELEM(co,2,i) = NULL;
+      if (p_IsConstant(h, R)) pos_of_1 = i;
+    }
+  }
+  else
+  {
+    for (i=1; i<=l; i++)
+    {
+      h = sel;
+      pIter(sel);
+      pNext(h)=NULL;
+      MATELEM(co,1,i) = h;
+      //MATELEM(co,2,i) = NULL;
+      if (p_IsConstant(h, R)) pos_of_1 = i;
+    }
+  }
+  for(int j=0;j<IDELEMS(I);j++)
+  {
+    poly f=I->m[j];
+    while (f!=NULL)
+    {
+      i = 1;
+      loop
+      {
+        if (i!=pos_of_1)
+        {
+          h = mp_Exdiv(f, MATELEM(co,1,i),vars, R);
+          if (h!=NULL)
+          {
+            MATELEM(co,j+2,i) = p_Add_q(MATELEM(co,j+2,i), h, R);
+            break;
+          }
+        }
+        if (i == l)
+        {
+          // check monom 1 last:
+          if (pos_of_1 != -1)
+          {
+            h = mp_Exdiv(f, MATELEM(co,1,pos_of_1),vars, R);
+            if (h!=NULL)
+            {
+              MATELEM(co,j+2,pos_of_1) = p_Add_q(MATELEM(co,j+2,pos_of_1), h, R);
+            }
+          }
+          break;
+        }
+        i ++;
+      }
+      pIter(f);
+    }
+  }
+  return co;
+}
+
 /*2
 *exact divisor: let d  == x^i*y^j, m is thought to have only one term;
 *    return m/d iff d divides m, and no x^k*y^l (k>i or l>j) divides m
@@ -685,6 +763,28 @@ static poly mp_Select (poly fro, poly what, const ring R)
   return res;
 }
 
+static poly mp_SelectId (ideal I, poly what, const ring R)
+{
+  int i;
+  poly h, res;
+  res = NULL;
+  for(int j=0;j<IDELEMS(I);j++)
+  {
+    poly fro=I->m[j];
+    while (fro!=NULL)
+    {
+      h = p_One(R);
+      for (i=1; i<=rVar(R); i++)
+        p_SetExp(h,i, p_GetExp(fro,i, R) * p_GetExp(what, i, R), R);
+      p_SetComp(h, p_GetComp(fro, R), R);
+      p_Setm(h, R);
+      res = p_Insert(h, res, R);
+      fro = fro->next;
+    }
+  }
+  return res;
+}
+
 /*
 *static void ppp(matrix a)
 *{
@@ -1698,26 +1798,9 @@ matrix mp_Wedge(matrix a, int ar, const ring R)
   return (result);
 }
 
-static void p_DecomposeComp(poly p, poly *a, int l, const ring r)
-{
-  poly h=p;
-  while(h!=NULL)
-  {
-    poly hh=pNext(h);
-    pNext(h)=a[__p_GetComp(h,r)-1];
-    a[__p_GetComp(h,r)-1]=h;
-    p_SetComp(h,0,r);
-    p_SetmComp(h,r);
-    h=hh;
-  }
-  for(int i=0;i<l;i++)
-  {
-    if(a[i]!=NULL) a[i]=pReverse(a[i]);
-  }
-}
-// helper for mp_Tensor
+// helper for sm_Tensor
 // destroyes f, keeps B
-static ideal mp_MultAndShift(poly f, ideal B, int s, const ring r)
+static ideal sm_MultAndShift(poly f, ideal B, int s, const ring r)
 {
   assume(f!=NULL);
   ideal res=idInit(IDELEMS(B),B->rank+s);
@@ -1734,9 +1817,9 @@ static ideal mp_MultAndShift(poly f, ideal B, int s, const ring r)
   p_Delete(&f,r);
   return res;
 }
-// helper for mp_Tensor
+// helper for sm_Tensor
 // updates res, destroyes contents of sm
-static void mp_AddSubMat(ideal res, ideal sm, int col, const ring r)
+static void sm_AddSubMat(ideal res, ideal sm, int col, const ring r)
 {
   for(int i=0;i<IDELEMS(sm);i++)
   {
@@ -1745,9 +1828,9 @@ static void mp_AddSubMat(ideal res, ideal sm, int col, const ring r)
   }
 }
 
-ideal mp_Tensor(ideal A, ideal B, const ring r)
+ideal sm_Tensor(ideal A, ideal B, const ring r)
 {
-  // size of the result n*q x m*p
+  // size of the result m*p x n*q
   int n=IDELEMS(A); // m x n
   int m=A->rank;
   int q=IDELEMS(B); // p x q
@@ -1757,16 +1840,16 @@ ideal mp_Tensor(ideal A, ideal B, const ring r)
   for(int i=0; i<n; i++)
   {
     memset(a,0,m*sizeof(poly));
-    p_DecomposeComp(p_Copy(A->m[i],r),a,m,r);
+    p_Vec2Array(A->m[i],a,m,r);
     for(int j=0;j<m;j++)
     {
       if (a[j]!=NULL)
       {
-        ideal sm=mp_MultAndShift(a[j], // A_i_j
+        ideal sm=sm_MultAndShift(a[j], // A_i_j
                                  B,
                                  j*p, // shift j*p down
                                  r);
-        mp_AddSubMat(res,sm,i*q,r); // add this columns to col i*q ff
+        sm_AddSubMat(res,sm,i*q,r); // add this columns to col i*q ff
         id_Delete(&sm,r); // delete the now empty ideal
       }
     }
@@ -1774,4 +1857,322 @@ ideal mp_Tensor(ideal A, ideal B, const ring r)
   omFreeSize(a,m*sizeof(poly));
   return res;
 }
+// --------------------------------------------------------------------------
+/****************************************
+*  Computer Algebra System SINGULAR     *
+****************************************/
 
+/*
+* ABSTRACT: basic operation for sparse matrices:
+* type: ideal (of column vectors)
+* nrows: I->rank, ncols: IDELEMS(I)
+*/
+
+ideal sm_Add(ideal a, ideal b, const ring R)
+{
+  assume(IDELEMS(a)==IDELEMS(b));
+  assume(a->rank==b->rank);
+  ideal c=idInit(IDELEMS(a),a->rank);
+  for (int k=IDELEMS(a)-1; k>=0; k--)
+    c->m[k] = p_Add_q(p_Copy(a->m[k], R), p_Copy(b->m[k], R), R);
+  return c;
+}
+
+ideal sm_Sub(ideal a, ideal b, const ring R)
+{
+  assume(IDELEMS(a)==IDELEMS(b));
+  assume(a->rank==b->rank);
+  ideal c=idInit(IDELEMS(a),a->rank);
+  for (int k=IDELEMS(a)-1; k>=0; k--)
+    c->m[k] = p_Sub(p_Copy(a->m[k], R), p_Copy(b->m[k], R), R);
+  return c;
+}
+
+ideal sm_Mult(ideal a, ideal b, const ring R)
+{
+  int i, j, k;
+  int m = a->rank;
+  int p = IDELEMS(a);
+  int q = IDELEMS(b);
+
+  assume (IDELEMS(a)==b->rank);
+  ideal c = idInit(q,m);
+
+  for (i=0; i<m; i++)
+  {
+    for (k=0; k<p; k++)
+    {
+      poly aik;
+      if ((aik=SMATELEM(a,i,k,R))!=NULL)
+      {
+        for (j=0; j<q; j++)
+        {
+          poly bkj=SMATELEM(b,k,j,R);
+          if (bkj!=NULL)
+          {
+            poly s = p_Mult_q(p_Copy(aik,R) /*SMATELEM(a,i,k)*/, bkj/*SMATELEM(b,k,j)*/, R);
+            if (s!=NULL) p_SetComp(s,i+1,R);
+            c->m[j]=p_Add_q(c->m[j],s, R);
+          }
+        }
+        p_Delete(&aik,R);
+      }
+    }
+  }
+  for(i=q-1;i>=0;i--) p_Normalize(c->m[i], R);
+  return c;
+}
+
+ideal sm_Flatten(ideal a, const ring R)
+{
+  if (IDELEMS(a)==0) return id_Copy(a,R);
+  ideal res=idInit(1,IDELEMS(a)*a->rank);
+  for(int i=0;i<IDELEMS(a);i++)
+  {
+    if(a->m[i]!=NULL)
+    {
+      poly p=p_Copy(a->m[i],R);
+      if (i==0) res->m[0]=p;
+      else
+      {
+        p_Shift(&p,i*a->rank,R);
+        res->m[0]=p_Add_q(res->m[0],p,R);
+      }
+    }
+  }
+  return res;
+}
+
+ideal sm_UnFlatten(ideal a, int col, const ring R)
+{
+  if ((IDELEMS(a)!=1)
+  ||((a->rank % col)!=0))
+  {
+    Werror("wrong format: %d x %d for unflatten",(int)a->rank,IDELEMS(a));
+    return NULL;
+  }
+  int row=a->rank/col;
+  ideal res=idInit(col,row);
+  poly p=a->m[0];
+  while(p!=NULL)
+  {
+    poly h=p_Head(p,R);
+    int comp=p_GetComp(h,R);
+    int c=(comp-1)/row;
+    int r=comp%row; if (r==0) r=row;
+    p_SetComp(h,r,R); p_SetmComp(h,R);
+    res->m[c]=p_Add_q(res->m[c],h,R);
+    pIter(p);
+  }
+  return res;
+}
+
+/*2
+*returns the trace of matrix a
+*/
+poly sm_Trace ( ideal a, const ring R)
+{
+  int i;
+  int n = (IDELEMS(a)<a->rank) ? IDELEMS(a) : a->rank;
+  poly  t = NULL;
+
+  for (i=0; i<=n; i++)
+    t = p_Add_q(t, p_Copy(SMATELEM(a,i,i,R), R), R);
+  return t;
+}
+
+int sm_Compare(ideal a, ideal b, const ring R)
+{
+  if (IDELEMS(a)<IDELEMS(b)) return -1;
+  else if (IDELEMS(a)>IDELEMS(b)) return 1;
+  if ((a->rank)<(b->rank)) return -1;
+  else if ((a->rank)<(b->rank)) return 1;
+
+  unsigned ii=IDELEMS(a)-1;
+  unsigned j=0;
+  int r=0;
+  while (j<=ii)
+  {
+    r=p_Compare(a->m[j],b->m[j],R);
+    if (r!=0) return r;
+    j++;
+  }
+  return r;
+}
+
+BOOLEAN sm_Equal(ideal a, ideal b, const ring R)
+{
+  if ((a->rank!=b->rank) || (IDELEMS(a)!=IDELEMS(b)))
+    return FALSE;
+  int i=IDELEMS(a)-1;
+  while (i>=0)
+  {
+    if (a->m[i]==NULL)
+    {
+      if (b->m[i]!=NULL) return FALSE;
+    }
+    else if (b->m[i]==NULL) return FALSE;
+    else if (p_Cmp(a->m[i],b->m[i], R)!=0) return FALSE;
+    i--;
+  }
+  i=IDELEMS(a)-1;
+  while (i>=0)
+  {
+    if(!p_EqualPolys(a->m[i],b->m[i], R)) return FALSE;
+    i--;
+  }
+  return TRUE;
+}
+
+/*
+*   mu-Algorithmus:
+*/
+
+//  mu-Matrix
+static matrix mu(matrix A, const ring R)
+{
+  int n=MATROWS(A);
+  assume(MATCOLS(A)==n);
+    /*  Die Funktion erstellt die Matrix mu
+    *
+    *   Input:
+    *   int n: Dimension der Matrix
+    *   int A: Matrix der Groesse n*n
+    *   int X: Speicherplatz fuer Output
+    *
+    *   In der Matrix X speichert man die Matrix mu
+    */
+
+    // X als n*n Null-Matrix initalisieren
+    matrix X=mpNew(n,n);
+
+    //  Diagonaleintraege von X berrechnen
+    poly sum = NULL;
+    for (int i = n-1; i >= 0; i--)
+    {
+        MATELEM0(X,i,i) = p_Copy(sum,R);
+        sum=p_Sub(sum,p_Copy(MATELEM0(A,i,i),R),R);
+    }
+    p_Delete(&sum,R);
+
+    //  Eintraege aus dem oberen Dreieck von A nach X uebertragen
+    for (int i = n-1; i >=0; i--)
+    {
+        for (int j = i+1; j < n; j++)
+        {
+            MATELEM0(X,i,j)=p_Copy(MATELEM0(A,i,j),R);
+        }
+    }
+    return X;
+}
+
+// Funktion muDet
+poly mp_DetMu(matrix A, const ring R)
+{
+  int n=MATROWS(A);
+  assume(MATCOLS(A)==n);
+    /*
+    *   Intput:
+    *   int n: Dimension der Matrix
+    *   int A: n*n Matrix
+    *
+    *   Berechnet n-1 mal: X = mu(X)*A
+    *
+    *   Output: det(A)
+    */
+
+    //speichere A ab:
+    matrix workA=mp_Copy(A,R);
+
+    // berechen X = mu(X)*A
+    matrix X;
+    for (int i = n-1; i >0; i--)
+    {
+        X=mu(workA,R);
+        id_Delete((ideal*)&workA,R);
+        workA=mp_Mult(X,A,R);
+        id_Delete((ideal*)&X,R);
+    }
+
+    // berrechne det(A)
+    poly res;
+    if (n%2 == 0)
+    {
+        res=p_Neg(MATELEM0(workA,0,0),R);
+    }
+    else
+    {
+        res=MATELEM0(workA,0,0);
+    }
+    MATELEM0(workA,0,0)=NULL;
+    id_Delete((ideal*)&workA,R);
+    return res;
+}
+
+DetVariant mp_GetAlgorithmDet(matrix m, const ring r)
+{
+  if (MATROWS(m)+2*r->N>20+5*rField_is_Zp(r)) return DetMu;
+  if (MATROWS(m)<10+5*rField_is_Zp(r)) return DetSBareiss;
+  BOOLEAN isConst=TRUE;
+  int s=0;
+  for(int i=MATCOLS(m)*MATROWS(m)-1;i>=0;i--)
+  {
+    poly p=m->m[i];
+    if (p!=NULL)
+    {
+      if(!p_IsConstant(p,r)) isConst=FALSE;
+      s++;
+    }
+  }
+  if (isConst && rField_is_Q(r)) return DetFactory;
+  if (s*2<MATCOLS(m)*MATROWS(m)) // few entries
+    return DetSBareiss;
+  return DetMu;
+}
+DetVariant mp_GetAlgorithmDet(const char *s)
+{
+  if (strcmp(s,"Bareiss")==0) return DetBareiss;
+  if (strcmp(s,"SBareiss")==0) return DetSBareiss;
+  if (strcmp(s,"Mu")==0) return DetMu;
+  if (strcmp(s,"Factory")==0) return DetFactory;
+  WarnS("unknown method for det");
+  return DetDefault;
+}
+
+
+poly mp_Det(matrix a, const ring r, DetVariant d/*=DetDefault*/)
+{
+  if ((MATCOLS(a)==0)
+  && (MATROWS(a)==0))
+    return p_One(r);
+  if (d==DetDefault) d=mp_GetAlgorithmDet(a,r);
+  switch (d)
+  {
+    case DetBareiss: return mp_DetBareiss(a,r);
+    case DetMu: return mp_DetMu(a,r);
+    case DetFactory: return singclap_det(a,r);
+    case DetSBareiss:
+    {
+      ideal I=id_Matrix2Module(mp_Copy(a, r),r);
+      poly p=sm_CallDet(I, r);
+      id_Delete(&I, r);
+      return p;
+    }
+    default:
+      WerrorS("unknown algorithm for det");
+      return NULL;
+  }
+}
+
+poly sm_Det(ideal a, const ring r, DetVariant d/*=DetDefault*/)
+{
+  if ((MATCOLS(a)==0)
+  && (MATROWS(a)==0))
+    return p_One(r);
+  if (d==DetDefault) d=mp_GetAlgorithmDet((matrix)a,r);
+  if (d==DetSBareiss) return sm_CallDet(a,r);
+  matrix m=id_Module2Matrix(id_Copy(a,r),r);
+  poly p=mp_Det(m,r,d);
+  id_Delete((ideal *)&m,r);
+  return p;
+}
