@@ -16,8 +16,6 @@
 #ifndef P_POLYS_H
 #define P_POLYS_H
 
-#include "omalloc/omalloc.h"
-
 #include "misc/mylimits.h"
 #include "misc/intvec.h"
 #include "coeffs/coeffs.h"
@@ -74,7 +72,7 @@ poly p_One(const ring r);
 
 int p_MinDeg(poly p,intvec *w, const ring R);
 
-long p_DegW(poly p, const short *w, const ring R);
+long p_DegW(poly p, const int *w, const ring R);
 
 /// return TRUE if all monoms have the same component
 BOOLEAN   p_OneComp(poly p, const ring r);
@@ -96,6 +94,7 @@ poly      p_ISet(long i, const ring r);
 poly      p_NSet(number n, const ring r);
 
 void  p_Vec2Polys(poly v, poly**p, int *len, const ring r);
+poly  p_Vec2Poly(poly v, int k, const ring r);
 
 /// julia: vector to already allocated array (len=p_MaxComp(v,r))
 void  p_Vec2Array(poly v, poly *p, int len, const ring r);
@@ -212,6 +211,7 @@ void      p_ProjectiveUnique(poly p,const ring r);
 
 void      p_ContentForGB(poly p, const ring r);
 void      p_Content(poly p, const ring r);
+void      p_Content_n(poly p, number &c,const ring r);
 #if 1
 // currently only used by Singular/janet
 void      p_SimpleContent(poly p, int s, const ring r);
@@ -714,6 +714,12 @@ static inline void p_LmDelete(poly p, const ring r)
   n_Delete(&pGetCoeff(p), r->cf);
   omFreeBinAddr(p);
 }
+static inline void p_LmDelete0(poly p, const ring r)
+{
+  p_LmCheckPolyRing2(p, r);
+  if (pGetCoeff(p)!=NULL) n_Delete(&pGetCoeff(p), r->cf);
+  omFreeBinAddr(p);
+}
 static inline void p_LmDelete(poly *p, const ring r)
 {
   p_LmCheckPolyRing2(*p, r);
@@ -822,7 +828,8 @@ static inline poly p_Copy(poly p, const ring r)
     return NULL;
 }
 
-static inline poly p_Head(poly p, const ring r)
+/// copy the (leading) term of p
+static inline poly p_Head(const poly p, const ring r)
 {
   if (p == NULL) return NULL;
   p_LmCheckPolyRing1(p, r);
@@ -835,7 +842,16 @@ static inline poly p_Head(poly p, const ring r)
   return np;
 }
 
-// returns a copy of p with Lm(p) from lmRing and Tail(p) from tailRing
+/// like p_Head, but allow NULL coeff
+poly p_Head0(const poly p, const ring r);
+
+/// like p_Head, but with coefficient 1
+poly p_CopyPowerProduct(const poly p, const ring r);
+
+/// like p_Head, but with coefficient n
+poly p_CopyPowerProduct0(const poly p, const number n, const ring r);
+
+/// returns a copy of p with Lm(p) from lmRing and Tail(p) from tailRing
 static inline poly p_Copy(poly p, const ring lmRing, const ring tailRing)
 {
   if (p != NULL)
@@ -880,7 +896,7 @@ static inline void p_Delete(poly *p,  const ring lmRing, const ring tailRing)
 }
 
 // copys monomials of p, allocates new monomials from bin,
-// deletes monomoals of p
+// deletes monomials of p
 static inline poly p_ShallowCopyDelete(poly p, const ring r, omBin bin)
 {
   p_LmCheckPolyRing2(p, r);
@@ -993,6 +1009,16 @@ static inline poly pp_Mult_mm(poly p, poly m, const ring r)
     return r->p_Procs->pp_Mult_mm(p, m, r);
 }
 
+// returns m*Copy(p), does neither destroy p nor m
+static inline poly pp_mm_Mult(poly p, poly m, const ring r)
+{
+  if (p==NULL) return NULL;
+  if (p_LmIsConstant(m, r))
+    return __pp_Mult_nn(p, pGetCoeff(m), r);
+  else
+    return r->p_Procs->pp_mm_Mult(p, m, r);
+}
+
 // returns p*m, destroys p, const: m
 static inline poly p_Mult_mm(poly p, poly m, const ring r)
 {
@@ -1003,6 +1029,16 @@ static inline poly p_Mult_mm(poly p, poly m, const ring r)
     return r->p_Procs->p_Mult_mm(p, m, r);
 }
 
+// returns m*p, destroys p, const: m
+static inline poly p_mm_Mult(poly p, poly m, const ring r)
+{
+  if (p==NULL) return NULL;
+  if (p_LmIsConstant(m, r))
+    return __p_Mult_nn(p, pGetCoeff(m), r);
+  else
+    return r->p_Procs->p_mm_Mult(p, m, r);
+}
+
 static inline poly p_Minus_mm_Mult_qq(poly p, const poly m, const poly q, int &lp, int lq,
                                       const poly spNoether, const ring r)
 {
@@ -1064,13 +1100,7 @@ static inline poly p_Mult_q(poly p, poly q, const ring r)
 
   if (pNext(p) == NULL)
   {
-#ifdef HAVE_PLURAL
-    if (rIsPluralRing(r))
-      q = nc_mm_Mult_p(p, q, r);
-    else
-#endif /* HAVE_PLURAL */
-      q = r->p_Procs->p_Mult_mm(q, p, r);
-
+    q = r->p_Procs->p_mm_Mult(q, p, r);
     p_LmDelete(&p, r);
     return q;
   }
@@ -1081,8 +1111,8 @@ static inline poly p_Mult_q(poly p, poly q, const ring r)
     p_LmDelete(&q, r);
     return p;
   }
-#ifdef HAVE_PLURAL
-  if (rIsPluralRing(r))
+#if defined(HAVE_PLURAL) || defined(HAVE_SHIFTBBA)
+  if (rIsNCRing(r))
     return _nc_p_Mult_q(p, q, r);
   else
 #endif
@@ -1096,11 +1126,7 @@ static inline poly pp_Mult_qq(poly p, poly q, const ring r)
 
   if (pNext(p) == NULL)
   {
-#ifdef HAVE_PLURAL
-    if (rIsPluralRing(r))
-      return nc_mm_Mult_pp(p, q, r);
-#endif
-    return r->p_Procs->pp_Mult_mm(q, p, r);
+    return r->p_Procs->pp_mm_Mult(q, p, r);
   }
 
   if (pNext(q) == NULL)
@@ -1113,8 +1139,8 @@ static inline poly pp_Mult_qq(poly p, poly q, const ring r)
     qq = p_Copy(q, r);
 
   poly res;
-#ifdef HAVE_PLURAL
-  if (rIsPluralRing(r))
+#if defined(HAVE_PLURAL) || defined(HAVE_SHIFTBBA)
+  if (rIsNCRing(r))
     res = _nc_pp_Mult_qq(p, qq, r);
   else
 #endif
@@ -1478,6 +1504,15 @@ static inline void p_GetExpVL(poly p, int64 *ev, const ring r)
   for (unsigned j = r->N; j!=0; j--)
       ev[j-1] = p_GetExp(p, j, r);
 }
+// p_GetExpVLV is used in Singular,jl
+static inline int64 p_GetExpVLV(poly p, int64 *ev, const ring r)
+{
+  p_LmCheckPolyRing1(p, r);
+  for (unsigned j = r->N; j!=0; j--)
+      ev[j-1] = p_GetExp(p, j, r);
+  return (int64)p_GetComp(p,r);
+}
+// p_GetExpVL is used in Singular,jl
 static inline void p_SetExpV(poly p, int *ev, const ring r)
 {
   p_LmCheckPolyRing1(p, r);
@@ -1487,14 +1522,24 @@ static inline void p_SetExpV(poly p, int *ev, const ring r)
   if(ev[0]!=0) p_SetComp(p, ev[0],r);
   p_Setm(p, r);
 }
-// p_SetExpVL is used in Singular,jl
 static inline void p_SetExpVL(poly p, int64 *ev, const ring r)
 {
   p_LmCheckPolyRing1(p, r);
   for (unsigned j = r->N; j!=0; j--)
-      p_SetExp(p, j, ev[j], r);
+      p_SetExp(p, j, ev[j-1], r);
+  p_SetComp(p, 0,r);
+
+  p_Setm(p, r);
+}
+
+// p_SetExpVLV is used in Singular,jl
+static inline void p_SetExpVLV(poly p, int64 *ev, int64 comp, const ring r)
+{
+  p_LmCheckPolyRing1(p, r);
+  for (unsigned j = r->N; j!=0; j--)
+      p_SetExp(p, j, ev[j-1], r);
+  p_SetComp(p, comp,r);
 
-  if(ev[0]!=0) p_SetComp(p, ev[0],r);
   p_Setm(p, r);
 }
 
@@ -1527,7 +1572,7 @@ static inline int p_LmCmp(poly p, poly q, const ring r)
     goto LengthGeneral_OrdGeneral_LoopTop;
   }
   const long* _ordsgn = (long*) r->ordsgn;
-#if 1 /* two variants*/ 
+#if 1 /* two variants*/
   if (_v1 > _v2)
   {
     return _ordsgn[_i];
@@ -1662,12 +1707,27 @@ BOOLEAN p_ComparePolys(poly p1,poly p2, const ring r);
 static inline int p_Cmp(poly p1, poly p2, ring r)
 {
   if (p2==NULL)
+  {
+    if (p1==NULL) return 0;
     return 1;
+  }
   if (p1==NULL)
     return -1;
   return p_LmCmp(p1,p2,r);
 }
 
+static inline int p_CmpPolys(poly p1, poly p2, ring r)
+{
+  if (p2==NULL)
+  {
+    if (p1==NULL) return 0;
+    return 1;
+  }
+  if (p1==NULL)
+    return -1;
+  return p_ComparePolys(p1,p2,r);
+}
+
 
 /***************************************************************
  *
@@ -1913,7 +1973,7 @@ static inline BOOLEAN p_LmShortDivisibleBy(poly a, unsigned long sev_a, const ri
  ***************************************************************/
 
 
-// like the respective p_LmIs* routines, except that p might be empty
+/// like the respective p_LmIs* routines, except that p might be empty
 static inline BOOLEAN p_IsConstantComp(const poly p, const ring r)
 {
   if (p == NULL) return TRUE;
@@ -1923,13 +1983,13 @@ static inline BOOLEAN p_IsConstantComp(const poly p, const ring r)
 static inline BOOLEAN p_IsConstant(const poly p, const ring r)
 {
   if (p == NULL) return TRUE;
-  p_Test(p, r);
   return (pNext(p)==NULL) && p_LmIsConstant(p, r);
 }
 
 /// either poly(1)  or gen(k)?!
 static inline BOOLEAN p_IsOne(const poly p, const ring R)
 {
+  if (p == NULL) return FALSE; /* TODO check if 0 == 1 */
   p_Test(p, R);
   return (p_IsConstant(p, R) && n_IsOne(p_GetCoeff(p, R), R->cf));
 }
@@ -1976,10 +2036,12 @@ static inline BOOLEAN p_LmExpVectorAddIsOk(const poly p1, const poly p2,
 }
 void      p_Split(poly p, poly * r);   /*p => IN(p), r => REST(p) */
 BOOLEAN p_HasNotCF(poly p1, poly p2, const ring r);
+BOOLEAN p_HasNotCFRing(poly p1, poly p2, const ring r);
 poly      p_mInit(const char *s, BOOLEAN &ok, const ring r); /* monom s -> poly, interpreter */
 const char *    p_Read(const char *s, poly &p,const ring r); /* monom -> poly */
 poly      p_MDivide(poly a, poly b, const ring r);
 poly      p_DivideM(poly a, poly b, const ring r);
+poly      pp_DivideM(poly a, poly b, const ring r);
 poly      p_Div_nn(poly p, const number n, const ring r);
 
 // returns the LCM of the head terms of a and b in *m, does not p_Setm
@@ -2046,8 +2108,8 @@ void p_SetModDeg(intvec *w, ring r);
 /*------------ Jet ----------------------------------*/
 poly pp_Jet(poly p, int m, const ring R);
 poly p_Jet(poly p, int m,const ring R);
-poly pp_JetW(poly p, int m, short *w, const ring R);
-poly p_JetW(poly p, int m, short *w, const ring R);
+poly pp_JetW(poly p, int m, int *w, const ring R);
+poly p_JetW(poly p, int m, int *w, const ring R);
 
 poly n_PermNumber(const number z, const int *par_perm, const int OldPar, const ring src, const ring dst);
 
@@ -2075,5 +2137,9 @@ poly p_GcdMon(poly f, poly g, const ring r);
 
 /// divide polynomial by monomial
 poly p_Div_mm(poly p, const poly m, const ring r);
+
+
+/// max exponent of variable x_i in p
+int p_MaxExpPerVar(poly p, int i, const ring r);
 #endif // P_POLYS_H
 
