@@ -8,24 +8,17 @@
 */
 
 /* includes */
-#include "omalloc/omalloc.h"
 #include "misc/auxiliary.h"
 #include "coeffs/coeffs.h"
 #include "misc/intvec.h"
 #include "misc/int64vec.h"
+#include "coeffs/coeffs.h" // ring,number
 #include "polys/monomials/monomials.h"
 //#include "polys/monomials/polys-impl.h"
 //
 
 /* forward declaration of types */
 class idrec; typedef idrec *   idhdl; // _only_ for idhdl ip_sring::idroot
-//struct  spolyrec;
-//typedef struct spolyrec    polyrec;
-//typedef struct spolyrec *         poly;
-//typedef struct spolyrec const *   const_poly;
-struct ip_sring;
-typedef struct ip_sring *         ring;
-typedef struct ip_sring const *   const_ring;
 struct p_Procs_s;
 typedef struct p_Procs_s p_Procs_s;
 class kBucket;
@@ -291,7 +284,7 @@ struct ip_sring
 
   omBin    PolyBin; /* Bin from where monoms are allocated */
   intvec * pModW;   /* std: module weights */
-  poly      ppNoether; /*  variables, set by procedures from hecke/kstd1:
+  poly     ppNoether; /*  variables, set by procedures from hecke/kstd1:
                             the highest monomial below pHEdge */
   void * ext_ref;   /* libsing GAP object */
 // #ifdef HAVE_RINGS
@@ -318,6 +311,7 @@ struct ip_sring
 
 #ifdef HAVE_SHIFTBBA
   short          isLPring; /* 0 for non-letterplace rings, otherwise the number of LP blocks, at least 1, known also as lV */
+  short          LPncGenCount;
 #endif
 
   BOOLEAN   VectorOut;
@@ -352,8 +346,10 @@ struct ip_sring
      VarL_Offset[i] gets i-th long var in exp vector */
   int*      VarL_Offset;
 
-  /* mask for getting single exponents */
+  /* mask for getting single exponents, also maxExp */
   unsigned long bitmask;
+  /* wanted maxExp */
+  unsigned long wanted_maxExp;
   /* mask used for divisiblity tests */
   unsigned long divmask; // rComplete
 
@@ -403,15 +399,31 @@ ring   rEnvelope(ring r);
 /// we must always have this test!
 static inline BOOLEAN rIsPluralRing(const ring r)
 {
-  assume(r != NULL); assume(r->cf != NULL);
+  assume(r != NULL);
 #ifdef HAVE_PLURAL
   nc_struct *n;
-  return (r != NULL) && ((n=r->GetNC()) != NULL) /*&& (n->type != nc_error)*/;
+  return ((n=r->GetNC()) != NULL) /*&& (n->type != nc_error)*/;
 #else
   return FALSE;
 #endif
 }
 
+static inline BOOLEAN rIsLPRing(const ring r)
+{
+  assume(r != NULL);
+#ifdef HAVE_SHIFTBBA
+  return (r->isLPring!=0);
+#else
+  return FALSE;
+#endif
+}
+
+static inline BOOLEAN rIsNCRing(const ring r)
+{
+  assume(r != NULL);
+  return rIsPluralRing(r) || rIsLPRing(r);
+}
+
 static inline BOOLEAN rIsRatGRing(const ring r)
 {
   assume(r != NULL);
@@ -458,24 +470,20 @@ BOOLEAN rSamePolyRep(ring r1, ring r2);
 
 void   rUnComplete(ring r);
 
-BOOLEAN rRing_is_Homog(ring r);
-BOOLEAN rRing_has_CompLastBlock(ring r);
+BOOLEAN rRing_is_Homog(const ring r);
+BOOLEAN rRing_has_CompLastBlock(const ring r);
+BOOLEAN rRing_ord_pure_dp(const ring r);
+BOOLEAN rRing_ord_pure_Dp(const ring r);
+BOOLEAN rRing_ord_pure_lp(const ring r);
 
 #ifdef HAVE_RINGS
 static inline BOOLEAN rField_is_Ring_2toM(const ring r)
 { assume(r != NULL); assume(r->cf != NULL); return ( nCoeff_is_Ring_2toM(r->cf) ); }
 
-static inline BOOLEAN rField_is_Ring_ModN(const ring r)
-{ assume(r != NULL); assume(r->cf != NULL); return ( nCoeff_is_Ring_ModN(r->cf) ); }
-
 static inline BOOLEAN rField_is_Ring_PtoM(const ring r)
 { assume(r != NULL); assume(r->cf != NULL); return ( nCoeff_is_Ring_PtoM(r->cf) ); }
 
-static inline BOOLEAN rField_is_Ring_Z(const ring r)
-{ assume(r != NULL); assume(r->cf != NULL); return ( nCoeff_is_Ring_Z(r->cf) ); }
-
-static inline BOOLEAN rField_is_Ring(const ring r)
-{ assume(r != NULL); assume(r->cf != NULL); return nCoeff_is_Ring(r->cf); }
+#define rField_is_Ring(R) nCoeff_is_Ring((R)->cf)
 
 static inline BOOLEAN rField_is_Domain(const ring r)
 { assume(r != NULL); assume(r->cf != NULL); return nCoeff_is_Domain(r->cf); }
@@ -485,9 +493,7 @@ static inline BOOLEAN rField_has_Units(const ring r)
 #else
 #define rField_is_Ring(A) (0)
 #define rField_is_Ring_2toM(A) (0)
-#define rField_is_Ring_ModN(A) (0)
 #define rField_is_Ring_PtoM(A) (0)
-#define rField_is_Ring_Z(A) (0)
 #define rField_is_Domain(A) (1)
 #define rField_has_Units(A) (1)
 #endif
@@ -504,6 +510,9 @@ static inline BOOLEAN rField_is_Q(const ring r)
 static inline BOOLEAN rField_is_Z(const ring r)
 { assume(r != NULL); assume(r->cf != NULL); return nCoeff_is_Z(r->cf); }
 
+static inline BOOLEAN rField_is_Zn(const ring r)
+{ assume(r != NULL); assume(r->cf != NULL); return nCoeff_is_Zn(r->cf); }
+
 static inline BOOLEAN rField_is_numeric(const ring r) /* R, long R, long C */
 { assume(r != NULL); assume(r->cf != NULL); return nCoeff_is_numeric(r->cf); }
 
@@ -544,7 +553,8 @@ static inline BOOLEAN rField_has_simple_inverse(const ring r)
 static inline BOOLEAN rField_has_simple_Alloc(const ring r)
 { assume(r != NULL); assume(r->cf != NULL); return nCoeff_has_simple_Alloc(r->cf); }
 
-n_coeffType rFieldType(const ring r);
+/// the type of the coefficient filed of r (n_Zp, n_Q, etc)
+static inline n_coeffType rFieldType(const ring r) { return (r->cf->type); }
 
 /// this needs to be called whenever a new ring is created: new fields
 /// in ring are created (like VarOffset), unless they already exist
@@ -733,7 +743,6 @@ ring   rAssure_CompLastBlock(const ring r, BOOLEAN complete = TRUE);
 /// makes sure that c/C ordering is last ordering and SyzIndex is first
 ring   rAssure_SyzComp_CompLastBlock(const ring r);
 ring   rAssure_TDeg(const ring r, int &pos);
-BOOLEAN rHasTDeg(const ring r);
 
 /// return the max-comonent wchich has syzIndex i
 /// Assume: i<= syzIndex_limit
@@ -822,5 +831,15 @@ int rTypeOfMatrixOrder(const intvec *order);
 
 void rDelete(ring r); // To be used instead of rKill!
 
-extern omBin sip_sring_bin;
+EXTERN_VAR omBin sip_sring_bin;
+
+// ring manipulation
+/// K[x],"y" -> K[x,y] resp. K[y,x]
+ring rPlusVar(const ring r, char *v,int left);
+
+/// undo rPlusVar
+ring rMinusVar(const ring r, char *v);
+
+static inline ring rIncRefCnt(ring r) { r->ref++; return r; }
+static inline void rDecRefCnt(ring r) { r->ref--; }
 #endif
