@@ -63,12 +63,12 @@ static ideal BBA_Proc_Dummy (const ideal, const ideal, const intvec *, const int
 
 // the following funtion poiters are quasi-static:
 // they will be set in siInit and never changes afterwards:
-NF_Proc nc_NF=NF_Proc_Dummy;
-BBA_Proc gnc_gr_bba=BBA_Proc_Dummy;
-BBA_Proc gnc_gr_mora=BBA_Proc_Dummy;
-BBA_Proc sca_bba=BBA_Proc_Dummy;
-BBA_Proc sca_mora=BBA_Proc_Dummy;
-BBA_Proc sca_gr_bba=BBA_Proc_Dummy;
+VAR NF_Proc nc_NF=NF_Proc_Dummy;
+VAR BBA_Proc gnc_gr_bba=BBA_Proc_Dummy;
+VAR BBA_Proc gnc_gr_mora=BBA_Proc_Dummy;
+VAR BBA_Proc sca_bba=BBA_Proc_Dummy;
+VAR BBA_Proc sca_mora=BBA_Proc_Dummy;
+VAR BBA_Proc sca_gr_bba=BBA_Proc_Dummy;
 
 /* copy : */
 poly nc_p_CopyGet(poly a, const ring r);
@@ -77,7 +77,7 @@ poly nc_p_CopyPut(poly a, const ring r);
 poly nc_p_Bracket_qq(poly p, const poly q, const ring r);
 
 // only SCA can be used by default, formulas are off by default
-int  iNCExtensions = SCAMASK | NOFORMULAMASK;
+VAR int  iNCExtensions = SCAMASK | NOFORMULAMASK;
 
 int& getNCExtensions()
 {
@@ -108,8 +108,8 @@ bool ncExtensions(int iMask) //  = 0x0FFFF
 // polynomial multiplication functions for p_Procs :
 poly gnc_pp_Mult_mm(const poly p, const poly m, const ring r, poly &last);
 poly gnc_p_Mult_mm(poly p, const poly m, const ring r);
-poly gnc_mm_Mult_p(const poly m, poly p, const ring r);
-poly gnc_mm_Mult_pp(const poly m, const poly p, const ring r);
+poly gnc_p_mm_Mult(poly m, const poly p, const ring r);
+poly gnc_pp_mm_Mult(const poly p, const poly m, const ring r);
 
 
 /* syzygies : */
@@ -214,7 +214,7 @@ poly _gnc_p_Mult_q(poly p, poly q, const int copy, const ring r) // not used any
 // return pPolyP * pPolyQ; destroy or reuse pPolyP and pPolyQ
 poly _nc_p_Mult_q(poly pPolyP, poly pPolyQ, const ring rRing)
 {
-  assume( rIsPluralRing(rRing) );
+  assume( rIsNCRing(rRing) );
 #ifdef PDEBUG
   p_Test(pPolyP, rRing);
   p_Test(pPolyQ, rRing);
@@ -253,7 +253,7 @@ poly _nc_p_Mult_q(poly pPolyP, poly pPolyQ, const ring rRing)
 // return pPolyP * pPolyQ; preserve pPolyP and pPolyQ
 poly _nc_pp_Mult_qq(const poly pPolyP, const poly pPolyQ, const ring rRing)
 {
-  assume( rIsPluralRing(rRing) );
+  assume( rIsNCRing(rRing) );
 #ifdef PDEBUG
   p_Test(pPolyP, rRing);
   p_Test(pPolyQ, rRing);
@@ -357,7 +357,7 @@ poly gnc_p_Mult_mm_Common(poly p, const poly m, int side, const ring r)
 #ifdef PDEBUG
         const char* s;
         if (side==1) s="gnc_p_Mult_mm";
-        else s="gnc_mm_Mult_p";
+        else s="gnc_p_mm_Mult";
         Print("%s: exponent mismatch %d and %d\n",s,expP,expM);
 #endif
         expOut=0;
@@ -399,12 +399,13 @@ poly gnc_p_Mult_mm(poly p, const poly m, const ring r)
   return( gnc_p_Mult_mm_Common(p, m, 1, r) );
 }
 
-poly gnc_mm_Mult_p(const poly m, poly p, const ring r)
+/* m * p */
+poly gnc_p_mm_Mult(poly p, const poly m, const ring r)
 {
   return( gnc_p_Mult_mm_Common(p, m, 0, r) );
 }
 
-poly gnc_mm_Mult_pp(const poly m, const poly p, const ring r)
+poly gnc_pp_mm_Mult(const poly p, const poly m, const ring r)
 {
   return( gnc_p_Mult_mm_Common(p_Copy(p,r), m, 0, r) );
 }
@@ -662,7 +663,7 @@ poly gnc_mm_Mult_nn(int *F0, int *G0, const ring r)
 #ifdef PDEBUG
     p_Test(Rout,r);
 #endif
-    out=gnc_mm_Mult_p(Rout,out,r); /* getting the final result */
+    out=gnc_p_mm_Mult(out,Rout,r); /* getting the final result */
     freeT(Prv,rN);
     p_Delete(&Rout,r);
   }
@@ -790,7 +791,7 @@ poly gnc_mm_Mult_uu(int *F,int jG,int bG, const ring r)
        freeT(Prv,rN);
        Prv = NULL;
 
-       out=gnc_mm_Mult_p(Rout,out,r); /* getting the final result */
+       out=gnc_p_mm_Mult(out,Rout,r); /* getting the final result */
     }
 
     freeT(lF,rN);
@@ -860,7 +861,7 @@ poly gnc_mm_Mult_uu(int *F,int jG,int bG, const ring r)
        if (t!=first)   /* typical expr */
        {
          w=gnc_p_Mult_mm(D,Pn,r);
-         Rout=gnc_mm_Mult_p(Pp,w,r);
+         Rout=gnc_p_mm_Mult(w,Pp,r);
          w=NULL;
        }
        else                   /* last step */
@@ -877,7 +878,7 @@ poly gnc_mm_Mult_uu(int *F,int jG,int bG, const ring r)
      }
      else                     /* first step */
      {
-       Rout=gnc_mm_Mult_p(Pp,D,r);
+       Rout=gnc_p_mm_Mult(D,Pp,r);
      }
 #ifdef PDEBUG
      p_Test(Pp,r);
@@ -921,7 +922,7 @@ poly gnc_mm_Mult_uu(int *F,int jG,int bG, const ring r)
     p_SetExpV(Rout,Prv,r);
     p_Setm(Rout,r);
     freeT(Prv, rN);
-    out=gnc_mm_Mult_p(Rout,out,r); /* getting the final result */
+    out=gnc_p_mm_Mult(out,Rout,r); /* getting the final result */
     p_Delete(&Rout,r);
   }
 
@@ -954,7 +955,7 @@ poly gnc_uu_Mult_ww_vert (int i, int a, int j, int b, const ring r)
      {
        t = nc_p_CopyGet(MATELEM(cMT,k-1,1),r);
        //        t=p_Copy(MATELEM(cMT,k-1,1),r);
-       t = gnc_mm_Mult_p(y,t,r);
+       t = gnc_p_mm_Mult(t,y,r);
        cMT=r->GetNC()->MT[cMTindex]; // since multiplication can change the MT table...
        assume( t != NULL );
 #ifdef PDEBUG
@@ -1194,7 +1195,7 @@ poly gnc_uu_Mult_ww_horvert (int i, int a, int j, int b, const ring r)
       if (t==NULL)   /* remove after debug */
       {
         t = p_Copy(MATELEM(cMT,m-1,1),r);
-        t = gnc_mm_Mult_p(y,t,r);
+        t = gnc_p_mm_Mult(t,y,r);
         MATELEM(cMT,m,1) = t;
         /*        omCheckAddr(cMT->m); */
       }
@@ -1267,7 +1268,7 @@ poly gnc_uu_Mult_ww_horvert (int i, int a, int j, int b, const ring r)
       if (t==NULL)   /* remove after debug */
       {
         t = p_Copy(MATELEM(cMT,k-1,b),r);
-        t = gnc_mm_Mult_p(y,t,r);
+        t = gnc_p_mm_Mult(t,y,r);
         MATELEM(cMT,k,b) = t;
         /*        omCheckAddr(cMT->m); */
       }
@@ -1292,7 +1293,7 @@ poly gnc_uu_Mult_ww_horvert (int i, int a, int j, int b, const ring r)
         if (t==NULL)   /* remove after debug */
         {
           t = p_Copy(MATELEM(cMT,m-1,1),r);
-          t = gnc_mm_Mult_p(y,t,r);
+          t = gnc_p_mm_Mult(t,y,r);
           MATELEM(cMT,m,1) = t;
           /*        omCheckAddr(cMT->m); */
         }
@@ -2346,7 +2347,7 @@ poly nc_mm_Bracket_nn(poly m1, poly m2, const ring r)
             p_Setm(prefix, r);
             p_SetExpV(suffix,aSUFFIX, r);
             p_Setm(suffix, r);
-            if (!p_LmIsConstant(prefix, r)) bres = gnc_mm_Mult_p(prefix, bres, r);
+            if (!p_LmIsConstant(prefix, r)) bres = gnc_p_mm_Mult(bres, prefix, r);
             if (!p_LmIsConstant(suffix, r)) bres = gnc_p_Mult_mm(bres, suffix, r);
             ares=p_Add_q(ares, bres, r);
             /* What to give free? */
@@ -2372,7 +2373,7 @@ poly nc_mm_Bracket_nn(poly m1, poly m2, const ring r)
         p_SetExpV(suffix,aSUFFIX, r);
         p_Setm(suffix, r);
         bres=ares;
-        if (!p_LmIsConstant(prefix, r)) bres = gnc_mm_Mult_p(prefix, bres, r);
+        if (!p_LmIsConstant(prefix, r)) bres = gnc_p_mm_Mult(bres, prefix, r);
         if (!p_LmIsConstant(suffix, r)) bres = gnc_p_Mult_mm(bres, suffix, r);
         res=p_Add_q(res, bres, r);
         p_Delete(&prefix, r);
@@ -2715,11 +2716,6 @@ BOOLEAN nc_CallPlural(matrix CCC, matrix DDD,
   assume( (CCC != NULL) != (CCN != NULL) ); // exactly one data about coeffs (C).
   assume( !((DDD != NULL) && (DDN != NULL)) ); // at most one data about tails (D).
 
-//  ring save = currRing;
-//  if( save != curr )
-//    rChangeCurrRing(curr);
-
-
 #if OUTPUT
   if( CCC != NULL )
   {
@@ -2744,9 +2740,6 @@ BOOLEAN nc_CallPlural(matrix CCC, matrix DDD,
   if( (!bBeQuiet) && (r->GetNC() != NULL) )
     WarnS("going to redefine the algebra structure");
 
-//  if( currRing != r )
-//    rChangeCurrRing(r);
-
   matrix CC = NULL;
   poly CN = NULL;
   matrix C; bool bCnew = false;
@@ -2777,9 +2770,6 @@ BOOLEAN nc_CallPlural(matrix CCC, matrix DDD,
     if ((CCC != NULL) && ( (MATCOLS(CCC)!=r->N) || (MATROWS(CCC)!=r->N) ))
     {
       Werror("Square %d x %d  matrix expected", r->N, r->N);
-
-//      if( currRing != save )
-//        rChangeCurrRing(save);
       return TRUE;
     }
   }
@@ -2796,9 +2786,6 @@ BOOLEAN nc_CallPlural(matrix CCC, matrix DDD,
     if ((DDD != NULL) && ( (MATCOLS(DDD)!=r->N) || (MATROWS(DDD)!=r->N) ))
     {
       Werror("Square %d x %d  matrix expected",r->N,r->N);
-
-//      if( currRing != save )
-//        rChangeCurrRing(save);
       return TRUE;
     }
   }
@@ -2821,9 +2808,6 @@ BOOLEAN nc_CallPlural(matrix CCC, matrix DDD,
     if (n_IsZero(nN, curr->cf))
     {
       WerrorS("Incorrect input : zero coefficients are not allowed");
-
-//      if( currRing != save )
-//        rChangeCurrRing(save);
       return TRUE;
     }
 
@@ -2845,8 +2829,8 @@ BOOLEAN nc_CallPlural(matrix CCC, matrix DDD,
     id_Test((ideal)C, r);
 #endif
 
-  } else
-  if ( (CN == NULL) && (CC != NULL) ) /* copy matrix C */
+  }
+  else if ( (CN == NULL) && (CC != NULL) ) /* copy matrix C */
   {
     /* analyze C */
 
@@ -2858,7 +2842,7 @@ BOOLEAN nc_CallPlural(matrix CCC, matrix DDD,
       {
         if (!pN_set) n_Delete(&pN,curr->cf); // free initial nInit(0)
         pN = p_GetCoeff(MATELEM(CC,1,2), curr);
-	pN_set=TRUE;
+        pN_set=TRUE;
       }
 
     tmpIsSkewConstant = true;
@@ -2879,14 +2863,10 @@ BOOLEAN nc_CallPlural(matrix CCC, matrix DDD,
           qN = p_GetCoeff(MATELEM(CC,i,j),curr);
         }
 
-
         if ( qN == NULL )   /* check the consistency: Cij!=0 */
         // find also illegal pN
         {
           WerrorS("Incorrect input : matrix of coefficients contains zeros in the upper triangle");
-
-//        if( currRing != save )
-//            rChangeCurrRing(save);
           return TRUE;
         }
 
@@ -2902,6 +2882,7 @@ BOOLEAN nc_CallPlural(matrix CCC, matrix DDD,
       bCnew = true;
     }
     else
+
       C = CC;
 
     IsSkewConstant = tmpIsSkewConstant;
@@ -2986,9 +2967,6 @@ BOOLEAN nc_CallPlural(matrix CCC, matrix DDD,
     if( bDnew ) mp_Delete( &D, r );
 
     WerrorS("Matrix of polynomials violates the ordering condition");
-
-//    if( currRing != save )
-//      rChangeCurrRing(save);
     return TRUE;
   }
 
@@ -3017,9 +2995,6 @@ BOOLEAN nc_CallPlural(matrix CCC, matrix DDD,
 
   r->ext_ref=NULL;
 
-//  if( currRing != save )
-//    rChangeCurrRing(save);
-
   return gnc_InitMultiplication(r, bSetupQuotient);
 }
 
@@ -3145,8 +3120,8 @@ void gnc_p_ProcsSet(ring rGR, p_Procs_s* p_Procs)
   p_Procs->p_Minus_mm_Mult_qq = rGR->p_Procs->p_Minus_mm_Mult_qq = nc_p_Minus_mm_Mult_qq;
 
   // non-commutaitve multiplication by monomial from the left
-  rGR->GetNC()->p_Procs.mm_Mult_p   = gnc_mm_Mult_p;
-  rGR->GetNC()->p_Procs.mm_Mult_pp  = gnc_mm_Mult_pp;
+  p_Procs->p_mm_Mult   = gnc_p_mm_Mult;
+  p_Procs->pp_mm_Mult  = gnc_pp_mm_Mult;
 
 #if 0
   // Previous Plural's implementation...
@@ -3160,7 +3135,7 @@ void gnc_p_ProcsSet(ring rGR, p_Procs_s* p_Procs)
   rGR->GetNC()->p_Procs.SPoly       = gnc_CreateSpolyNew;
   rGR->GetNC()->p_Procs.ReduceSPoly = gnc_ReduceSpolyNew;
 
-  rGR->GetNC()->p_Procs.BucketPolyRed  = gnc_kBucketPolyRedNew;
+  rGR->GetNC()->p_Procs.BucketPolyRed_NF= gnc_kBucketPolyRedNew;
   rGR->GetNC()->p_Procs.BucketPolyRed_Z= gnc_kBucketPolyRed_ZNew;
 #endif
 
@@ -3385,7 +3360,7 @@ poly pOppose(ring Rop, poly p, const ring dst)
   // coinside???
 
   int *perm=(int *)omAlloc0((Rop->N+1)*sizeof(int));
-  if (!p_IsConstantPoly(p, Rop))
+  if (!p_IsConstant(p, Rop))
   {
     /* we know perm exactly */
     int i;
