@@ -27,8 +27,6 @@
 // #undef PDEBUG
 // #define PDEBUG 2
 
-#include "omalloc/omalloc.h"
-
 #include "polys/monomials/ring.h"
 #include "polys/monomials/p_polys.h"
 
@@ -40,7 +38,7 @@
  *
  ***************************************************************/
 // avoid recursive calls
-static BOOLEAN d_poly_error_reporting = FALSE;
+STATIC_VAR BOOLEAN d_poly_error_reporting = FALSE;
 BOOLEAN dPolyReportError(poly p, ring r, const char* fmt, ...)
 {
   if (d_poly_error_reporting) return FALSE;
@@ -51,7 +49,9 @@ BOOLEAN dPolyReportError(poly p, ring r, const char* fmt, ...)
   fprintf(stderr, "\n// ***dPolyReportError: ");
   vfprintf(stderr, fmt, ap);
   fprintf(stderr, "\n occurred at\n");
+  #ifdef HAVE_OMALLOC
   omPrintCurrentBackTraceMax(stderr, 8);
+  #endif
   if (p != NULL)
   {
     fprintf(stderr, " occurred for poly: ");
@@ -359,9 +359,9 @@ BOOLEAN _pp_Test(poly p, ring lmRing, ring tailRing, int level)
 #endif // PDEBUG
 
 #if defined(PDEBUG) || defined(PDIV_DEBUG)
-static unsigned long pDivisibleBy_number = 1;
-static unsigned long pDivisibleBy_FALSE = 1;
-static unsigned long pDivisibleBy_ShortFalse = 1;
+STATIC_VAR unsigned long pDivisibleBy_number = 1;
+STATIC_VAR unsigned long pDivisibleBy_FALSE = 1;
+STATIC_VAR unsigned long pDivisibleBy_ShortFalse = 1;
 
 BOOLEAN pDebugLmShortDivisibleBy(poly p1, unsigned long sev_1, ring r_1,
                                poly p2, unsigned long not_sev_2, ring r_2)
