@@ -12,8 +12,6 @@
 
 #include "misc/auxiliary.h"
 
-#include "omalloc/omalloc.h"
-
 #include "misc/options.h"
 #include "misc/intvec.h"
 
@@ -26,11 +24,11 @@
 
 #include "simpleideals.h"
 
-omBin sip_sideal_bin = omGetSpecBin(sizeof(sip_sideal));
+VAR omBin sip_sideal_bin = omGetSpecBin(sizeof(sip_sideal));
 
-static poly * idpower;
+STATIC_VAR poly * idpower;
 /*collects the monomials in makemonoms, must be allocated befor*/
-static int idpowerpoint;
+STATIC_VAR int idpowerpoint;
 /*index of the actual monomial in idpower*/
 
 /// initialise an ideal / module
@@ -99,8 +97,19 @@ int id_PosConstant(ideal id, const ring r)
 /// initialise the maximal ideal (at 0)
 ideal id_MaxIdeal (const ring r)
 {
-  ideal hh = idInit(rVar(r), 1);
-  for (int l=rVar(r)-1; l>=0; l--)
+  int nvars;
+#ifdef HAVE_SHIFTBBA
+  if (r->isLPring)
+  {
+    nvars = r->isLPring;
+  }
+  else
+#endif
+  {
+    nvars = rVar(r);
+  }
+  ideal hh = idInit(nvars, 1);
+  for (int l=nvars-1; l>=0; l--)
   {
     hh->m[l] = p_One(r);
     p_SetExp(hh->m[l],l+1,1,r);
@@ -118,7 +127,7 @@ void id_Delete (ideal * h, ring r)
 
   id_Test(*h, r);
 
-  const int elems = (*h)->nrows * (*h)->ncols;
+  const long elems = (long)(*h)->nrows * (long)(*h)->ncols;
 
   if ( elems > 0 )
   {
@@ -126,7 +135,7 @@ void id_Delete (ideal * h, ring r)
 
     if (r!=NULL)
     {
-      int j = elems;
+      long j = elems;
       do
       {
         j--;
@@ -425,7 +434,7 @@ void id_DBTest(ideal h1, int level, const char *f,const int l, const ring r, con
 
     assume( h1->rank >= 0 );
 
-    const int n = (h1->ncols * h1->nrows);
+    const long n = ((long)h1->ncols * (long)h1->nrows);
 
     assume( !( n > 0 && h1->m == NULL) );
 
@@ -435,7 +444,7 @@ void id_DBTest(ideal h1, int level, const char *f,const int l, const ring r, con
     long new_rk = 0; // inlining id_RankFreeModule(h1, r, tailRing);
 
     /* to be able to test matrices: */
-    for (int i=n - 1; i >= 0; i--)
+    for (long i=n - 1; i >= 0; i--)
     {
       _pp_Test(h1->m[i], r, tailRing, level);
       const long k = p_MaxComp(h1->m[i], r, tailRing);
@@ -912,7 +921,8 @@ int idGetNumberOfChoise(int t, int d, int begin, int end, int * choise)
 */
 int binom (int n,int r)
 {
-  int i,result;
+  int i;
+  int64 result;
 
   if (r==0) return 1;
   if (n-r<r) return binom(n,n-r);
@@ -920,14 +930,14 @@ int binom (int n,int r)
   for (i=2;i<=r;i++)
   {
     result *= n-r+i;
-    if (result<0)
-    {
-      WarnS("overflow in binomials");
-      return 0;
-    }
     result /= i;
   }
-  return result;
+  if (result>MAX_INT_VAL)
+  {
+    WarnS("overflow in binomials");
+    result=0;
+  }
+  return (int)result;
 }
 
 
@@ -935,6 +945,10 @@ int binom (int n,int r)
 ideal id_FreeModule (int i, const ring r)
 {
   assume(i >= 0);
+  if (r->isLPring)
+  {
+    PrintS("In order to address bimodules, the command freeAlgebra should be used.");
+  }
   ideal h = idInit(i, i);
 
   for (int j=0; j<i; j++)
@@ -999,33 +1013,101 @@ static void makemonoms(int vars,int actvar,int deg,int monomdeg, const ring r)
   }
 }
 
+#ifdef HAVE_SHIFTBBA
+/*2
+*computes recursively all letterplace monomials of a certain degree
+*vars is the number of original variables (lV)
+*deg is the degree of the monomials to compute
+*
+*NOTE: We use idpowerpoint as the last index of the previous call
+*/
+static void lpmakemonoms(int vars, int deg, const ring r)
+{
+  assume(deg <= r->N/r->isLPring);
+  if (deg == 0)
+  {
+    idpower[0] = p_One(r);
+    return;
+  }
+  else
+  {
+    lpmakemonoms(vars, deg - 1, r);
+  }
+
+  int size = idpowerpoint + 1;
+  for (int j = 2; j <= vars; j++)
+  {
+    for (int i = 0; i < size; i++)
+    {
+      idpowerpoint = (j-1)*size + i;
+      idpower[idpowerpoint] = p_Copy(idpower[i], r);
+    }
+  }
+  for (int j = 1; j <= vars; j++)
+  {
+    for (int i = 0; i < size; i++)
+    {
+      idpowerpoint = (j-1)*size + i;
+      p_SetExp(idpower[idpowerpoint], ((deg - 1) * r->isLPring) + j, 1, r);
+      p_Setm(idpower[idpowerpoint],r);
+      p_Test(idpower[idpowerpoint],r);
+    }
+  }
+}
+#endif
+
 /*2
 *returns the deg-th power of the maximal ideal of 0
 */
 ideal id_MaxIdeal(int deg, const ring r)
 {
-  if (deg < 0)
-  {
-    WarnS("maxideal: power must be non-negative");
-  }
   if (deg < 1)
   {
     ideal I=idInit(1,1);
     I->m[0]=p_One(r);
     return I;
   }
-  if (deg == 1)
+  if (deg == 1
+#ifdef HAVE_SHIFTBBA
+      && !r->isLPring
+#endif
+     )
   {
     return id_MaxIdeal(r);
   }
 
-  int vars = rVar(r);
-  int i = binom(vars+deg-1,deg);
+  int vars, i;
+#ifdef HAVE_SHIFTBBA
+  if (r->isLPring)
+  {
+    vars = r->isLPring - r->LPncGenCount;
+    i = 1;
+    // i = vars^deg
+    for (int j = 0; j < deg; j++)
+    {
+      i *= vars;
+    }
+  }
+  else
+#endif
+  {
+    vars = rVar(r);
+    i = binom(vars+deg-1,deg);
+  }
   if (i<=0) return idInit(1,1);
   ideal id=idInit(i,1);
   idpower = id->m;
   idpowerpoint = 0;
-  makemonoms(vars,1,deg,0,r);
+#ifdef HAVE_SHIFTBBA
+  if (r->isLPring)
+  {
+    lpmakemonoms(vars, deg, r);
+  }
+  else
+#endif
+  {
+    makemonoms(vars,1,deg,0,r);
+  }
   idpower = NULL;
   idpowerpoint = 0;
   return id;
@@ -1175,14 +1257,14 @@ ideal id_Matrix2Module(matrix mat, const ring R)
 
   for(j=0;j<mc /*MATCOLS(mat)*/;j++) /* j is also index in result->m */
   {
-    for (i=1;i<=mr /*MATROWS(mat)*/;i++)
+    for (i=0;i<mr /*MATROWS(mat)*/;i++)
     {
-      h = MATELEM(mat,i,j+1);
+      h = MATELEM0(mat,i,j);
       if (h!=NULL)
       {
         l=pLength(h);
-        MATELEM(mat,i,j+1)=NULL;
-        p_SetCompP(h,i, R);
+        MATELEM0(mat,i,j)=NULL;
+        p_SetCompP(h,i+1, R);
         sBucket_Merge_p(bucket, h, l);
       }
     }
@@ -1224,19 +1306,19 @@ matrix id_Module2Matrix(ideal mod, const ring R)
         int k,l,o=mod->rank;
         mod->rank=cp;
         matrix d=mpNew(mod->rank,IDELEMS(mod));
-        for (l=1; l<=o; l++)
+        for (l=0; l<o; l++)
         {
-          for (k=1; k<=IDELEMS(mod); k++)
+          for (k=0; k<IDELEMS(mod); k++)
           {
-            MATELEM(d,l,k)=MATELEM(result,l,k);
-            MATELEM(result,l,k)=NULL;
+            MATELEM0(d,l,k)=MATELEM0(result,l,k);
+            MATELEM0(result,l,k)=NULL;
           }
         }
         id_Delete((ideal *)&result,R);
         result=d;
       }
 #endif
-      MATELEM(result,cp,i+1) = p_Add_q(MATELEM(result,cp,i+1),h,R);
+      MATELEM0(result,cp-1,i) = p_Add_q(MATELEM0(result,cp-1,i),h,R);
     }
   }
   // obachman 10/99: added the following line, otherwise memory leack!
@@ -1266,7 +1348,7 @@ matrix id_Module2formatedMatrix(ideal mod,int rows, int cols, const ring R)
       {
         p_SetComp(h,0,R);
         p_SetmComp(h,R);
-        MATELEM(result,cp,i+1) = p_Add_q(MATELEM(result,cp,i+1),h,R);
+        MATELEM0(result,cp-1,i) = p_Add_q(MATELEM0(result,cp-1,i),h,R);
       }
       else
         p_Delete(&h,R);
@@ -1276,6 +1358,39 @@ matrix id_Module2formatedMatrix(ideal mod,int rows, int cols, const ring R)
   return result;
 }
 
+ideal id_ResizeModule(ideal mod,int rows, int cols, const ring R)
+{
+  // columns?
+  if (cols!=IDELEMS(mod))
+  {
+    for(int i=IDELEMS(mod)-1;i>=cols;i--) p_Delete(&mod->m[i],R);
+    pEnlargeSet(&(mod->m),IDELEMS(mod),cols-IDELEMS(mod));
+    IDELEMS(mod)=cols;
+  }
+  // rows?
+  if (rows<mod->rank)
+  {
+    for(int i=IDELEMS(mod)-1;i>=0;i--)
+    {
+      if (mod->m[i]!=NULL)
+      {
+        while((mod->m[i]!=NULL) && (p_GetComp(mod->m[i],R)>rows))
+          mod->m[i]=p_LmDeleteAndNext(mod->m[i],R);
+        poly p=mod->m[i];
+        while(pNext(p)!=NULL)
+        {
+          if (p_GetComp(pNext(p),R)>rows)
+            pNext(p)=p_LmDeleteAndNext(pNext(p),R);
+          else
+            pIter(p);
+        }
+      }
+    }
+  }
+  mod->rank=rows;
+  return mod;
+}
+
 /*2
 * substitute the n-th variable by the monomial e in id
 * destroy id
@@ -1425,7 +1540,7 @@ ideal id_Jet(const ideal i,int d, const ring R)
   r->ncols = i-> ncols;
   //r->rank = i-> rank;
 
-  for(int k=(i->nrows)*(i->ncols)-1;k>=0; k--)
+  for(long k=((long)(i->nrows))*((long)(i->ncols))-1;k>=0; k--)
     r->m[k]=pp_Jet(i->m[k],d,R);
 
   return r;
@@ -1440,13 +1555,13 @@ ideal id_JetW(const ideal i,int d, intvec * iv, const ring R)
   }
   else
   {
-    short *w=iv2array(iv,R);
+    int *w=iv2array(iv,R);
     int k;
     for(k=0; k<IDELEMS(i); k++)
     {
       r->m[k]=pp_JetW(i->m[k],d,w,R);
     }
-    omFreeSize((ADDRESS)w,(rVar(R)+1)*sizeof(short));
+    omFreeSize((ADDRESS)w,(rVar(R)+1)*sizeof(int));
   }
   return r;
 }
@@ -1784,7 +1899,7 @@ ideal id_ChineseRemainder(ideal *xx, number *q, int rl, const ring r)
   number *x=(number *)omAlloc(rl*sizeof(number));
   poly *p=(poly *)omAlloc(rl*sizeof(poly));
   CFArray inv_cache(rl);
-  extern int n_SwitchChinRem; //TEST
+  EXTERN_VAR int n_SwitchChinRem; //TEST
   int save_n_SwitchChinRem=n_SwitchChinRem;
   n_SwitchChinRem=1;
   for(i=cnt-1;i>=0;i--)
