@@ -16,20 +16,7 @@
 #include <sys/param.h>
 
 
-char* feArgv0 = NULL;
-
-#ifdef AIX_4
-#ifndef HAVE_PUTENV
-#define HAVE_PUTENV 1
-#endif
-#endif
-
-#if defined(HPUX_10) || defined(HPUX_9)
-#ifndef HAVE_SETENV
-extern "C" int setenv(const char *name, const char *value, int overwrite);
-#endif
-#endif
-
+VAR char* feArgv0 = NULL;
 
 //char* feResource(const char id, int warn = -1);
 //char* feResource(const char* key, int warn = -1);
@@ -51,7 +38,7 @@ extern "C" int setenv(const char *name, const char *value, int overwrite);
 // feCleanResource makes furthermore  the following transformations (except for URL resources)
 // 1.) '/' characters are replaced by respective directory - separators
 // 2.) ';' characters are replaced by respective path separators
-feResourceConfig_s feResourceConfigs[] =
+VAR feResourceConfig_s feResourceConfigs[] =
 {
   {"SearchPath",    's', feResPath,  NULL,
    "$SINGULARPATH;"
@@ -62,20 +49,26 @@ feResourceConfig_s feResourceConfigs[] =
    "%D/factory;"
    "%r/share/factory;"
    "%b/LIB;"
+   "%b/../LIB;" // not installed, shared is in .libs/Singular
    "%b/../factory;"
+   "%b/../../factory;" // not installed, shared is in .libs/Singular
    // path for dynamic modules, should match ProcDir:
    "%b/MOD;"
+   "%b/../MOD;" // Singular in .libs/Singular
    "%r/lib/singular/MOD;"
    "%r/libexec/singular/MOD;"
    LIB_DIR "/singular/MOD;"
    LIBEXEC_DIR "/singular/MOD;"
-   "%b",
+   "%b;"
+   "%b/..", // Singular in .libs/Singular
    (char *)""},
   {"Singular",  'S',    feResBinary,"SINGULAR_EXECUTABLE",  "%d/Singular",          (char *)""},
   {"BinDir",    'b',    feResDir,   "SINGULAR_BIN_DIR",     "",                  (char *)""},
   // should be changed to %b/../lib/singular/pProcs/:
   {"ProcDir",   'P',    feResPath,  "SINGULAR_PROCS_DIR",
      "%b/MOD;"
+     "%b/../MOD;" // Singular in .libs/Singular
+     "%b/..;" // Singular in .libs/Singular, programs in .
      "%r/lib/singular/MOD;"
      "%r/libexec/singular/MOD;"
      LIB_DIR "/singular/MOD;"   /*debian: -> /usr/lib/singular/MOD */
@@ -83,10 +76,10 @@ feResourceConfig_s feResourceConfigs[] =
   {"RootDir",   'r',    feResDir,   "SINGULAR_ROOT_DIR",    "%b/..",                (char *)""},
   {"DataDir",   'D',    feResDir,   "SINGULAR_DATA_DIR",    "%b/../share/",          (char *)""},
   {"DefaultDir",'d',    feResDir,   "SINGULAR_DEFAULT_DIR",  SINGULAR_DEFAULT_DIR,  (char *)""},
-  {"InfoFile",  'i',    feResFile,  "SINGULAR_INFO_FILE",   "%D/info/singular.hlp", (char *)""},
+  {"InfoFile",  'i',    feResFile,  "SINGULAR_INFO_FILE",   "%D/info/singular.info", (char *)""},
   {"IdxFile",   'x',    feResFile,  "SINGULAR_IDX_FILE",    "%D/singular/singular.idx",  (char *)""},
   {"HtmlDir",   'h',    feResDir,   "SINGULAR_HTML_DIR",    "%D/singular/html",              (char *)""},
-  {"ManualUrl", 'u',    feResUrl,   "SINGULAR_URL",         "http://www.singular.uni-kl.de/Manual/",    (char *)""},
+  {"ManualUrl", 'u',    feResUrl,   "SINGULAR_URL",         "https://www.singular.uni-kl.de/Manual/",    (char *)""},
   {"ExDir",     'm',    feResDir,   "SINGULAR_EXAMPLES_DIR","%r/examples",          (char *)""},
   {"Path",      'p',    feResPath,  NULL,                   "%b;%P;$PATH",             (char *)""},
 
@@ -187,21 +180,6 @@ void feInitResources(const char* argv0)
   // don't complain about stuff when initializing SingularPath
   feResource('s',0);
   feResource('P');
-
-#if defined(HAVE_SETENV) || defined(HAVE_PUTENV)
-  char* path = feResource('p');
-#ifdef RESOURCE_DEBUG
-  printf("feInitResources(argv0): setting path with '%s'\n", path);
-#endif
-#ifdef HAVE_PUTENV
-  if (path != NULL) { char *s=(char *)malloc(strlen(path)+6);
-                      sprintf(s,"PATH=%s",path);
-                      putenv(s);
-                    }
-#else
-  if (path != NULL) setenv("PATH", path, 1);
-#endif
-#endif
 }
 
 void feReInitResources()
